Type: Package
Package: rpf
Title: Response Probability Functions
Version: 1.0.15
Date: 2025-05-03
Authors@R: 
    c(person(given = "Joshua",
             family = "Pritikin",
             role = c("cre", "aut"),
             email = "jpritikin@pobox.com"),
      person(given = "Jonathan",
             family = "Weeks",
             role = "ctb",
             email = "weeksjp@gmail.com"),
      person(given = "Li",
             family = "Cai",
             role = "ctb"),
      person(given = "Carrie",
             family = "Houts",
             role = "ctb"),
      person(given = "Phil",
             family = "Chalmers",
             role = "ctb",
             email = "rphilip.chalmers@gmail.com"),
      person(given = c("Michael", "D."),
             family = "Hunter",
             role = "ctb"),
      person(given = c("Carl", "F."),
             family = "Falk",
             role = "ctb",
             email = "falkcarl@msu.edu"))
Maintainer: Joshua Pritikin <jpritikin@pobox.com>
Description: Factor out logic
    and math common to Item Factor Analysis fitting, diagnostics, and
    analysis. It is envisioned as core support code suitable for more
    specialized IRT packages to build upon. Complete access to optimized C
    functions are made available with R_RegisterCCallable().
    This software is described in Pritikin & Falk (2020) <doi:10.1177/0146621620929431>.
License: GPL (>= 3)
URL: https://github.com/jpritikin/rpf
Depends: methods, parallel, R (>= 2.14.0)
Imports: Rcpp (>= 1.0.2), mvtnorm, lifecycle
Suggests: testthat, roxygen2, ggplot2, reshape2, gridExtra, numDeriv,
        knitr, mirt, markdown
LinkingTo: Rcpp, RcppEigen
VignetteBuilder: knitr
RdMacros: lifecycle
Encoding: UTF-8
LazyData: yes
LazyDataCompression: xz
RoxygenNote: 7.3.2
SystemRequirements: GNU make
Collate: 'init.R' 'classes.R' 'fit.R' 'drm.R' 'nrm.R' 'mcm.R' 'grm.R'
        'LSAT.R' 'sample.R' 'dataframe.R' 'diagnose.R' 'science.R'
        'kct.R' 'openmx.R' 'flexmirt.R' 'util.R' 'lmp.R' 'grmp.R'
        'gpcmp.R' 'RcppExports.R'
NeedsCompilation: yes
Packaged: 2025-05-04 06:15:04 UTC; joshua
Author: Joshua Pritikin [cre, aut],
  Jonathan Weeks [ctb],
  Li Cai [ctb],
  Carrie Houts [ctb],
  Phil Chalmers [ctb],
  Michael D. Hunter [ctb],
  Carl F. Falk [ctb]
Repository: CRAN
Date/Publication: 2025-05-04 10:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 23:21:08 UTC; unix
Archs: rpf.so.dSYM

import(methods)
import(parallel)
importFrom("stats", "integrate", "na.omit", "pchisq", "rbeta",
	   "rlnorm", "rnorm", "runif", "uniroot")
importFrom("utils", "count.fields", "read.delim")
importFrom(Rcpp, evalCpp)
importFrom(lifecycle, deprecate_soft)
exportClassPattern("^rpf.+")
exportMethods(rpf.logprob)
exportMethods(rpf.prob)
exportMethods(rpf.rparam)
exportPattern("^rpf.+")
export(read.flexmirt)
export(write.flexmirt)
export(chen.thissen.1997)  # old name
export(ChenThissen1997)
export(ordinal.gamma)
export(ptw2011.gof.test)
export(logit)
export(sumScoreEAP)
export(crosstabTest)
export(SitemFit)
export(SitemFit1)
export(observedSumScore)
export(itemOutcomeBySumScore)
export(orderCompletely)
export(tabulateRows)
export(expandDataFrame)
export(compressDataFrame)
export(multinomialFit)
export(EAPscores)
export(as.IFAgroup)
export(stripData)
export(sumScoreEAPTest)
export(bestToOmit)
export(omitItems)
export(omitMostMissing)
export(toFactorLoading)
export(fromFactorLoading)
export(toFactorThreshold)
export(fromFactorThreshold)
export(collapseCategoricalCells)
S3method(print, summary.ChenThissen1997)
S3method("+", summary.ChenThissen1997)
S3method(print, summary.SitemFit)
S3method("+", summary.SitemFit)
S3method(print, summary.sumScoreEAPTest)
S3method("+", summary.sumScoreEAPTest)
S3method(print, summary.itemOutcomeBySumScore)
S3method(print, summary.observedSumScore)
S3method(print, summary.multinomialFit)
useDynLib(rpf, .registration = TRUE)

Title:       rpf

# Why use RPF?

<!-- badges: start -->
[![Travis build status](https://app.travis-ci.com/jpritikin/rpf.svg?branch=master)](https://app.travis-ci.com/github/jpritikin/rpf)
[![Codecov test coverage](https://codecov.io/gh/jpritikin/rpf/branch/master/graph/badge.svg)](https://app.codecov.io/gh/jpritikin/rpf?branch=master)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/rpf?color=blue)](https://cran.r-project.org/package=rpf)
[![Monthly Downloads](https://cranlogs.r-pkg.org/badges/rpf)](https://cranlogs.r-pkg.org/badges/rpf)
[![Total Downloads](https://cranlogs.r-pkg.org/badges/grand-total/rpf)](https://cranlogs.r-pkg.org/badges/grand-total/rpf)
<!-- badges: end -->

The idea behind RPF is modularity. Most item factor analysis software
is not modular. Modularity facilitates more contributors and cross
pollination between projects.

# Installation

To get the current released version from CRAN:

```R
install.packages("rpf")
```

# Developer notes

There are a number of useful scripts in the `tools` subdir:

* `install` -- Installs the package as quickly as possible. Skips
  building the vignettes and documentation.

* `build` -- Builds a source tarball

* `check` -- Builds a source tarball and checks it

* `rox` -- Re-generates the documentation.

* `test` -- Runs the test suite using the uninstalled tests against the
  installed package.

* `autodep` -- Recalculates the header file dependences

If you're working on the C++ code, you probably want to adjust the
settings in `src/Makevars`.

## ----cache=FALSE, include=FALSE-----------------------------------------------
library(knitr)
library(rpf)
library(ggplot2)
library(reshape2)
library(gridExtra)
opts_chunk$set(echo=TRUE)

## -----------------------------------------------------------------------------
spec <- list()
spec[1:6] <- rpf.grm(factors=2)
gen.param <- sapply(spec, rpf.rparam)
colnames(gen.param) <- paste("i", 1:ncol(gen.param), sep="")
gen.param[2,] <- c(0,0,.5,.5,1,1)

resp <- rpf.sample(1000, spec, gen.param)

# hide latent factor that we don't know about
tspec <- list()
tspec[1:length(spec)] <- rpf.grm(factors=1)

grp <- list(spec=tspec, param=gen.param[-2,], mean=c(0), cov=diag(1), data=resp)

ChenThissen1997(grp)

## -----------------------------------------------------------------------------
(got <- SitemFit(grp))

## ----echo=FALSE,fig.height=2.5------------------------------------------------
SSplot <- function(sout, itemName, showSampleSize=TRUE) {
    s1 <- sout[[itemName]]
    obs <- s1$orig.observed
    ex <- s1$orig.expected
    rowTotal <- apply(obs, 1, sum)
    mask <- rowTotal > 0
    obs <- (obs / rowTotal)[mask,]
    ex <- (ex / rowTotal)[mask,]
    ss <- data.frame(sscore=as.numeric(names(rowTotal)), n=rowTotal)
    both <- rbind(cbind(type="expected", melt(ex)),
                  cbind(type="observed", melt(obs)))
    both$outcome <- factor(both$outcome, colnames(obs))
    plot <- ggplot(both, aes(x=sumScore, y=value)) + facet_wrap(~type) + ylim(0,1) +
        labs(y="probability", title=itemName)
    guide.style <- guide_legend(keywidth=.1, keyheight=.5, direction = "horizontal", title.position = "top",
                                label.position="bottom", label.hjust = 0.5, label.vjust = .5,
                                label.theme = element_text(angle = 90, size=8))
    plot <- plot + geom_line(aes(color=outcome)) + guides(color = guide.style)
    if (showSampleSize) {
        plot <- plot + geom_text(data=ss, aes(label=n, x=sscore), y = 1, size=2, angle=90)
    }
    plot
}
SSplot(got, "i1")
SSplot(got, "i2")
SSplot(got, "i3")
SSplot(got, "i4")
SSplot(got, "i5")
SSplot(got, "i6")

## -----------------------------------------------------------------------------
(got <- sumScoreEAP(grp))

## ----echo=FALSE,fig.height=2.5------------------------------------------------
got <- sumScoreEAPTest(grp)
df <- data.frame(score=as.numeric(rownames(got$tbl)),
            expected=got$tbl[,'p'] * got$n, observed=got$observed)
df <- melt(df, id="score", variable.name="source", value.name="n")
ggplot(df, aes(x=score, y=n, color=source)) + geom_line()

## -----------------------------------------------------------------------------
  data(science)
  spec <- list()
  spec[1:25] <- rpf.nrm(outcomes=3, T.c = lower.tri(diag(2),TRUE) * -1)
  param <- rbind(a=1, alf1=1, alf2=0,
        gam1=sfif$MEASURE + sfsf[sfsf$CATEGORY==1,"Rasch.Andrich.threshold.MEASURE"],
        gam2=sfif$MEASURE + sfsf[sfsf$CATEGORY==2,"Rasch.Andrich.threshold.MEASURE"])
  colnames(param) <- sfif$NAME
  iorder <- match(sfif$NAME, colnames(sfpf))
  responses <- sfpf[,iorder]
  rownames(responses) <- sfpf$NAME
  
  rpf.1dim.fit(spec, param, responses, sfpf$MEASURE, 2, wh.exact=TRUE)
  head(rpf.1dim.fit(spec, param, responses, sfpf$MEASURE, 1, wh.exact=TRUE))

## ----message=FALSE------------------------------------------------------------
library(rpf)

lmp.item<-rpf.lmp(q=2) # create item w/ 5th order polynomial
par<-c(.69,.71,-.5,-8.48,.52,-3.32) # item parameters
theta<-seq(-3,3,.1) # grid for latent trait

## Test the traceline or "prob" C++ function
P<-rpf.prob(lmp.item, par, theta)

## Prettier plots than this are of course possible
plot(theta, P[2,], type="l", ylim=c(0,1), xlab="Theta", ylab="P(Theta)")


## -----------------------------------------------------------------------------
## Derivatives of negative log-likelihood at arbitrary point with arbitrary weights
## Rounding only for easy reading for tutorial
round(rpf.dLL(lmp.item, par, theta[1], weight=c(5,7)),2)

## -----------------------------------------------------------------------------
## Analytical derivatives "deriv1" followed by "deriv2"
rpf.dTheta(lmp.item, par, where=-.5, dir=1)

## Numerical derivatives
library(numDeriv)
dTheta.wrap<-function(theta, spec, par, cat=1){
  rpf.prob(spec, par, theta)[cat]
}

## should match first element of gradient from rpf.dTheta
grad(dTheta.wrap, -.5, spec=lmp.item, par=par)

## should match first element of hessian from rpf.dTheta
hessian(dTheta.wrap, -.5, spec=lmp.item, par=par)

##' Liking for Science dataset
##'
##' These data are from Wright & Masters (1982, p. 18).
##' 
##' All items were fit to a 3 category Partial Credit Model (PCM)
##' using Ministep 3.75.0.
##'
##' @docType data
##' @keywords data
##' @name science
##' @aliases sfif sfpf sfsf sfxf
##' @references Wright, B. D. & Masters, G. N. (1982). \emph{Rating Scale
##' Analysis.} Chicago: Mesa Press.
##' @examples
##' data(science)
NULL

##' Knox Cube Test dataset
##'
##' These data from Wright & Stone (1979, p. 31) were fit with
##' Winsteps 3.73 using a 1PL model (slope fixed to 1).
##'
##' @docType data
##' @keywords data
##' @name kct
##' @aliases kct.items kct.people
##' @references
##' Wright, B. D. & Stone, M. H. (1979). \emph{Best Test Design: Rasch Measurement.} Univ of Chicago Social Research.
##' @examples
##' data(kct)
NULL

##' Identify the columns with most missing data
##'
##' If a reference column is given then only rows that are not missing
##' on the reference column are considered. Otherwise all rows are
##' considered.
##'
##' @template detail-group
##' @template arg-grp
##' @param omit the maximum number of items to omit
##' @param ref the reference column (optional)
##' @family scoring
bestToOmit <- function(grp, omit, ref=NULL) {
	if (missing(omit)) stop("How many items to omit?")
	if (omit == 0) return(NULL)
	dat <- grp$data
	wcol <- 1
	if (!is.null(grp$weightColumn)) {
		wcol <- dat[[grp$weightColumn]]
		dat <- dat[,-match(grp$weightColumn, colnames(dat))]
	}
	if (omit >= ncol(dat)) stop("Cannot omit all columns")
	if (!is.null(ref)) {
		mask <- !is.na(dat[[ref]])
		dat <- dat[mask,]
		if (length(wcol) > 1) wcol <- wcol[mask]
	}
	nacount <- apply(dat, 2, function(c) sum(is.na(c) * wcol))
	omit <- min(omit, sum(nacount > 0))
	if (omit == 0) return(NULL)
	names(sort(-nacount)[1:omit])
}

##' Omit the given items
##'
##' @template detail-group
##' @template arg-grp
##' @param excol vector of column names to omit
##' @family scoring
omitItems <- function(grp, excol) {
	if (missing(excol)) stop("Which items to omit?")
	if (length(excol) == 0) return(grp)
	imask <- -match(excol, colnames(grp$param))
	grp$spec <- grp$spec[imask]
	grp$param <- grp$param[,imask]
	grp$free <- grp$free[,imask]
	grp$labels <- grp$labels[,imask]
	grp$uniqueFree <- length(unique(grp$labels[grp$free], incomparables=NA))
	grp$data <- grp$data[,-match(excol, colnames(grp$data))]

	# We need to repack the data because
	# rows that only differed on the column
	# we removed are now the same.
	if (!is.null(grp$weightColumn)) {
		data <- expandDataFrame(grp$data, grp$weightColumn)
		grp$data <- compressDataFrame(data, .asNumeric=TRUE)
	}

	if (!is.null(grp$observedStats)) {
		grp$observedStats <- nrow(grp$data)
	}
	grp$omitted <- c(grp$omitted, excol)
	grp
}

##' Omit items with the most missing data
##'
##' Items with no missing data are never omitted, regardless of the
##' number of items requested.
##'
##' @template detail-group
##' @template arg-grp
##' @param omit the maximum number of items to omit
##' @family scoring
omitMostMissing <- function(grp, omit) {
	omitItems(grp, bestToOmit(grp, omit))
}

ssEAP <- function(grp, mask, twotier=FALSE) {
	if (missing(mask)) {
		mask <- rep(TRUE, ncol(grp$param))
	}
	.Call('_rpf_ssEAP_wrapper', grp, mask, twotier)
}

#' Collapse small sample size categorical frequency counts
#'
#' @param observed the observed frequency table
#' @param expected the expected frequency table
#' @param minExpected the minimum expected cell frequency
#'
#' Pearson's X^2 test requires some minimum frequency per cell to
#' avoid an inflated false positive rate. This function will merge
#' cells with the lowest frequency counts until all the counts are
#' above the minimum threshold. Cells that have been merged are filled
#' with NAs. The resulting tables and number of merged cells is
#' returned.
#'
#' @examples
#' O = matrix(c(7,31,42,20,0), 1,5)
#' E = matrix(c(3,39,50,8,0), 1,5)
#' collapseCategoricalCells(O,E,9)
collapseCategoricalCells <- function(observed, expected, minExpected=1) {
	.Call('_rpf_collapse', observed, expected, minExpected)
}

sumScoreEAPTestInternal <- function(result) {
	class(result) <- "summary.sumScoreEAPTest"
	if (result[['n']] == 0) return(result)
	expected <- matrix(result$expected, ncol=1)
	obs <- matrix(result$observed, ncol=1)

	result$rms.p <- log(ptw2011.gof.test(obs, expected))

	kc <- .Call('_rpf_collapse', obs, expected, 1.0)
	obs <- kc$O
	expected <- kc$E
	mask <- !is.na(expected) & expected!=0
	result$pearson.chisq <- sum((obs[mask] - expected[mask])^2 / expected[mask])
	result$pearson.df <- sum(mask)-1L
	result$pearson.p <- pchisq(result$pearson.chisq, result$pearson.df, lower.tail=FALSE, log.p=TRUE)
	result
}

##' Conduct the sum-score EAP distribution test
##'
##' @template detail-group
##' @template arg-grp
##' @template arg-dots
##' @param qwidth DEPRECATED
##' @param qpoints DEPRECATED
##' @param .twotier whether to enable the two-tier optimization
##' @family diagnostic
##' @references
##' Li, Z., & Cai, L. (2018). Summed Score Likelihood-Based Indices for
##' Testing Latent Variable Distribution Fit in
##' Item Response Theory. \emph{Educational and
##' Psychological Measurement, 78}(5), 857-886.
sumScoreEAPTest <- function(grp, ..., qwidth=6.0, qpoints=49L, .twotier=TRUE) {
	if (length(list(...)) > 0) {
		stop(paste("Remaining parameters must be passed by name", deparse(list(...))))
	}
	if (is.null(grp$data)) {
		stop("distributionTest cannot be conducted because there is no data")
	}
  if (!missing(qwidth) || !missing(qpoints)) complainAboutQuadSpec()

	tbl <- ssEAP(grp, twotier=.twotier)
	rownames(tbl) <- 0:(nrow(tbl)-1)
	result <- list(tbl=tbl)
	oss <- observedSumScore(grp)
	result$n <- oss$n
	result$observed <- oss$dist
	result$expected <- result$n * tbl[,1]
	names(result$observed) <- rownames(tbl)
	names(result$expected) <- rownames(tbl)
	result$omitted <- grp$omitted
	result <- sumScoreEAPTestInternal(result)
	result
}

"+.summary.sumScoreEAPTest" <- function(e1, e2) {
	e2name <- deparse(substitute(e2))
	if (!inherits(e2, "summary.sumScoreEAPTest")) {
		stop("Don't know how to add ", e2name, " to a sumScoreEAPTest",
		     call. = FALSE)
	}

	if (length(e1$observed) != length(e2$observed)) {
		stop("The two groups have a different maximum sum-score. Sum-score tests cannot be combined")
	}
	if (any(e1$omitted != e2$omitted)) {
		stop("The two groups have different items omitted. Sum-score tests cannot be combined")
	}

	cb <- list(observed=e1$observed + e2$observed,
		   expected=e1$expected + e2$expected,
		   n=e1$n + e2$n,
		   omitted=e1$omitted)
	cb <- sumScoreEAPTestInternal(cb)
	cb
}

print.summary.sumScoreEAPTest <- function(x,...) {
	cat(sprintf("Latent distribution fit test (n=%d):\n", x$n))
	if (!is.null(x$omitted)) {
		cat(paste("  Omitted:", paste(x$omitted, collapse=" "), "\n"))
	}
	if (!is.null(x$rms.p)) {
		cat(sprintf("  RMS log(p) = %.2f\n", x$rms.p))
	}
	if (!is.null(x$pearson.p)) {
		cat(sprintf("  Pearson X^2(%3d) = %.2f, log(p) = %.2f\n",
			    x$pearson.df, x$pearson.chisq, x$pearson.p))
	}
}

##' Compute the sum-score EAP table
##'
##' Observed tables cannot be computed when data is
##' missing. Therefore, you can optionally omit items with the
##' greatest number of responses missing when conducting the
##' distribution test.
##'
##' When two-tier covariance structure is detected, EAP scores are
##' only reported for primary factors. It is possible to compute EAP
##' scores for specific factors, but it is not clear why this would be
##' useful because they are conditional on the specific factor sum
##' scores. Moveover, the algorithm to compute them efficiently has not been
##' published yet (as of Jun 2014).
##'
##' @template detail-group
##' @template arg-grp
##' @template arg-dots
##' @param qwidth DEPRECATED
##' @param qpoints DEPRECATED
##' @param .twotier whether to enable the two-tier optimization
##' @family scoring
##' @examples
##' # see Thissen, Pommerich, Billeaud, & Williams (1995, Table 2)
##'  spec <- list()
##'  spec[1:3] <- list(rpf.grm(outcomes=4))
##'
##'  param <- matrix(c(1.87, .65, 1.97, 3.14,
##'                    2.66, .12, 1.57, 2.69,
##'                    1.24, .08, 2.03, 4.3), nrow=4)
##'  # fix parameterization
##'  param <- apply(param, 2, function(p) c(p[1], p[2:4] * -p[1]))
##'
##'  grp <- list(spec=spec, mean=0, cov=matrix(1,1,1), param=param)
##'  sumScoreEAP(grp)
sumScoreEAP <- function(grp, ..., qwidth=6.0, qpoints=49L, .twotier=TRUE) {
	if (length(list(...)) > 0) {
		stop(paste("Remaining parameters must be passed by name", deparse(list(...))))
	}

  if (!missing(qwidth) || !missing(qpoints)) complainAboutQuadSpec()

	tbl <- ssEAP(grp, twotier=.twotier)
	rownames(tbl) <- 0:(nrow(tbl)-1)
	tbl
}

##' Compute the observed sum-score
##'
##' When \code{summary=TRUE}, tabulation uses row frequency
##' multiplied by row weight.
##'
##' @template detail-group
##' @template arg-grp
##' @template arg-dots
##' @param mask a vector of logicals indicating which items to include
##' @param summary whether to return a summary (default) or per-row scores
##' @family scoring
##' @examples
##' spec <- list()
##' spec[1:3] <- rpf.grm(outcomes=3)
##' param <- sapply(spec, rpf.rparam)
##' data <- rpf.sample(5, spec, param)
##' colnames(param) <- colnames(data)
##' grp <- list(spec=spec, param=param, data=data)
##' observedSumScore(grp)
observedSumScore <- function(grp, ..., mask, summary=TRUE) {
	if (length(list(...)) > 0) {
		stop(paste("Remaining parameters must be passed by name", deparse(list(...))))
	}
	if (missing(mask)) {
		mask <- rep(TRUE, ncol(grp$param))
	}
	if (!summary) {
		cols <- colnames(grp$param)[mask]
		dat <- grp$data[,cols]
		ss <- apply(sapply(dat, unclass) - 1, 1, sum)
		names(ss) <- rownames(dat)
		return(ss)
	}
	got <- .Call('_rpf_observedSumScore_cpp', grp, mask)
	if (got[['n']] == 0) {
		warning("Some columns are all missing; cannot compute observedSumScore")
	}
	class(got) <- "summary.observedSumScore"
	got
}

print.summary.observedSumScore <- function(x,...) {
	print(x$dist)
	cat(sprintf("  N = %d\n", x$n))
}

##' Produce an item outcome by observed sum-score table
##'
##' @template arg-grp
##' @param mask a vector of logicals indicating which items to include
##' @param interest index or name of the item of interest
##' @template detail-group
##' @family scoring
##' @examples
##' set.seed(1)
##' spec <- list()
##' spec[1:3] <- rpf.grm(outcomes=3)
##' param <- sapply(spec, rpf.rparam)
##' data <- rpf.sample(5, spec, param)
##' colnames(param) <- colnames(data)
##' grp <- list(spec=spec, param=param, data=data)
##' itemOutcomeBySumScore(grp, c(FALSE,TRUE,TRUE), 1L)
itemOutcomeBySumScore <- function(grp, mask, interest) {
	if (is.character(interest)) {
		interest <- match(interest, colnames(grp$param))
	}
	got <- .Call('_rpf_itemOutcomeBySumScore_cpp', grp, mask, interest)
	rownames(got$table) <- 0:(nrow(got$table)-1L)
	col <- colnames(grp$param)[interest]
	colnames(got$table) <- levels(grp$data[,col])
	class(got) <- "summary.itemOutcomeBySumScore"
	got
}

print.summary.itemOutcomeBySumScore <- function(x,...) {
	print(x$table)
	cat(sprintf("  N = %d\n", x$n))
}

##' Compute Expected A Posteriori (EAP) scores
##'
##' If you have missing data then you must specify
##' \code{minItemsPerScore}.  This option will set scores to NA when
##' there are too few items to make an accurate score estimate.  If
##' you are using the scores as point estimates without considering
##' the standard error then you should set \code{minItemsPerScore} as
##' high as you can tolerate. This will increase the amount of missing
##' data but scores will be more accurate. If you are carefully
##' considering the standard errors of the scores then you can set
##' \code{minItemsPerScore} to 1. This will mimic the behavior of most
##' other IFA software wherein scores are estimated if there is at
##' least 1 non-NA item for the score. However, it may make more sense
##' to set \code{minItemsPerScore} to 0. When set to 0, all NA rows
##' are scored to the prior distribution.
##'
##' Output is not affected by the presence of a \code{weightColumn}.
##'
##' @template detail-group
##' @template arg-grp
##' @template arg-dots
##' @param compressed output one score per observed data row even when freqColumn is set (default FALSE)
##' @family scoring
##' @examples
##' spec <- list()
##' spec[1:3] <- list(rpf.grm(outcomes=3))
##' param <- sapply(spec, rpf.rparam)
##' data <- rpf.sample(5, spec, param)
##' colnames(param) <- colnames(data)
##' grp <- list(spec=spec, param=param, data=data, minItemsPerScore=1L)
##' EAPscores(grp)
EAPscores <- function(grp, ..., compressed=FALSE) {
	if (length(list(...)) > 0) {
		stop(paste("Remaining parameters must be passed by name", deparse(list(...))))
	}

	ctbl <- .Call('_rpf_eap_wrapper', grp)

	if (!compressed && !is.null(grp$freqColumn)) {
		freq <- grp$data[[ grp$freqColumn ]]
		rows <- sum(freq)
		indexVector <- rep(NA, rows)
		rx <- 1L
		ix <- 1L
		while (rx <= length(freq)) {
			indexVector[ix:(ix + freq[rx] - 1)] <- rx
			ix <- ix + freq[rx]
			rx <- rx + 1L
		}
		ctbl <- ctbl[indexVector,]
	}

	ctbl
}

#' Convert response function slopes to factor loadings
#'
#' All slopes are divided by the ogive constant. Then the following
#' transformation is applied to the slope matrix,
#'
#' \deqn{\frac{\mathrm{slope}}{\left[ 1 + \mathrm{rowSums}(\mathrm{slope}^2) \right]^\frac{1}{2}}}
#'
#' @param slope a matrix with items in the columns and slopes in the rows
#' @param ogive the ogive constant (default \link{rpf.ogive})
#' @family factor model equivalence
#' @return
#' a factor loading matrix with items in the rows and factors in the columns
toFactorLoading <- function(slope, ogive=rpf.ogive) {
  tmp <- t(slope / ogive)
  got <- tmp / sqrt(1 + rowSums(tmp ^ 2))
  h2 <- rowSums(got^2)
  if(any(h2 > .975)) {
    warning("Solution has Heywood cases. Interpret with caution.")
  }
  got
}

#' Convert factor loadings to response function slopes
#'
#' @param loading a matrix with items in the rows and factors in the columns
#' @param ogive the ogive constant (default \link{rpf.ogive})
#' @family factor model equivalence
#' @return
#' a slope matrix with items in the columns and factors in the rows
fromFactorLoading <- function(loading, ogive=rpf.ogive) {
  t(ogive * loading / sqrt(1 - rowSums(loading ^ 2)))
}

#' Convert response function intercepts to factor thresholds
#'
#' @param intercept a matrix with items in the columns and intercepts in the rows
#' @param slope a matrix with items in the columns and slopes in the rows
#' @param ogive the ogive constant (default \link{rpf.ogive})
#' @family factor model equivalence
#' @return
#' a factor threshold matrix with items in the columns and factor thresholds in the rows
toFactorThreshold <- function(intercept, slope, ogive=rpf.ogive) {
  tmp <- t(slope / ogive)
  thr <- t(intercept / ogive)
  got <- -t( thr / sqrt(1 + rowSums(tmp ^ 2)) )
  got
}

#' Convert factor thresholds to response function intercepts
#'
#' @param threshold a matrix with items in the columns and thresholds in the rows
#' @param loading a matrix with items in the rows and factors in the columns
#' @param ogive the ogive constant (default \link{rpf.ogive})
#' @family factor model equivalence
#' @return
#' an item intercept matrix with items in the columns and intercepts in the rows
fromFactorThreshold <- function(threshold, loading, ogive=rpf.ogive) {
  got <- t(-ogive*t(threshold) / sqrt(1 - rowSums(loading ^ 2)) )
  got
}

#options(error = browser)
library(testthat)
library(rpf)

context("weightColumn")

data(LSAT6)
LSAT6[1:5] <- as.data.frame(lapply(LSAT6[1:5], ordered, 0:1))
LSAT6$Freq <- as.integer(LSAT6$Freq)
spec <- list()
spec[1:5] <- list(rpf.grm())
names(spec) <- colnames(LSAT6)[1:5]
param <- matrix(c(1.0,0.0), nrow=2, ncol=5, dimnames=list(c('a','b'), names(spec)))

grp1 <- list(spec=spec,
             param=param,
             data=LSAT6,
             freqColumn='Freq',
             observedStats=nrow(LSAT6)-1L,
             minItemsPerScore=5L)

grp2 <- grp1
grp2$data <- expandDataFrame(LSAT6, freqName = "Freq")
grp2$freqColumn <- NULL

# Sample sample size, but 1 less observation
grp3 <- grp1
grp3$data$Weight <- 1.0
grp3$data$Freq[1] <- 2L
grp3$data$Weight[1] <- 1.5
grp3$weightColumn <- 'Weight'

test_that("EAPscores", {
  grp1$score <- EAPscores(grp1)
  grp2$score <- EAPscores(grp2)
  grp3$score <- EAPscores(grp3)
  
  expect_equal(nrow(EAPscores(grp1, compressed=TRUE)), nrow(LSAT6))
  expect_equal(nrow(grp1$score), nrow(grp2$score))
  expect_equal(grp1$score[,1], grp2$score[,1])
  expect_equal(nrow(grp3$score), 999L)
  expect_equal(nrow(EAPscores(grp3, compressed=TRUE)), nrow(LSAT6))
})

test_that("observedSumScore", {
  t1 <- observedSumScore(grp1)
  t2 <- observedSumScore(grp2)
  t3 <- observedSumScore(grp3)
  expect_equal(t1$n, t2$n)
  expect_equal(t1$dist, t2$dist)
  expect_equal(t1$n, t3$n)
  expect_equal(t1$dist, t3$dist)
})

test_that("sumScoreEAPTest", {
  t1 <- sumScoreEAPTest(grp1)
  t2 <- sumScoreEAPTest(grp2)
  t3 <- sumScoreEAPTest(grp3)
  expect_equal(t1$n, t2$n)
  expect_equal(t1$observed, t2$observed)
  expect_equal(t1$pearson.p, t2$pearson.p)
  expect_equal(t1$n, t3$n)
  expect_equal(t1$observed, t3$observed)
  expect_equal(t1$pearson.p, t3$pearson.p)
})

test_that("itemOutcomeBySumScore", {
  t1 <- itemOutcomeBySumScore(grp1, c(FALSE, rep(TRUE,4)), 1)
  t2 <- itemOutcomeBySumScore(grp2, c(FALSE, rep(TRUE,4)), 1)
  t3 <- itemOutcomeBySumScore(grp3, c(FALSE, rep(TRUE,4)), 1)
  expect_equal(t1$n, t2$n)
  expect_equal(t1$table, t2$table)
  expect_equal(t1$n, t3$n)
  expect_equal(t1$table, t3$table)
})

test_that("ChenThissen1997", {
  t1 <- ChenThissen1997(grp1)
  t2 <- ChenThissen1997(grp2)
  t3 <- ChenThissen1997(grp3)
  expect_equal(t1$pval, t2$pval)
  expect_equal(t1$pval, t3$pval)
})

test_that("SitemFit", {
  t1 <- SitemFit(grp1)
  tbl1 <- t(sapply(t1, function(r) c(n=r$n, df=r$df, stat=r$statistic, pval=r$pval)))
  t2 <- SitemFit(grp2)
  tbl2 <- t(sapply(t2, function(r) c(n=r$n, df=r$df, stat=r$statistic, pval=r$pval)))
  t3 <- SitemFit(grp3)
  tbl3 <- t(sapply(t3, function(r) c(n=r$n, df=r$df, stat=r$statistic, pval=r$pval)))
  expect_equal(tbl1, tbl2)
  expect_equal(tbl1, tbl3)
})

#options(error = browser)
library(testthat)
library(rpf)

suppressWarnings(RNGversion("3.5"))

context("score")

test_that("tpbw1995-table2", {
  set.seed(1)
  spec <- list()
  spec[1:3] <- list(rpf.grm(outcomes=4))
  
  param <- matrix(c(1.87, .65, 1.97, 3.14,
                    2.66, .12, 1.57, 2.69,
                    1.24, .08, 2.03, 4.3), nrow=4)
  # fix parameterization
  param <- apply(param, 2, function(p) c(p[1], p[2:4] * -p[1]))
  colnames(param) <- paste('i', 1:3, sep="")
  demoData <- rpf.sample(2, spec, param)
  
  grp <- list(spec=spec, mean=0, cov=matrix(1,1,1), param=param,
              data = demoData[0,,drop=FALSE])
  
  expect_error(EAPscores(grp), "EAP requested but there are no data rows")

  grp$data <- demoData
  scores <- EAPscores(grp)
  
  expect_equal(scores[,1], c(0.084, -0.154), tolerance=.01)
  expect_equal(scores[,2], c(0.539, 0.572), tolerance=.01)
})

suppressWarnings(RNGversion("3.5"))
#options(error = browser)
library(testthat)
library(rpf)

context("ot2000")
options(mc.cores=1L) #otherwise not picked up by covr

test_that("simple case", {
  require(rpf)
  set.seed(1)

  spec <- list()
  spec[1:3] <- list(rpf.drm())

  gen.p <- matrix(c(1,0,0,1,
                    1,-1,0,1,
                    1,1,0,1), ncol=3, nrow=4)  # note byrow=FALSE
  gen.p[3:4,] <- logit(gen.p[3:4,])
  data <- rpf.sample(200, spec, gen.p)
                                        #  write.csv(data, "fit-test.csv", row.names=FALSE, quote=FALSE)

  param <- matrix(c(.71,.03,0, 1,
                    1.53,-.85, 0, 1,
                    1.1,.9,0, 1), ncol=3, nrow=4)
  param[3:4,] <- logit(param[3:4,])
  
  colnames(param) <- paste("i", 1:3, sep="")
  grp <- list(spec=spec, param=param, data=data, mean=0, cov=matrix(1,1,1))

  got <- SitemFit(grp, method="pearson")
  
  tbl <- round(t(sapply(got, function(row) c(stat=row$statistic, df=row$df, p=row$pval))),2)
  expect_equal(sum(tbl[,'df']), 9)  # not sure TODO
  expect_equal(sum(tbl[,'stat']), 22.55)
})

test_that("orlando-thissen-2000", {
  require(rpf)
  set.seed(7)
  grp <- list(spec=list())
  grp$spec[1:20] <- list(rpf.grm())
  grp$param <- sapply(grp$spec, rpf.rparam, version=1L)
  colnames(grp$param) <- paste("i", 1:20, sep="")
  grp$mean <- 0
  grp$cov <- diag(1)
  grp$free <- grp$param != 0
  grp$data <- rpf.sample(500, grp=grp)
  
  got <- SitemFit(grp, method="pearson", omit=1)
  expect_true(all(sapply(got, function(ii) is.null(ii$omitted))))
  stat <- sapply(got, function(x) x$statistic)
  names(stat) <- NULL
  Estat <- c(10.96, 14.32, 19.53, 15.53, 15.16, 7.75, 31.24, 8.63, 8.36,
             12.55, 13.94, 11.01, 5.32, 9.49, 10.62, 12.88, 22.21, 15.17, 
             14.15, 16.01)
  expect_equal(stat, Estat, tolerance=.01)
  
  E1orig <- structure(c(2, 3.99, 12.95, 21.82, 21.66, 25.29, 30.5, 32.29,  35.09, 24.41, 34.33, 15.74,
              18.61, 14.44, 10.79, 5.72, 3.82,  1.5, 0.23, 0.12, 0, 0.01, 0.05, 0.18, 0.34,
              0.71, 1.5, 2.71,  4.91, 5.59, 12.67, 9.26, 17.39, 21.56, 26.21, 23.28, 27.18,
              19.5,  5.77, 5.88), .Dim = c(20L, 2L))
  oexp <- got[[1]]$orig.expected
  dimnames(oexp) <- NULL
  expect_equal(oexp, E1orig, tolerance=.01)
  
  E1 <- structure(c(2, 3.99, 12.95, 21.82, 21.66, 25.29, 30.5, 32.29,  35.09, 24.41, 34.33,
                    15.74, 18.61, 14.44, 10.79, 5.72, 3.82,  1.85, 0, 0, 0, 0, 0, 0, 0, 1.3,
                    1.5, 2.71, 4.91, 5.59, 12.67,  9.26, 17.39, 21.56, 26.21, 23.28, 27.18,
                    19.5, 5.77, 5.88), .Dim = c(20L,  2L))
  mask <- !is.na(got[[1]]$expected)
  expect_equal(got[[1]]$expected[mask], E1[mask], tolerance=.01)
  
  expect_equal(got[[1]]$pval, -0.8065, tolerance=.01)

  got <- SitemFit(grp, method="pearson", alt=TRUE)
  stat <- sapply(got, function(x) x$statistic)
  names(stat) <- NULL
  #cat(deparse(round(stat, 2)))
  Estat <- c(16.6, 13.68, 13.19, 14.03, 19.86, 11.02, 29.92, 6.95, 18.31,
             14.78, 10.06, 9.27, 5.67, 9.3, 14.75, 18.03, 20.18, 20.19, 15.89,  11.57)
  expect_equal(stat, Estat, tolerance=.01)
})

test_that("fit w/ mcar", {
  require(rpf)
  require(testthat)
  set.seed(7)
  grp <- list(spec=list(), qwidth=5, qpoints=31)
  grp$spec[1:20] <- list(rpf.grm())
  grp$param <- sapply(grp$spec, rpf.rparam, version=1L)
  colnames(grp$param) <- paste("i", 1:20, sep="")
  grp$free <- grp$param != 0
  grp$data <- rpf.sample(500, grp=grp, mcar=.1)

  got <- sumScoreEAPTest(omitMostMissing(grp, 3L))
  expect_equal(got$n, 101L)
  expect_equal(got$rms.p, -1.87, tolerance=.01)
  expect_equal(got$pearson.df, 16L)
  expect_equal(got$pearson.p, -1.09, tolerance=.01)

  grp1 <- grp
  grp1$data <- grp$data[1:250,]
  grp2 <- grp
  grp2$data <- grp$data[251:500,]
  e1 <- sumScoreEAPTest(grp1) + sumScoreEAPTest(grp2)
  e2 <- sumScoreEAPTest(grp)
  chk <- c('n','pearson.df', 'pearson.chisq', 'pearson.p')
  expect_equal(unlist(e1[chk]), unlist(e2[chk]))
  
  got <- SitemFit(grp, omit=2L)
  stat <- sapply(got, function(x) x$statistic)
  names(stat) <- NULL
  Estat <- c(9.99, 8.1, 10.64, 14.65, 6.69, 10.88, 16.19, 2.95, 7.3, 11.12, 
             12.61, 3.73, 7.53, 8.71, 12.2, 11.66, 14.19, 12.15, 10.44, 12.21 )
  expect_equal(stat, Estat, tolerance=.01)
  
  e1 <- SitemFit(grp1) + SitemFit(grp2)
  e2 <- SitemFit(grp)
  expect_equal(sapply(e1, function(ii) ii$statistic),
               sapply(e2, function(ii) ii$statistic))
})

test_that("2tier fit", {
  set.seed(1)
  require(rpf)
  numItems <- 6
  spec <- list()
  spec[1:numItems] <- list(rpf.drm(factors=3))
  param <- sapply(spec, rpf.rparam, version=1)
  gsize <- numItems/3
  for (gx in 0:2) {
    if (gx != 1) {
      param['a2', seq(gx * gsize+1, (gx+1)*gsize)] <- 0
    }
    if (gx != 2) {
      param['a3', seq(gx * gsize+1, (gx+1)*gsize)] <- 0
    }
  }
  grp <- list(spec=spec, param=param, mean=runif(3, -1, 1), cov=diag(runif(3,.5,2)))
  grp$data <- rpf.sample(500, grp=grp)
  colnames(grp$param) <- colnames(grp$data)
  grp$qwidth <- 2
  grp$qpoints <- 5L
  
  got <- SitemFit(grp, .twotier=FALSE)
  tt <- SitemFit(grp, .twotier=TRUE)
  expect_equal(sapply(got, function(x) x$statistic),
               sapply(tt, function(x) x$statistic), .001)

  got <- SitemFit(grp, .twotier=FALSE, alt = TRUE)
  tt <- SitemFit(grp, .twotier=TRUE, alt=TRUE)
  expect_equal(sapply(got, function(x) x$statistic),
               sapply(tt, function(x) x$statistic), .001)
})

if (0) {
  library(mirt)
  dat <- expand.table(LSAT6)
  model <- mirt.model('F = 1-5
                    CONSTRAIN = (1-5, a1)')
  (mod <- mirt(dat, model))
  itemfit(mod, X2=TRUE, S_X2.tables=TRUE)$E[[3]]
  itemfit(mod, X2=TRUE)
  
  library(rpf)
  spec <- list()
  spec[1:5] <- list(rpf.drm())
  param <- sapply(coef(mod)[-6], function(x) x)
  param[3:4,] <- rpf::logit(param[3:4,])
  dat2 <- as.data.frame(lapply(as.data.frame(dat), ordered, levels=0:1))
  grp <- list(spec=spec, param=param, mean=0, cov=diag(1), data=dat2)
  got <- SitemFit(grp, method="pearson", alt=FALSE)
}

#options(error = browser)
library(testthat)
library(rpf)

suppressWarnings(RNGversion("3.5"))

context("sumscore")

test_that("observedSumScore", {
  require(rpf)
  set.seed(1)
  spec <- list()
  spec[1:3] <- list(rpf.grm(outcomes=3))
  param <- sapply(spec, rpf.rparam, version=1)
  data <- rpf.sample(5, spec, param)
  colnames(param) <- colnames(data)
  grp <- list(spec=spec, param=param, data=data)
  obs <- observedSumScore(grp)
  expect_equal(obs$dist, c(1L, 1L, 0L, 1L, 1L, 0L, 1L))
  
  dperm <- sample.int(3)
  data <- data[,dperm]
  
  mask <- c(TRUE, FALSE, TRUE)
  obs <- observedSumScore(grp, mask=mask)
  expect_equal(obs$dist, rep(1L, 5))
})

test_that("itemOutcomeBySumScore", {
  set.seed(1)
  spec <- list()
  spec[1:3] <- list(rpf.grm(outcomes=3))
  param <- sapply(spec, rpf.rparam, version=1)
  data <- rpf.sample(5, spec, param)
  colnames(param) <- colnames(data)
  grp <- list(spec=spec, param=param, data=data)
  levels(grp$data[,1]) <- c('a','b','c')
  tbl <- itemOutcomeBySumScore(grp, c(FALSE,TRUE,TRUE), 1L)
  
  want <- structure(c(1L, 0L, 0L, 0L, 0L, 1L, 0L, 0L, 1L, 0L, 0L, 1L, 0L,  0L, 1L),
                    .Dim = c(5L, 3L), .Dimnames = list(c("0", "1", "2",  "3", "4"), c("a","b","c")))
  expect_equal(tbl$table, want)  
})

# Thissen, Pommerich, Billeaud, & Williams (1995)
test_that("tpbw1995-table2", {
  spec <- list()
  spec[1:3] <- list(rpf.grm(outcomes=4))
  
  param <- matrix(c(1.87, .65, 1.97, 3.14,
                    2.66, .12, 1.57, 2.69,
                    1.24, .08, 2.03, 4.3), nrow=4)
  # fix parameterization
  param <- apply(param, 2, function(p) c(p[1], p[2:4] * -p[1]))

  grp <- list(spec=spec, mean=0, cov=matrix(1,1,1), param=param)
  
  got <- sumScoreEAP(grp)
  
  expect_equal(sum(got[,'p']), 1, tolerance=.001)
  
  #cat(deparse(round(got[,2],3)))
  rownames(got) <- NULL
  
  ssP <- c(0.325, 0.241, 0.183, 0.123, 0.069, 0.035, 0.016, 0.006, 0.002,  0)
  expect_equal(got[,'p'], ssP, tolerance=.01)
  ssEAP <- c(-0.885, -0.179, 0.332, 0.744, 1.115, 1.482, 1.843, 2.212, 2.622,  2.999)
  expect_equal(got[,'s1'], ssEAP, tolerance=.01)
  ssVar <- c(0.494, 0.378, 0.329, 0.299, 0.297, 0.296, 0.29, 0.296, 0.313,  0.328)
  expect_equal(got[,'se1'], sqrt(ssVar), tolerance=.01)
  expect_equal(got[,'cov1'], ssVar, tolerance=.01)
})

verifySumP <- function(grp, sseap, N=2000) {  # a good fit is close to 1
  sim <- apply(sapply(rpf.sample(N, grp=grp), unclass), 1, function(r) sum(r-1))
  observed <- tabulate(1+sim, length(sseap[,1]))
  #  print(observed/N)
  ptw2011.gof.test(observed, N*sseap[,1])
}

if (0) {
  fm <- read.flexmirt("~/ifa/ifa-2d-mg/2d-mg-prm.txt")
  
  got <- sumScoreEAP(fm$G1, 5, 21L)  # matches flexmirt exactly
  verifySumP(fm$G1, got)
  
  got <- sumScoreEAP(fm$G2, 5, 21L)  # doesn't match flexmirt
  verifySumP(fm$G2, got, N=5000)  # but looks feasible
  
  got <- sumScoreEAP(fm$G3, 5, 21L)  # doesn't match flexmirt
  verifySumP(fm$G3, got, N=5000)  # but looks feasible
}

test_that("2tier sumScoreEAP", {
  set.seed(1)
  require(rpf)
  numItems <- 6
  spec <- list()
  spec[1:numItems] <- list(rpf.drm(factors=3))
  param <- sapply(spec, rpf.rparam, version=1)
  gsize <- numItems/3
  for (gx in 0:2) {
    if (gx != 1) {
      param['a2', seq(gx * gsize+1, (gx+1)*gsize)] <- 0
    }
    if (gx != 2) {
      param['a3', seq(gx * gsize+1, (gx+1)*gsize)] <- 0
    }
  }
  grp <- list(spec=spec, param=param, mean=runif(3, -1, 1),
              cov=diag(runif(3,.5,2)), qwidth=2, qpoints=5L)
  grp$data <- rpf.sample(500, grp=grp)
  colnames(grp$param) <- colnames(grp$data)
  
  got <- sumScoreEAP(grp, .twotier=FALSE)
  tt <- sumScoreEAP(grp, .twotier=TRUE)
  expect_equal(tt, got[,c(1:2,5,8)], .001)
  
  grp2 <- omitItems(grp, c('i4','i6'))
  got <- sumScoreEAP(grp2)
#  cat(deparse(round(log(got[,'p']), 2)))
  expect_equal(log(got[,'p']), c(-2.81, -1.37, -0.97, -1.4, -2.77),
               check.names=FALSE, tolerance=.01)
})

#options(error = browser)
library(testthat)
library(rpf)

suppressWarnings(RNGversion("3.5"))

context("multinomialFit")

test_that("multinomialFit full info, simple", {
  require(rpf)
  set.seed(7)
  grp <- list(spec=list())
  grp$spec[1:10] <- list(rpf.grm())
  grp$param <- sapply(grp$spec, rpf.rparam)
  colnames(grp$param) <- paste("i", 1:10, sep="")
  grp$mean <- 0
  grp$cov <- diag(1)
  grp$uniqueFree <- sum(grp$param != 0)
  grp$data <- compressDataFrame(rpf.sample(1500, grp=grp), .asNumeric=TRUE)
  grp$weightColumn <- 'freq'
  grp$observedStats <- nrow(grp$data) - 1
  
  # screw up fit
  grp$param[2,] <- grp$param[2,] + runif(10, -.75, .75)

  got <- multinomialFit(grp, method="pearson")
  expect_equal(got$statistic, 1287.46, tolerance=.01)
  expect_equal(got$df, 209)
  expect_equal(got$pval, -354, tolerance=.1)

  got <- multinomialFit(grp, method="lr")
  expect_equal(got$statistic, 914.35, tolerance=.01)
  expect_equal(got$df, 209)
  expect_equal(got$pval, -202, tolerance=.1)
})

test_that("multinomialFit full info, simple w/ missingness", {
  require(rpf)
  require(testthat)
  
  set.seed(7)
  grp <- list(spec=list())
  grp$spec[1:10] <- list(rpf.grm())
  grp$param <- sapply(grp$spec, rpf.rparam)
  colnames(grp$param) <- paste("i", 1:10, sep="")
  grp$mean <- 0
  grp$cov <- diag(1)
  grp$free <- grp$param != 0
  grp$labels <- matrix(NA, nrow(grp$param), ncol(grp$param))
  grp$uniqueFree <- sum(grp$param != 0)
  grp$data <- compressDataFrame(rpf.sample(1500, grp=grp, mcar=.1), .asNumeric=TRUE)
  grp$weightColumn <- 'freq'
  grp$observedStats <- nrow(grp$data) - 1
  
  got <- multinomialFit(grp, method="pearson")
  expect_equal(got$statistic, 1115.63, tolerance=.01)
  expect_equal(got$df, 710)
  expect_equal(got$pval, -45.7, tolerance=.1)

  got <- multinomialFit(grp, method="lr")
  expect_equal(got$statistic, 414.691, tolerance=.01)
  expect_equal(got$df, 710)
  expect_equal(got$pval, 0, tolerance=.01)
  expect_equal(got$n, 521)

  got <- multinomialFit(omitMostMissing(grp, 1), method="lr")
  expect_equal(got$statistic, 253.39, tolerance=.01)
  expect_equal(got$df, 585)
  expect_equal(got$pval, 0, tolerance=.01)
  expect_equal(got$n, 598)
})

test_that("multinomialFit full info, two-tier", {
  require(rpf)
  spec <- list()
  spec[1:5] <- list(rpf.drm(factors=3))
  gen.param <- sapply(spec, rpf.rparam)
  gen.param['a2', 1:2] <- 0
  gen.param['a3', 3] <- 0
  gen.param[c('a2','a3'), 4:5] <- 0
  colnames(gen.param) <- paste("i", 1:ncol(gen.param), sep="")
  resp <- rpf.sample(1500, spec, gen.param)
  grp <- list(spec=spec, param=gen.param, mean=runif(3, 0, 1), cov=diag(runif(3,1,2)),
              data=resp, uniqueFree=sum(gen.param!=0), qwidth=5, qpoints=21)
  
  got1 <- multinomialFit(grp, .twotier = FALSE)
  got2 <- multinomialFit(grp, .twotier = TRUE)
  expect_equal(got1$statistic, got2$statistic, tolerance=.001)
  expect_equal(got1$df, got2$df)
  expect_equal(got1$pval, got2$pval, tolerance=.001)
})

if (0) {
  # matches flexmirt exactly
  library(mirt)
  dat <- expand.table(LSAT7)
  (mod1 <- mirt(dat, 1, calcNull=TRUE))
  i1 <- extract.item(mod1, 1)
  expected.item(i1, seq(-4,4,.5), min=0L)
  M2(mod1)
  M2(mod1@null.mod)
  write.table(dat, file = "LSAT7.csv", row.names = FALSE, col.names=FALSE)
}

######################################################################################################################################
######################################################################################################################################

#Supplemental R code for plotting ICCs, TCCs, IIF, and TIFs from flexMIRT output files

#Authors: Li Cai & Carrie R. Houts
#Date: 10/12/2012

######################################################################################################################################
######################################################################################################################################

#The R programs provided are free software; you can redistribute and/or modify them under the terms of the GNU General Public License
#(http://www.gnu.org/copyleft/gpl.html) as published by the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.  The programs are distributed in the hope that they will be useful, but WITHOUT ANY WARRANTY;
#without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#See the GNU General Public License for more details.

######################################################################################################################################
######################################################################################################################################

##' Read a flexMIRT PRM file
##'
##' \lifecycle{experimental}
##' This was last updated in 2017 and may no longer work.
##'
##' Load the item parameters from a flexMIRT PRM file.
##' @template detail-group
##'
##' @param fname file name
##' @return a list of groups as described in the details
read.flexmirt <- function(fname) {
  groups <- list()

  ncol <- max(count.fields(fname, sep = "\t"))
  prm <- read.delim(fname,header=F, col.names=1:ncol)

  # find out how many rows
  nlines <- nrow(prm)

  # determine the number of groups
  ng <- 0
  for (i in 1:nlines) {
    if (prm[i,1] == 0) { # group
      ng <- ng+1
    }
  }

  for (g in 1:ng) {
    # select all items and this group
    index <- (prm[,3] == g)
    thisGroup <- prm[index,]

    g.name <- "?"
    g.dist <- list()
    g.label <- list()
    g.spec <- list()
    g.param <- list()

    for (i in 1:nrow(thisGroup)) {
      spec <- NULL
      param <- c()

      dims <- thisGroup[i,4]

      if (thisGroup[i,1] == 1) { # item
        if (thisGroup[i,5] == 1) { # 3PL
          # grab item parameters
          logitg <- thisGroup[i,7]
          c <- thisGroup[i,8]
          a <- thisGroup[i,9:(9+dims-1)]

          spec <- rpf.drm(factors = dims)
          param <- c(unlist(a), c, logitg, logit(1))
        }
        if (thisGroup[i,5] == 2) { # graded
          # grab item parameters
          nc <- thisGroup[i,6]
          c <- rep(0,nc-1)
          for (k in 1:(nc-1)) {
            c[k] <- thisGroup[i,6+k]
          }
          a <- thisGroup[i,(6+nc):(6+nc+dims-1)]

          spec <- rpf.grm(outcomes=nc, factors=dims)
          param <- c(unlist(a), c)
        }
        if (thisGroup[i,5] == 3) { # nominal
          offset <- 6
          # grab item parameters
          nc <- thisGroup[i,offset]
          offset <- offset+1
          Tmat <- thisGroup[i,offset]
          offset <- offset+1
          if (Tmat == 0) T <- 'trend'
          if (Tmat == 1) T <- 'id'
          if (Tmat == 2) { # user-defined
            v <- matrix(0,nc*(nc-1))
            for (k in 1:(nc*(nc-1))) {
              v[k] <- thisGroup[i,offset]
              offset <- offset+1
            }
            T <- matrix(c(v), nc, nc-1, byrow=TRUE)
          }
          # scoring fn
          alf <- rep(0,nc-1)
          for (k in 1:(nc-1)) {
            alf[k] <- thisGroup[i,offset]
            offset <- offset+1
          }
          # slope
	  a <- thisGroup[i,offset:(offset+dims-1)]
	  offset <- offset+dims

          # intercept
          Lmat <- thisGroup[i,offset]
          offset <- offset+1
          if (Lmat == 0) L <- 'trend'
          if (Lmat == 1) L <- 'id'
          if (Lmat == 2) { # user-defined
            v <- matrix(0,nc*(nc-1))
            for (k in 1:(nc*(nc-1))) {
              v[k] <- thisGroup[i,offset]
              offset <- offset+1
            }
            L <- matrix(c(v), nc, nc-1, byrow=TRUE)
          }
          gam <- rep(0,nc-1)
          for (k in 1:(nc-1)) {
            gam[k] <- thisGroup[i,offset]
            offset <- offset+1
          }

          spec <- rpf.nrm(outcomes = nc, T.a = T, T.c = L)
          param <- c(a, alf, gam)
        }

        if (!is.null(spec)) {
          g.label[[i]] <- as.character(thisGroup[i,2])
          g.spec[[i]] <- spec
          g.param[[i]] <- param
        }
      } else { # group
        cov <- matrix(NA, nrow=dims, ncol=dims)
        col <- 6+dims
        for (rx in 1:dims) {
          for (cx in 1:rx) {
            cov[rx,cx] <- thisGroup[i,col]
            if (rx != cx) cov[cx,rx] <- thisGroup[i,col]
            col <- col+1
          }
        }
        g.dist <- list(mean=unlist(thisGroup[i,6:(6+dims-1)]), cov=cov)
        g.name <- thisGroup[i,2]
      }
    } # for every item

    names(g.spec) <- g.label
    pmat <- matrix(NA, ncol=length(g.param), nrow=max(vapply(g.param, length, 0)))
    for (i in 1:length(g.param)) {
      v <- g.param[[i]]
      pmat[1:length(v), i] <- v
    }
    colnames(pmat) <- g.label
    groups[[g]] <- list(spec=g.spec, param=pmat, mean=g.dist$mean, cov=g.dist$cov)
    names(groups)[[g]] <- as.character(g.name)
  }  # for every group

  groups
}

serialize.T <- function(spec, Tmat, T) {
  if (all(abs(T - Tnom.trend(spec@outcomes)) < 1e-4)) {
    c(0)
  } else if (Tmat == 'a' && all(abs(T - Tnom.ida(spec@outcomes)) < 1e-4)) {
    c(1)
  } else if (Tmat == 'c' && all(abs(T - Tnom.idc(spec@outcomes)) < 1e-4)) {
    c(1)
  } else {
    c(2, rep(0, dim(T)[1]), t(T))
  }
}

##' Write a flexMIRT PRM file
##'
##' \lifecycle{experimental}
##' This was last updated in 2017 and may no longer work.
##'
##' Formats item parameters in the way that flexMIRT expects to read
##' them.
##'
##' NOTE: Support for the graded response model may not be complete.
##' @template detail-group
##'
##' @param groups a list of groups each with items and latent parameters
##' @param file the destination file name
##' @param fileEncoding how to encode the text file (optional)
write.flexmirt <- function(groups, file=NULL, fileEncoding="") {
  if (missing(file)) {
    stop("You must specify the destination file=")
  }
  if (file == "")
    file <- stdout()
  else if (is.character(file)) {
    file <- if (nzchar(fileEncoding))
      file(file, "w", encoding = fileEncoding)
    else file(file, "w")
    on.exit(close(file))
  }
  else if (!isOpen(file, "w")) {
    open(file, "w")
    on.exit(close(file))
  }
  if (!inherits(file, "connection"))
    stop("'file' must be a character string or connection")

  for (gx in 1:length(groups)) {
    grp <- groups[[gx]]
    for (ix in 1:length(grp$spec)) {
      spec <- grp$spec[[ix]]
      nfact <- spec@factors
      name <- names(grp$spec)[ix]
      iparam <- grp$param[,ix]
      if (is.null(name)) name <- paste("i",ix,sep="")
      if (inherits(spec, "rpf.mdim.drm")) {
        if (iparam[4] < 1) warnings("Nonzero upper bound ignored")
        if (iparam[3] == 0) {
          warnings("Guessing set to 0; using Graded(2) instead")
          cat(paste(1, name, gx, nfact, 2, 2, iparam[2], iparam[1], sep="\t"),
              file=file, fill=TRUE)
        } else {
          logitg <- iparam[3]
          cat(paste(1, name, gx, nfact, 1, 2, logitg, iparam[2], iparam[1], sep="\t"),
              file=file, fill=TRUE)
        }
      } else if (inherits(spec, "rpf.mdim.grm")) {
        if (spec@outcomes > 2) stop("Not implemented")
        cat(c(1, name, gx, nfact, 2, spec@outcomes, iparam[(nfact+1):length(iparam)],
                  iparam[1:nfact]), sep="\t", file=file, fill=TRUE)
      } else if (inherits(spec, "rpf.mdim.nrm")) {
        T.a <- serialize.T(spec, 'a', getT(spec, 0))
        T.c <- serialize.T(spec, 'c', getT(spec, 1))
        cat(paste(c(1, name, gx, nfact, 3, spec@outcomes, T.a,
                    iparam[(spec@factors+1):(spec@outcomes + spec@factors - 1)],
                    iparam[1:spec@factors],
                    T.c, iparam[(spec@factors + spec@outcomes):length(iparam)]), sep="\t"),
            file=file, fill=TRUE)
      } else {
        stop(paste("Not implemented for", class(spec)))
      }
    }
    tri <- rep(NA, nfact*(nfact+1)/2)
    tx <- 1
    for (rx in 1:nfact) {
      for (cx in 1:rx) {
        tri[tx] <- grp$cov[rx,cx]
        tx <- tx+1
      }
    }
    cat(c(0, names(groups)[gx], gx, nfact, 0, grp$mean, tri),
        sep="\t", file=file, fill=TRUE)
  }
}

##' Randomly sample response patterns given a list of items
##'
##' Returns a random sample of response patterns given a list of item
##' models and parameters. If \code{grp} is given then theta, items, params,
##' mean, and cov can be omitted.
##'
##' @template detail-group
##' @name rpf.sample
##' @param theta either a vector (for 1 dimension) or a matrix (for >1
##' dimension) of person abilities or the number of response patterns
##' to generate randomly
##' @param items a list of item models
##' @param params a list or matrix of item parameters. If omitted, random item
##' parameters are generated for each item model.
##' @template arg-dots
##' @param prefix Column names are taken from param or items.
##' If no column names are available, some will be generated using
##' the given prefix.
##' @param mean mean vector of latent distribution (optional)
##' @param cov covariance matrix of latent distribution (optional)
##' @param mcar proportion of generated data to set to NA (missing completely at random)
##' @template arg-grp
##' @return Returns a data frame of response patterns
##' @examples
##' # 1 dimensional items
##' i1 <- rpf.drm()
##' i1.p <- rpf.rparam(i1)
##' i2 <- rpf.nrm(outcomes=3)
##' i2.p <- rpf.rparam(i2)
##' rpf.sample(5, list(i1,i2), list(i1.p, i2.p))
##' @seealso \code{\link{sample}}
rpf.sample <- function(theta, items, params, ..., prefix="i",
                       mean=NULL, cov=NULL, mcar=0.0, grp=NULL)
{
    garbageArguments <- list(...)
    if (length(garbageArguments) > 0) {
        stop("rpf.sample does not accept values for the '...' argument")
    }

    if (!missing(grp)) {
	    if (missing(items)) items <- grp$spec
	    if (missing(params) && !is.null(grp$param)) params <- grp$param
	    if (missing(mean) && !is.null(grp$mean)) mean <- grp$mean
	    if (missing(cov) && !is.null(grp$cov)) cov <- grp$cov
	    if (missing(theta) && !is.null(grp$data)) theta <- nrow(grp$data)
    }

  numItems <- length(items)
  maxDim <- max(vapply(items, function(i) i@factors, 0))

    if (maxDim > 1) {
	    design <- matrix(rep(1:maxDim, numItems), nrow=maxDim)
	    design[sapply(items, function(i) 1:maxDim > i@factors)] <- NA
    } else {
	    design <- matrix(rep(1, numItems), nrow=1)
    }

  maxAbilities <- max(design, na.rm=TRUE)

  numPeople <- NA
  if (is.numeric(theta) && length(theta) == 1) {
    if (theta <= 1) stop("Request at least 2 samples")
    numPeople <- theta
    if (missing(mean)) mean <- rep(0, maxAbilities)
    if (length(mean) != maxAbilities) stop(paste("Mean vector must have length",maxAbilities,"not",length(mean)))
    if (missing(cov)) cov <- diag(maxAbilities)
    if (any(dim(cov) != maxAbilities)) stop(paste("Cov matrix must be square matrices of size",maxAbilities))
    theta <- array(t(mvtnorm::rmvnorm(numPeople, mean=mean, sigma=cov)),
                   dim=c(maxAbilities, numPeople))
  } else if (maxDim == 1 && is.vector(theta)) {
    numPeople <- length(theta)
    theta <- array(theta, dim=c(maxAbilities, numPeople))
  } else {
      if (dim(theta)[1] > maxAbilities) {
          stop(paste("Only", maxAbilities, "abilities but theta provides", dim(theta)[1],
                     "-- maybe transpose theta?"))
      }
    numPeople <- dim(theta)[2]
  }

  if (missing(params)) {
    params <- lapply(items, rpf.rparam)
  }

  outcomes <- vapply(items, function(i) i@outcomes, 0)

  name <- colnames(params)
  if (is.null(name)) name <- names(items)

    maxParam <- max(sapply(items, rpf.numParam))
    if (is.matrix(params) && nrow(params) > maxParam) {
	    warning(paste("Item parameter matrix has", nrow(params) - maxParam, "extra rows"))
    }

  ret <- list()
  for (ix in 1:numItems) {
    i <- items[[ix]]
    param <- c()
    if (is.list(params)) {
      param <- params[[ix]]
    } else {
      param <- params[,ix]  # item parameters are in columns
    }

    if (length(param) < rpf.numParam(i)) {
      stop(paste("Item",class(i),"needs",rpf.numParam(i),
                 "parameters but only",length(param),"given"))
    }

    cols <- design[,ix]
    cols <- cols[!is.na(cols)]
    i.theta <- theta[cols,,drop=FALSE]
    P <- rpf.prob(i, param[1:rpf.numParam(i)], i.theta)
#    if (any(is.na(P))) stop(paste("Item", i@spec, "with param", param," produced NAs"))
    ret1 <- apply(P, 2, sample, x=1:i@outcomes, size=1, replace=F)
    labels <- 1:i@outcomes
    if (!missing(grp) && !is.null(grp$data)) labels <- levels(grp$data[1, name[ix]])
    ret1 <- factor(ret1, levels=1:i@outcomes, ordered=TRUE, labels=labels)
    attr(ret1, 'mxFactor') <- TRUE  # for OpenMx
    ret[[ix]] <- ret1
  }
  ret <- as.data.frame(ret, optional=TRUE)
  if (is.null(name)) name <- paste(prefix,1:numItems,sep="")
  colnames(ret) <- name
  if (mcar > 0) {
      size <- prod(dim(ret))
      mask <- rep(FALSE, size)
      mask[sample.int(size, size * mcar)] <- TRUE
      shaped.mask <- array(mask, dim=dim(ret))
      ret[shaped.mask] <- NA
  }
  return(ret)
}

suppressWarnings(RNGversion("3.5"))
library(testthat)
library(rpf)
library(mirt)
#options(error = utils::recover)

context("mirt ICC")

myseed <- as.integer(runif(1) * 1e7)
#print(paste("set.seed =",myseed))
set.seed(myseed)

i.count <- 5
spec <- list()

test_that("3PL", {
  spec[1:i.count] <- list(rpf.drm())
  data <- rpf.sample(100, spec)
  data <- simplify2array(lapply(data, unclass)) - 1

  suppressWarnings(fit <- mirt(data, 1, rep('3PL',i.count), D=1,
                               verbose=FALSE, technical=list(NCYCLES=1)))

  for (ix in 1:i.count) {
    ii <- extract.item(fit, ix)
    expect_equal(c(t(probtrace(ii, c(-1,0,1)))),
                 c(rpf.prob(spec[[1]], c(ii@par[1:2], ii@par[3:4]), c(-1,0,1))))
  }
})

spec[1:i.count] <- list(rpf.grm(outcomes=3, multidimensional=TRUE))

data <- rpf.sample(100, spec)
data <- simplify2array(lapply(data, unclass)) - 1

test_that("GRM", {
  suppressWarnings(fit <- mirt(data, 1, rep('graded',i.count), D=1,
                               verbose=FALSE, technical=list(NCYCLES=1)))

  for (ix in 1:i.count) {
    ii <- extract.item(fit, ix)
    if (length(ii@par) < 3) {
                                        # mirt can lose a category if it is not represented in the data
                                        #print(data[,ix])
      next
    }
    expect_equal(c(t(probtrace(ii, c(-1,0,1)))),
                 c(rpf.prob(spec[[1]], ii@par[1:3], c(-1,0,1))))
  }
})

test_that("nominal", {
  spec[1:i.count] <- list(rpf.nrm(outcomes=3,
                             T.a=rbind(0, diag(2)),
                             T.c=rbind(0, diag(2))))

  suppressWarnings(fit <- mirt(data, 1, rep('nominal',i.count),
                               verbose=FALSE, D=1, technical=list(NCYCLES=1)))
  for (ix in 1:i.count) {
    ii <- extract.item(fit, ix)
    if (length(ii@par) < 7) next
    expect_equal(c(t(probtrace(ii, c(-1,0,1)))),
                 c(rpf.prob(spec[[1]], ii@par[c(1, 3:4, 6:7)], c(-1,0,1))))
    expect_equal(c(log(t(probtrace(ii, c(-1,0,1))))),
                 c(rpf.logprob(spec[[1]], ii@par[c(1, 3:4, 6:7)], c(-1,0,1))))
  }
})

library(testthat)
library(rpf)

context("sample")

unfactor <- function(data) {
  for(s in ls(data)) {
    if (is.factor(data[[s]])) {
      data[[s]] <- unclass(data[[s]])
    }
  }
}

compare.df <- function(df1, df2) {
  df1 <- unfactor(df1)
  df2 <- unfactor(df2)
  expect_identical(dim(df1), dim(df2))
  expect_identical(df1, df2)
}

set.seed(1)

test_that("1 dimensional items", {
  i1 <- rpf.drm()
  i1.p <- rpf.rparam(i1, version=1)
  i2 <- rpf.nrm(outcomes=3)
  i2.p <- rpf.rparam(i2, version=1)
  data <- rpf.sample(3, list(i1,i2), list(i1.p, i2.p))
  compare.df(unclass(data), data.frame(i1=c(2,1,1), i2=c(3,3,3)))

  data <- rpf.sample(runif(3), list(i1,i2), list(i1.p, i2.p))
  compare.df(unclass(data), data.frame(i1=c(2,1,2), i2=c(3,2,3)))
})

test_that("multidimension, no design", {
	set.seed(1)
  numItems <- 3
  items <- vector("list", numItems)
  correct <- vector("list", numItems)

  i1 <- rpf.drm(factors=2)
  for (ix in 1:numItems) {
    items[[ix]] <- i1
    correct[[ix]] <- rpf.rparam(i1, version=1)
  }

  data <- rpf.sample(3, items, correct)
  expect_identical(c(simplify2array(data)),
                   c("2", "1", "2", "2", "1", "1", "1", "2", "2"),
                   info=paste(deparse(simplify2array(data)), collapse="\n"))
})

test_that("1d and 2d", {
  numItems <- 4
  i1 <- rpf.drm()
  i2 <- rpf.drm(factors=2)
  items <- vector("list", numItems)
  for (ix in seq(1,numItems,2)) items[[ix]] <- i1
  for (ix in seq(2,numItems,2)) items[[ix]] <- i2
  correct <- lapply(items, rpf.rparam, version=1)
  data <- rpf.sample(4, items, correct)
  expect_true(all(dim(data) == c(4,4)))
})

library(testthat)
library(OpenMx)  # an unrelease version of OpenMx is required for this test
library(rpf)
library(numDeriv)
#options(error = utils::recover)

context("dLL")

unpackHession <- function(deriv, np) {
  hess <- matrix(NA, nrow=np, ncol=np)
  dx <- np+1
  for (hr in 1:np) {
    hess[1:hr,hr] <- hess[hr,1:hr] <- deriv$D[dx:(dx+hr-1)]
    dx <- dx + hr
  }
  hess
}

#myseed <- as.integer(runif(1) * 1e7)
#print(paste("set.seed =",myseed))
#set.seed(myseed)
set.seed(2845451)

items <- list()
items[[1]] <- rpf.drm(factors=2)
items[[2]] <- rpf.grm(outcomes=3, factors=2)
T.a <- matrix(rnorm(9),3,3) + diag(3)
T.c <- matrix(rnorm(9),3,3) + diag(3)

items[[3]] <- rpf.nrm(outcomes=4, factors=2,
                      T.a=T.a, T.c=T.c)
numItems <- length(items)

data <- rpf.sample(200, items)  # small sample size will only work with some seeds

starting <- list(c(1.4, 1, 0, .1, .9),
                 c(1.4, 1, -.5, -1),
                 c(1.4,  1,  rep(0,6)))
starting.len <- max(vapply(starting, length, 0))

ip.mat <- mxMatrix(name="itemParam", nrow=starting.len, ncol=numItems,
                   values=0, free=FALSE)

for (sx in 1:length(starting)) {
  v <- starting[[sx]]
  ip.mat@values[1:length(v),sx] <- v
  ip.mat@free[1:length(v),sx] <- TRUE
}
starting.mat <- ip.mat@values
starting.free <- ip.mat@free

m.mat <- mxMatrix(name="mean", nrow=1, ncol=2, values=0, free=FALSE)
cov.mat <- mxMatrix(name="cov", nrow=2, ncol=2, values=diag(2), free=FALSE)
m2 <- mxModel(model="drm1", ip.mat, m.mat, cov.mat,
              mxData(observed=data, type="raw"),
              mxExpectationBA81(mean="mean", cov="cov",
                                ItemSpec=items,
                                ItemParam="itemParam", EItemParam=starting.mat,
#                                design=matrix(c(1L,2L,1L,2L,1L,2L,1L,NA),nrow=2),
                qpoints=13),
              mxFitFunctionML(),
              mxComputeSequence(steps=list(
                                  mxComputeOnce('expectation', context='EM'),
                                  mxComputeOnce('fitfunction', gradient=TRUE, hessian=TRUE)
                                  )))

spoint <- list(c(1.4, 1, 0, .1, .9),
               c(1.4, 1, .5, -.5),
               c(0.89,  0.33,  0.05,  0.18,  0.07, -2.03,  0.17,  0.26))
spoint.len <- vapply(spoint, length, 0)

for (ii in 1:numItems) {
  test_that(paste("item",ii), {
    np <- length(spoint[[ii]])
    m2@matrices$itemParam@free[,] <- FALSE
    m2@matrices$itemParam@values <- starting.mat
    m2@matrices$itemParam@values[1:np,ii] <- spoint[[ii]]
    m2@matrices$itemParam@free[,ii] <- starting.free[,ii]
    m2 <- mxRun(m2, silent=TRUE)
    
    grad1 <- m2@output$gradient
    names(grad1) <- NULL
    hess <- m2@output$hessian

    deriv <- genD(function(param) {
      np <- length(param)
      m2@matrices$itemParam@values[1:np,ii] <- param
      lModel <- mxModel(m2,
                        mxComputeSequence(steps=list(
                                            mxComputeOnce('expectation', context='EM'),
                                            mxComputeOnce('fitfunction')
                                            )))
      fit <- mxRun(lModel, silent=TRUE)
      got <- fit@fitfunction@result
      got
    }, spoint[[ii]], method.args=list(eps=0.01, d=0.01, r=2))

    emp.hess <- unpackHession(deriv, np)
#    emp.hess[is.na(emp.hess)] <- 0

    if (0) {
      print(paste("Item", ii))
      print(grad1)
      print(deriv$D[1:np])
      cat("T.a=",deparse(T.a),"\n")
      cat("T.c=",deparse(T.c),"\n")
      cat("an=",deparse(hess),"\n")
      cat("emp=",deparse(emp.hess),"\n")
      print(round(hess - emp.hess, 2))
    }
    
    expect_equal(deriv$D[1:np], grad1, tolerance=1e-6)
    expect_equal(emp.hess, hess, tolerance=1e-4)
  })
}
#warnings()

library(rpf)
library(testthat)

context("null")

test_that("error check", {
  bad <- rpf.drm(factors=0)
  bad@spec <- 1
  expect_error(rpf.numSpec(bad),
               "Item spec must be of length 3")
})

test_that("param info", {
  ans1 <-structure(list(type = "intercept", upper = NA_real_, lower = NA_real_),
                   .Names = c("type",  "upper", "lower"))
  expect_identical(rpf.paramInfo(rpf.drm(factors=0)), ans1)
  
  ans2 <- structure(list("intercept", NA_real_, NA_real_, "intercept",      NA_real_, NA_real_),
                    .Dim = c(3L, 2L), .Dimnames = list(c("type",  "upper", "lower"), NULL))
  expect_identical(rpf.paramInfo(rpf.grm(outcomes=3, factors=0)), ans2)
  
  ans3 <- structure(list("intercept", NA_real_, NA_real_,
                         "intercept",      NA_real_, NA_real_,
                         "intercept", NA_real_, NA_real_),
                    .Dim = c(3L,  3L), .Dimnames = list(c("type", "upper", "lower"), NULL))
  expect_identical(rpf.paramInfo(rpf.nrm(outcomes=4, factors=0)), ans3)
})

spec <- list()
param <- list()
spec [[length(spec) +1]] <- rpf.drm(factors = 0)
param[[length(param)+1]] <- c(0)

spec [[length(spec) +1]] <- rpf.grm(outcomes=3, factors=0)
param[[length(param)+1]] <- c(1, -1)

spec [[length(spec) +1]] <- rpf.nrm(outcomes=3, factors=0)
param[[length(param)+1]] <- c(0, -.6)

spec1 <- lapply(spec, rpf.modify, 1)
param1 <- list(c(1,param[[1]],logit(0), logit(1)),
               c(1,param[[2]]),
               c(1,1,0,param[[3]]))

for (ix in 1:length(spec)) {
  test_that(class(spec[[ix]]), {
    expect_equal(rpf.prob(spec[[ix]], param[[ix]], NULL),
                 rpf.prob(spec1[[ix]], param1[[ix]], 0))
    expect_equal(rpf.logprob(spec[[ix]], param[[ix]], NULL),
                 rpf.logprob(spec1[[ix]], param1[[ix]], 0))
    rp <- rpf.rparam(spec[[ix]])
    expect_equal(length(rp), rpf.numParam(spec[[ix]]))
  })
}

test_that("sample null", {
  set.seed(1)
  got <- rpf.sample(3, spec, param)
  got <- sapply(got, unclass)
  colnames(got) <- NULL
  ans <- structure(c(1L, 1L, 1L, 2L, 2L, 2L, 3L, 1L, 3L),
                   .Dim = c(3L,  3L), .Dimnames = list(NULL, NULL))
  expect_equal(got, ans)
})

test_that("null dLL drm", {
  for (rep in 1:3) {
    w <- rchisq(2, 50)
    d0 <- rpf.dLL(spec[[1]], param[[1]], NULL, w)
    d1 <- rpf.dLL(spec1[[1]], param1[[1]], 0, w)
    expect_equal(d0[1], d1[2])
    expect_equal(d0[2], d1[7])
  }
})

test_that("null dLL grm", {
  for (rep in 1:3) {
    w <- rchisq(3, 50)
    item <- 2
    d0 <- rpf.dLL(spec[[item]], param[[item]], NULL, w)
    d1 <- rpf.dLL(spec1[[item]], param1[[item]], 0, w)
    expect_equal(d0[1], d1[2])
    expect_equal(d0[2], d1[3])
    expect_equal(d0[3], d1[6])
    expect_equal(d0[4], d1[8])
    expect_equal(d0[5], d1[9])
  }
})

test_that("null dLL nrm", {
  for (rep in 1:3) {
    w <- rchisq(3, 50)
    item <- 3
    d0 <- rpf.dLL(spec[[item]], param[[item]], NULL, w)
    d1 <- rpf.dLL(spec1[[item]], param1[[item]], 0, w)
    expect_equal(d0[1], d1[4])
    expect_equal(d0[2], d1[5])
    expect_equal(d0[3], d1[15])
    expect_equal(d0[4], 0)
    expect_equal(d0[5], d1[20])
  }
})

##' Calculate cell central moments
##'
##' Popular central moments include 2 (variance) and 4 (kurtosis).
##'
##' @param spec list of item models
##' @param params data frame of item parameters, 1 per row
##' @param scores model derived person scores
##' @param m which moment
##' @return moment matrix
##' @docType methods
rpf.1dim.moment <- function(spec, params, scores, m) {
  out <- array(dim=c(length(scores), length(spec)))
  for (ix in 1:length(spec)) {
    i <- spec[[ix]]
    prob <- t(rpf.prob(i, params[,ix], scores))  # remove t() TODO
    Escore <- apply(prob, 1, function(r) sum(r * 0:(i@outcomes-1)))
    grid <- t(array(0:(i@outcomes-1), dim=c(i@outcomes, length(scores))))
    out[,ix] <- apply((grid - Escore)^m * prob, 1, sum)
  }
  out
}

##' Calculate residuals
##'
##' @param spec list of item models
##' @param params data frame of item parameters, 1 per row
##' @param responses persons in rows and items in columns
##' @param scores model derived person scores
##' @return residuals
##' @docType methods
rpf.1dim.residual <- function(spec, params, responses, scores) {
  Zscore <- array(dim=c(length(scores), length(spec)))
  for (ix in 1:length(spec)) {
    i <- spec[[ix]]
    prob <- t(rpf.prob(i, params[,ix], scores))  # remove t() TODO
    Escore <- apply(prob, 1, function(r) sum(r * 0:(i@outcomes-1)))
    data <- responses[,ix]
    if (is.ordered(data)) {
      data <- unclass(data) - 1
    } else if (is.factor(data)) {
      stop(paste("Column",ix,"is an unordered factor"))
    }
    if (length(data) != length(Escore)) stop("Length mismatch")
    Zscore[,ix] <- data - Escore
  }
  Zscore
}

##' Calculate standardized residuals
##'
##' @param spec list of item models
##' @param params data frame of item parameters, 1 per row
##' @param responses persons in rows and items in columns
##' @param scores model derived person scores
##' @return standardized residuals
##' @docType methods
rpf.1dim.stdresidual <- function(spec, params, responses, scores) {
  res <- rpf.1dim.residual(spec, params, responses, scores)
  variance <- rpf.1dim.moment(spec, params, scores, 2)
  res / sqrt(variance)
}

##' Calculate item and person Rasch fit statistics
##'
##' Note: These statistics are only appropriate if all discrimination
##' parameters are fixed equal and items are conditionally independent
##' (see \code{\link{ChenThissen1997}}).  A best effort is made to
##' cope with missing data.
##'
##' Exact distributional properties of these statistics are unknown
##' (Masters & Wright, 1997, p. 112).  For details on the calculation,
##' refer to Wright & Masters (1982, p. 100).
##'
##' The Wilson-Hilferty transformation is biased for less than 25 items.
##' Consider wh.exact=FALSE for less than 25 items.
##'
##' @template detail-group
##'
##' @param spec list of item response models \lifecycle{deprecated}
##' @param params matrix of item parameters, 1 per column \lifecycle{deprecated}
##' @param responses persons in rows and items in columns \lifecycle{deprecated}
##' @param scores model derived person scores \lifecycle{deprecated}
##' @param margin for people 1, for items 2
##' @param wh.exact whether to use the exact Wilson-Hilferty transformation
##' @param group spec, params, data, and scores can be provided in a list instead of as arguments
##' @template detail-group
##'
##' @references Masters, G. N. & Wright, B. D. (1997). The Partial
##' Credit Model. In W. van der Linden & R. K. Kambleton (Eds.),
##' \emph{Handbook of modern item response theory}
##' (pp. 101-121). Springer.
##'
##' Wilson, E. B., & Hilferty, M. M. (1931). The distribution of
##' chi-square. \emph{Proceedings of the National Academy of Sciences of the
##' United States of America,} 17, 684-688.
##'
##' Wright, B. D. & Masters, G. N. (1982). \emph{Rating Scale
##' Analysis.} Chicago: Mesa Press.
##' @family diagnostic
##' @examples
##' data(kct)
##' responses <- kct.people[,paste("V",2:19, sep="")]
##' rownames(responses) <- kct.people$NAME
##' colnames(responses) <- kct.items$NAME
##' scores <- kct.people$MEASURE
##' params <- cbind(1, kct.items$MEASURE, logit(0), logit(1))
##' rownames(params) <- kct.items$NAME
##' items<-list()
##' items[1:18] <- rpf.drm()
##' params[,2] <- -params[,2]
##' rpf.1dim.fit(items, t(params), responses, scores, 2, wh.exact=TRUE)
rpf.1dim.fit <- function(spec, params, responses, scores, margin, group=NULL, wh.exact=TRUE) {
  if (missing(margin)) stop("Which margin?")
  if (!missing(group)) {
        spec <- group$spec
        params <- group$param
        responses <- group$data
        scores <- group$score[,1]  # should not assume first score TODO
  }

  if (any(is.na(responses))) warning("Rasch fit statistics should not be used with missing data")  # true? TODO

  if (dim(params)[2] != length(spec)) {
    stop(paste("spec is length", length(spec), "but param has",
               dim(params)[2], "columns"))
  }
  if (nrow(responses) != length(scores)) {
    stop(paste("nrow(responses) is", nrow(responses), "but",length(scores),
               "factor scores"))
  }

  if (dim(params)[2] < 25 && wh.exact) {
    if (missing(wh.exact)) {
      wh.exact <- FALSE
      warning("Consider wh.exact=FALSE for less than 25 items")
    }
  }
  if (dim(params)[2] > 25 && !wh.exact) {
    if (missing(wh.exact)) {
      wh.exact <- TRUE
      warning("Consider wh.exact=TRUE for more than 25 items")
    }
  }

  exclude.col <- c()
  outcomes <- sapply(spec, function(s) s@outcomes)
  for (ix in 1:dim(responses)[2]) {
    kat <- sum(table(responses[,ix]) > 0)
    if (kat != outcomes[ix]) {
      exclude.col <- c(exclude.col, ix)
      warning(paste("Excluding item", colnames(responses)[ix], "because outcomes !=", outcomes[ix]))
    }
  }

  if (length(exclude.col)) {
    responses <- responses[,-exclude.col]
    spec <- spec[-exclude.col]
    params <- params[,-exclude.col]
    outcomes <- outcomes[-exclude.col]
  }

  exclude.row <- c()
  for (ix in 1:dim(responses)[1]) {
    r1 <- sapply(responses[ix,], unclass)
    if (any(is.na(r1))) next
    if (all(r1 == 1) || all(r1 == outcomes)) {
      exclude.row <- c(exclude.row, ix)
      warning(paste("Excluding response", rownames(responses)[ix], "because it is a minimum or maximum"))
    }
  }

  if (length(exclude.row)) {
    responses <- responses[-exclude.row,]
    scores <- scores[-exclude.row]
  }

  na.rm=TRUE
  r.z <- rpf.1dim.stdresidual(spec, params, responses, scores)
  r.var <- rpf.1dim.moment(spec, params, scores,2)
  r.var[is.na(r.z)] <- NA
  r.k <- rpf.1dim.moment(spec, params, scores,4)
  r.k[is.na(r.z)] <- NA

  outfit.var <- r.var
  outfit.var[r.var^2 < 1e-5] <- sqrt(1e-5)
  outfit.n <- apply(r.var, margin, function(l) sum(!is.na(l)))
  outfit.sd <- sqrt(apply(r.k / outfit.var^2, margin, sum, na.rm=na.rm) /
                    outfit.n^2 - 1/outfit.n)
  outfit.sd[outfit.sd > 1.4142] <- 1.4142
  outfit.fudge <- outfit.sd/3

  infit.sd <- sqrt(apply(r.k - r.var^2, margin, sum, na.rm=na.rm)/
    apply(r.var, margin, sum, na.rm=na.rm)^2)
  infit.sd[infit.sd > 1.4142] <- 1.4142
  infit.fudge <- infit.sd/3
  if (!wh.exact) {
    infit.fudge <- 0
    outfit.fudge <- 0
  }

  outfit <- apply(r.z^2, margin, sum, na.rm=na.rm)/
                       apply(r.z, margin, function (l) sum(!is.na(l)))
  outfit.z <- (outfit^(1/3) - 1)*(3/outfit.sd) + outfit.fudge

  infit <- apply(r.z^2 * r.var, margin, sum, na.rm=na.rm)/
                     apply(r.var, margin, sum, na.rm=na.rm)
  infit.z <- (infit^(1/3) - 1)*(3/infit.sd) + infit.fudge

  df <- data.frame(n=outfit.n, infit, infit.z, outfit, outfit.z)
  if (margin == 2) {
    df$name <- colnames(params)
  } else {
    df$name <- rownames(responses)
  }
  df
}

##' Find the point where an item provides mean maximum information
##'
##' \lifecycle{experimental}
##'
##' @param spec an item spec
##' @param iparam an item parameter vector
##' @param grain the step size for numerical integration (optional)
rpf.mean.info1 <- function(spec, iparam, grain=.1) {
  range <- 9
  dim <- spec@factors
  if (dim != 1) stop("Not implemented")
  grid <- seq(-range, range, grain)
  info <- rpf.info(spec, iparam, grid)
  sum(info * grid) / sum(info)
}

##' Find the point where an item provides mean maximum information
##'
##' \lifecycle{experimental}
##' This is a point estimate of the mean difficulty of items that do
##' not offer easily interpretable parameters such as the Generalized
##' PCM. Since the information curve may not be unimodal, this
##' function integrates across the latent space.
##'
##' @param spec list of item specs
##' @param param list or matrix of item parameters
##' @param grain the step size for numerical integration (optional)
rpf.mean.info <- function(spec, param, grain=.1) {
  ret <- list()
  for (ix in 1:length(spec)) {
    iparam <- c()
    if (is.list(param)) {
      iparam <- param[[ix]]
    } else {
      iparam <- param[ix,]
    }
    ret[[ix]] <- rpf.mean.info1(spec[[ix]], iparam, grain)
  }
  ret
}

# copied from mirt
collapseCells <- function(On, En, mincell = 1){
		drop <- which(rowSums(is.na(En)) > 0)
		En[is.na(En)] <- 0
					#collapse known upper and lower sparce cells
		if(length(drop) > 0L){
			up <- drop[1L]:drop[length(drop)/2]
			low <- drop[length(drop)/2 + 1L]:drop[length(drop)]
			En[max(up)+1, ] <- colSums(En[c(up, max(up)+1), , drop = FALSE])
			On[max(up)+1, ] <- colSums(On[c(up, max(up)+1), , drop = FALSE])
			En[min(low)-1, ] <- colSums(En[c(low, min(low)-1), , drop = FALSE])
			On[min(low)-1, ] <- colSums(On[c(low, min(low)-1), , drop = FALSE])
			En[c(up, low), ] <- On[c(up, low), ] <- NA
			En <- na.omit(En)
			On <- na.omit(On)
		}
					#collapse accross
		if(ncol(En) > 2L){
			for(j in 1L:(ncol(On)-1L)){
				L <- En < mincell
				sel <- L[,j]
				if(!any(sel)) next
				On[sel, j+1L]  <- On[sel, j] + On[sel, j+1L]
				En[sel, j+1L]  <- En[sel, j] + En[sel, j+1L]
				On[sel, j] <- En[sel, j] <- NA
			}
			sel <- L[,j+1L]
			sel[rowSums(is.na(En[, 1L:j])) == (ncol(En)-1L)] <- FALSE
			put <- apply(En[sel, 1L:j, drop=FALSE], 1, function(x) max(which(!is.na(x))))
			put2 <- which(sel)
			for(k in 1L:length(put)){
				En[put2[k], put[k]] <- En[put2[k], put[k]] + En[put2[k], j+1L]
				En[put2[k], j+1L] <- On[put2[k], j+1L] <- NA
			}
		}
		L <- En < mincell
		L[is.na(L)] <- FALSE
		while(any(L)){
			drop <- c()
			for(j in 1L:(nrow(On)-1L)){
				if(any(L[j,])) {
					On[j+1L, L[j,]] <- On[j+1L, L[j,]] + On[j, L[j,]]
					En[j+1L, L[j,]] <- En[j+1L, L[j,]] + En[j, L[j,]]
					drop <- c(drop, j)
					break
				}
			}
			for(j in nrow(On):2L){
				if(any(L[j,])) {
					On[j-1L, L[j,]] <- On[j-1L, L[j,]] + On[j, L[j,]]
					En[j-1L, L[j,]] <- En[j-1L, L[j,]] + En[j, L[j,]]
					drop <- c(drop, j)
					break
				}
			}
			if(nrow(On) > 4L){
				for(j in 2L:(nrow(On)-1L)){
					if(any(L[j,])){
						On[j+1L, L[j,]] <- On[j+1L, L[j,]] + On[j, L[j,]]
						En[j+1L, L[j,]] <- En[j+1L, L[j,]] + En[j, L[j,]]
						drop <- c(drop, j)
						break
					}
				}
			}
					#drop
			if(!is.null(drop)){
				En <- En[-drop, ]
				On <- On[-drop, ]
			}
			L <- En < mincell
			L[is.na(L)] <- FALSE
		}
        return(list(O=On, E=En))
}

SitemFit1Internal <- function(out) {
	observed <- out$orig.observed
	expected <- out$orig.expected

    mask <- apply(observed, 1, sum) != 0
    observed = observed[mask,,drop=FALSE]
    expected = expected[mask,,drop=FALSE]
	if (!length(observed)) {
		out$statistic <- NA
		out$pval <- NA
		return(out)
	}

	method <- out$method
    if (method == "pearson") {
	if (out$alt) {
		kc <- collapseCells(observed, expected)
	} else {
		kc <- .Call('_rpf_collapse', observed, expected, 1.0)
	}
        out$observed <- observed <- kc$O
        out$expected <- expected <- kc$E
        mask <- !is.na(expected) & expected!=0
        out$statistic <- sum((observed[mask] - expected[mask])^2 / expected[mask])
	    out$df <- nrow(observed) * (ncol(observed) - 1)
        out$df <- out$df - out$free - sum(is.na(expected));
	out$df[out$df < 1] <- 1
        out$pval <- pchisq(out$statistic, out$df, lower.tail=FALSE, log.p=log)
    } else if (method == "rms") {
      pval <- crosstabTest(observed, expected)
        out$observed <- observed
        out$expected <- expected
        if (log) {
            out$pval <- log(pval)
        } else {
            out$pval <- pval
        }
    } else {
        stop(paste("Method", method, "not recognized"))
    }
    out
}

##' Compute the S fit statistic for 1 item
##'
##' Implements the Kang & Chen (2007) polytomous extension to
##' S statistic of Orlando & Thissen (2000). Rows with
##' missing data are ignored, but see the \code{omit} option.
##'
##' This statistic is good at finding a small number of misfitting
##' items among a large number of well fitting items. However, be
##' aware that misfitting items can cause other items to misfit.
##'
##' Observed tables cannot be computed when data is
##' missing. Therefore, you can optionally omit items with the
##' greatest number of responses missing relative to the item of
##' interest.
##'
##' Pearson is slightly more powerful than RMS in most cases I
##' examined.
##'
##' Setting \code{alt} to \code{TRUE} causes the tables to match
##' published articles. However, the default setting of \code{FALSE}
##' probably provides slightly more power when there are less than 10
##' items.
##'
##' The name of the test, "S", probably stands for sum-score.
##'
##' @template detail-group
##'
##' @template arg-grp
##' @param item the item of interest
##' @param free the number of free parameters involved in estimating the item (to adjust the df)
##' @template arg-dots
##' @param method whether to use a pearson or rms test
##' @param log whether to return p-values in log units
##' @param qwidth  \lifecycle{deprecated}
##' @param qpoints \lifecycle{deprecated}
##' @param alt whether to include the item of interest in the denominator
##' @param omit number of items to omit or a character vector with the names of the items to omit when calculating the observed and expected sum-score tables
##' @param .twotier whether to enable the two-tier optimization
##' @family diagnostic
##' @references Kang, T. and Chen, T. T. (2007). An investigation of
##' the performance of the generalized S-Chisq item-fit index for
##' polytomous IRT models. ACT Research Report Series.
##'
##' Orlando, M. and Thissen, D. (2000). Likelihood-Based
##' Item-Fit Indices for Dichotomous Item Response Theory Models.
##' \emph{Applied Psychological Measurement, 24}(1), 50-64.
SitemFit1 <- function(grp, item, free=0, ..., method="pearson", log=TRUE, qwidth=6, qpoints=49L,
		      alt=FALSE, omit=0L, .twotier=TRUE) {
	if (length(list(...)) > 0) {
		stop(paste("Remaining parameters must be passed by name", deparse(list(...))))
	}
  if (!missing(qwidth) || !missing(qpoints)) complainAboutQuadSpec()

    spec <- grp$spec
  c.spec <- lapply(spec, function(m) {
    if (length(m@spec)==0) { stop("Item model",m,"is not implemented") }
    else { m@spec }
  })

    param <- grp$param
    itemIndex <- which(item == colnames(param))

	mask <- rep(TRUE, ncol(param))
	if (!alt) mask[itemIndex] <- FALSE
	omitted <- NULL
	if (is.null(omit)) {
		# OK
	} else if (is.numeric(omit)) {
		omitted <- bestToOmit(grp, omit, item)
	} else if (is.character(omit)) {
		omitted <- omit
	} else {
		stop(paste("Not clear how to interpret omit =", omit))
	}
	mask[match(omitted, colnames(grp$param))] <- FALSE
	iobss <- itemOutcomeBySumScore(grp, mask, itemIndex)
	observed <-iobss$table

  max.param <- max(vapply(spec, rpf.numParam, 0))
  if (nrow(param) < max.param) {
    stop(paste("param matrix must have", max.param ,"rows"))
  }

    Eproportion <- ot2000md(grp, itemIndex, alt, mask, .twotier)
    if (nrow(Eproportion) != nrow(observed)) {
	    print(Eproportion)
	    stop(paste("Expecting", nrow(observed), "rows in expected matrix"))
    }
    Escale <- matrix(apply(observed, 1, sum), nrow=nrow(Eproportion), ncol=ncol(Eproportion))
    expected <- Eproportion * Escale
	names(dimnames(observed)) <- c("sumScore", "outcome")
	dimnames(expected) <- dimnames(observed)
    out <- list(orig.observed=observed, orig.expected = expected,
		log=log, method=method, n=iobss$n, free=free, alt=alt, omitted=omitted)

	out <- SitemFit1Internal(out)
	out
}

ot2000md <- function(grp, item, alt=FALSE, mask, .twotier) {
	.Call('_rpf_ot2000', grp, item, alt, mask, .twotier)
}

##' Compute the S fit statistic for a set of items
##'
##' Runs \code{\link{SitemFit1}} for every item and accumulates
##' the results.
##'
##' @template detail-group
##'
##' @template arg-grp
##' @template arg-dots
##' @param method whether to use a pearson or rms test
##' @param log whether to return p-values in log units
##' @param qwidth \lifecycle{deprecated}
##' @param qpoints \lifecycle{deprecated}
##' @param alt whether to include the item of interest in the denominator
##' @param omit number of items to omit (a single number) or a list of the length the number of items
##' @param .twotier whether to enable the two-tier optimization
##' @param .parallel whether to take advantage of multiple CPUs (default TRUE)
##' @return
##' a list of output from \code{\link{SitemFit1}}
##' @family diagnostic
##' @examples
##' grp <- list(spec=list())
##' grp$spec[1:20] <- list(rpf.grm())
##' grp$param <- sapply(grp$spec, rpf.rparam)
##' colnames(grp$param) <- paste("i", 1:20, sep="")
##' grp$mean <- 0
##' grp$cov <- diag(1)
##' grp$free <- grp$param != 0
##' grp$data <- rpf.sample(500, grp=grp)
##' SitemFit(grp)
SitemFit <- function(grp, ..., method="pearson", log=TRUE, qwidth=6, qpoints=49L,
		     alt=FALSE, omit=0L, .twotier=TRUE, .parallel=TRUE) {
	if (length(list(...)) > 0) {
		stop(paste("Remaining parameters must be passed by name", deparse(list(...))))
	}

  if (!missing(qwidth) || !missing(qpoints)) complainAboutQuadSpec()

    spec <- grp$spec
    param <- grp$param
    if (ncol(param) != length(spec)) stop("Dim mismatch between param and spec")
    if (is.null(colnames(param))) stop("grp$param must have column names")

	.la <- ifelse(.parallel, mclapply, lapply)
	got <- .la(1:length(spec), function(interest) {
		free <- 0
		if (!is.null(grp$free)) free <- sum(grp$free[,interest])
		itemname <- colnames(param)[interest]
		omit1 <- omit
		if (is.list(omit)) {
			omit1 <- omit[[interest]]
		}
		ot.out <- SitemFit1(grp, itemname, free, method=method, log=log,
				    alt=alt, omit=omit1, .twotier=.twotier)
		ot.out
	})
	lapply(got, function(d1) {
		if (inherits(d1, "try-error")) stop(d1)
	})
	names(got) <- colnames(param)
	class(got) <- "summary.SitemFit"
	got
}

"+.summary.SitemFit" <- function(e1, e2) {
	e2name <- deparse(substitute(e2))
	if (!inherits(e2, "summary.SitemFit")) {
		stop("Don't know how to add ", e2name, " to a SitemFit",
		     call. = FALSE)
	}
	if (length(e1) != length(e2)) {
		stop("Cannot combine two groups with a different number of items")
	}
	if (any(names(e1) != names(e2))) {
		stop("Cannot combine two groups with a different items")
	}
	if (!all(mapply(function(i1,i2){ isTRUE(all.equal(i1$omitted, i2$omitted)) }, e1, e2))) {
		stop("Cannot combine two groups with different omitted items")
	}

	got <- mapply(function(i1, i2){
		ii <- list(orig.observed = i1$orig.observed + i2$orig.observed,
			   orig.expected = i1$orig.expected + i2$orig.expected,
			   log = i1$log,
			   method = i1$method,
			   n = i1$n + i2$n,
			   free = max(i1$free, i2$free),
			   alt = i1$alt)
		SitemFit1Internal(ii)
	}, e1, e2, SIMPLIFY=FALSE)

	class(got) <- "summary.SitemFit"
	got
}

print.summary.SitemFit <- function(x,...) {
	cat("Orlando & Thissen (2000) sum-score based item fit test\n")
	cat("  Magnitudes larger than abs(log(.01))=4.6 are significant at the p=.01 level\n\n")
	width <- max(sapply(names(x), nchar))
	fmt <- paste("%", width, "s : n = %4d, ", sep="")
	for (ix in 1:length(x)) {
		report1 <- x[[ix]]
		msg <- sprintf(fmt, names(x)[ix], report1$n)
		stat <- round(report1$statistic, 2)
		if (report1$method == "pearson") {
			msg <- paste(msg, sprintf("S-X2(%3d) = %6.2f, ", report1$df, stat), sep="")
		} else if (report1$method == "rms") {
			msg <- paste(msg, "MS=", stat, ", ", sep="")
		} else {
			stop(report1$method)
		}
		if (report1$log) {
			msg <- paste(msg, "log(p) = ", round(report1$pval, 2), sep="")
		} else {
			msg <- paste(msg, "p = ", round(report1$pval, 4), sep="")
		}
		msg <- paste(msg, "\n", sep="")
		cat(msg)
	}
}

##' Compute the ordinal gamma association statistic
##'
##' @param mat a cross tabulation matrix
##' @references
##' Agresti, A. (1990). Categorical data analysis. New York: Wiley.
##' @examples
##' # Example data from Agresti (1990, p. 21)
##' jobsat <- matrix(c(20,22,13,7,24,38,28,18,80,104,81,54,82,125,113,92), nrow=4, ncol=4)
##' ordinal.gamma(jobsat)
ordinal.gamma <- function(mat) .Call(`_rpf_gamma_cor`, mat)

# root mean squared statistic (sqrt omitted)
ms <- function(observed, expected, draws) {
  draws * sum((observed - expected)^2)
}

P.cdf.fn <- function(x, g.var, t) {
  got <- sapply(t, function (t1) {
    n <- length(g.var)
    num <- exp(1-t1) * exp(1i * t1 * sqrt(n))
    den <- pi * (t1 - 1/(1-1i*sqrt(n)))
    pterm <- prod(sqrt(1 - 2*(t1-1)*g.var/x + 2i*t1*g.var*sqrt(n)/x))
    Im(num / (den * pterm))
  })
#  print(cbind(t,got))
  got
}

##' Compute the P value that the observed and expected tables come from the same distribution
##'
##' \lifecycle{experimental}
##' This test is an alternative to Pearson's X^2
##' goodness-of-fit test.  In contrast to Pearson's X^2, no ad hoc cell
##' collapsing is needed to avoid an inflated false positive rate
##' in situations of sparse cell frequences.
##' The statistic rapidly converges to the Monte-Carlo estimate
##' as the number of draws increases.
##'
##' @param observed observed matrix
##' @param expected expected matrix
##' @return The P value indicating whether the two tables come from
##' the same distribution. For example, a significant result (P <
##' alpha level) rejects the hypothesis that the two matrices are from
##' the same distribution.
##' @references Perkins, W., Tygert, M., & Ward, R. (2011). Computing
##' the confidence levels for a root-mean-square test of
##' goodness-of-fit. \emph{Applied Mathematics and Computations,
##' 217}(22), 9072-9084.
##' @examples
##' draws <- 17
##' observed <- matrix(c(.294, .176, .118, .411), nrow=2) * draws
##' expected <- matrix(c(.235, .235, .176, .353), nrow=2) * draws
##' ptw2011.gof.test(observed, expected)  # not signficiant

ptw2011.gof.test <- function(observed, expected) {
  orig.draws <- sum(observed)
  oeDiff <- abs(sum(expected) - orig.draws)
  if (is.na(oeDiff) || oeDiff > 1e-6) {
	  warning(paste("Total observed - total expected", oeDiff))
    return(NA)
  }
  if (any(c(expected)==0)) {
	  zeros <- sum(c(expected)==0)
	  warning(paste("There are", zeros, "zeros in the expected distribution.",
		     "Did you swap the observed and expected arguments"))
	  return(NA)
  }
  observed <- observed / orig.draws
  expected <- expected / orig.draws

  X <- ms(observed, expected, orig.draws)
  if (X == 0) return(1)

  n <- length(c(observed))
  D <- diag(1/c(expected))
  P <- matrix(-1/n, n,n)
  diag(P) <- 1 - 1/n
  B <- P %*% D %*% P
  g.var <- 1 / eigen(B, only.values=TRUE)$values[-n]

  # Eqn 8 needs n variances, but matrix B (Eqn 6) only has n-1 non-zero
  # eigenvalues. Perhaps n-1 degrees of freedom?

# debugging:
#  plot(function(t) P.cdf.fn(X, g.var, t), 0, 40)

  # 310 points should be good enough for 500 bins
  # Perkins, Tygert, Ward (2011, p. 10)

# If integration tolerance is too large, non-convergence can result,
# http://r.789695.n4.nabble.com/Need-help-to-understand-integrate-function-td2322093.html
  got <- try(integrate(function(t) P.cdf.fn(X, g.var, t), 0, 40, subdivisions=310L, rel.tol=1e-10), silent=TRUE)
  if (inherits(got, "try-error")) return(NA)
  p.value <- 1 - got$value
  smallest <- 6.3e-16  # approx exp(-35)
  if (p.value < smallest) p.value <- smallest
  p.value
}

CT1997Internal1 <- function(info, method) {
	observed <- info$orig.observed
	expected <- info$orig.expected

	s <- ordinal.gamma(observed) - ordinal.gamma(expected)
	if (!is.finite(s) || is.na(s) || s==0) s <- 1
	info <- c(info, sign=sign(s), gamma=s)

	if (method == "pearson") {
		kc <- .Call('_rpf_collapse', observed, expected, 1.0)
		observed <- kc$O
		expected <- kc$E
		mask <- !is.na(expected)
		x2 <- sum((observed[mask] - expected[mask])^2 / expected[mask])
		df <- prod(dim(observed)-1) - kc$collapsed
		if (df < 1L) df <- 1L
		info <- c(info, list(statistic=x2, df=df, observed=observed, expected=expected))
	} else if (method == "lr") {
		mask <- observed > 0
		g2 <- -2 * sum(observed[mask] * log(expected[mask] / observed[mask]))
		df <- prod(dim(observed)-1)
		info <- c(info, statistic=g2, df=df)
	}
	info
}

CT1997Internal2 <- function(inames, detail) {
	cnames <- inames[-length(inames)]
	gamma <- matrix(NA, length(inames), length(cnames))
	dimnames(gamma) <- list(inames, cnames)
	raw <- matrix(NA, length(inames), length(cnames))
	dimnames(raw) <- list(inames, cnames)
	std <- matrix(NA, length(inames), length(cnames))
	dimnames(std) <- list(inames, cnames)
	pval <- matrix(NA, length(inames), length(cnames))
	dimnames(pval) <- list(inames, cnames)

	px <- 1L
	for (iter1 in 2:length(inames)) {
		for (iter2 in 1:(iter1-1)) {
			d1 <- detail[[px]]
			gamma[iter1, iter2] <- d1$gamma
			s <- d1$sign
			stat <- d1$statistic
			df <- d1$df
			raw[iter1, iter2] <- stat
			std[iter1, iter2] <- s * abs((stat - df)/sqrt(2*df))
			pval[iter1, iter2] <- s * -pchisq(stat, df, lower.tail=FALSE, log.p=TRUE)
			px <- px + 1L
		}
	}

	retobj <- list(pval=pval[-1,,drop=FALSE], std=std[-1,,drop=FALSE],
		       raw=raw[-1,,drop=FALSE], gamma=gamma[-1,,drop=FALSE], detail=detail)
	class(retobj) <- "summary.ChenThissen1997"
	retobj
}

tableWithWeights <- function(colpair, weights) {
	if (length(colpair) != 2) stop("Not a pair")
	l1 <- levels(colpair[[1]])
	l2 <- levels(colpair[[2]])
	if (1) {
		result <- .Call('_rpf_fast_tableWithWeights', colpair[[1]], colpair[[2]], weights)
	} else {
		result <- matrix(0.0, length(l1), length(l2))
		if (nrow(colpair)) for (rx in 1:nrow(colpair)) {
			row <- colpair[rx,]
			ind <- sapply(row, unclass)
			w <- 1
			if (length(weights)) w <- weights[rx]
			result[ind[1], ind[2]] <- result[ind[1], ind[2]] + w
		}
	}
	dimnames(result) <- list(l1, l2)
	names(dimnames(result)) <- colnames(colpair)
	result
}

complainAboutQuadSpec <- function()
{
  stop("Specify qwidth and qpoints as part of the group; For example, grp$qpoints <- 31L; grp$qwidth=4")
}

##' Computes local dependence indices for all pairs of items
##'
##' Item Factor Analysis makes two assumptions: (1) that the latent
##' distribution is reasonably approximated by the multivariate Normal
##' and (2) that items are conditionally independent. This test
##' examines the second assumption. The presence of locally dependent
##' items can inflate the precision of estimates causing a test to
##' seem more accurate than it really is.
##'
##' Statically significant entries suggest that the item pair has
##' local dependence. Since log(.01)=-4.6, an absolute magitude of 5
##' is a reasonable cut-off. Positive entries indicate that the two
##' item residuals are more correlated than expected. These items may share an
##' unaccounted for latent dimension. Consider a redesign of the items
##' or the use of testlets for scoring. Negative entries indicate that
##' the two item residuals are less correlated than expected.
##'
##' @template detail-group
##'
##' @template arg-grp
##' @template arg-dots
##' @param data data \lifecycle{deprecated}
##' @param inames a subset of items to examine
##' @param qwidth \lifecycle{deprecated}
##' @param qpoints \lifecycle{deprecated}
##' @param method method to use to calculate P values. The default is the
##' Pearson X^2 statistic. Use "lr" for the similar likelihood ratio statistic.
##' @param .twotier whether to enable the two-tier optimization
##' @param .parallel whether to take advantage of multiple CPUs (default TRUE)
##' @return a list with raw, pval and detail. The pval matrix is a
##' lower triangular matrix of log P values with the sign
##' determined by relative association between the observed and
##' expected tables (see \code{\link{ordinal.gamma}})
##' @aliases chen.thissen.1997
##' @references Chen, W.-H. & Thissen, D. (1997). Local dependence
##' indexes for item pairs using Item Response Theory. \emph{Journal
##' of Educational and Behavioral Statistics, 22}(3), 265-289.
##'
##' Thissen, D., Steinberg, L., & Mooney, J. A. (1989). Trace lines for testlets: A use
##' of multiple-categorical-response models. \emph{Journal of Educational Measurement,
##' 26} (3), 247--260.
##'
##' Wainer, H. & Kiely, G. L. (1987). Item clusters and computerized
##' adaptive testing: A case for testlets.  \emph{Journal of
##' Educational measurement, 24}(3), 185--201.
##' @family diagnostic
##' @seealso \href{https://github.com/jpritikin/ifaTools}{ifaTools}
ChenThissen1997 <- function(grp, ..., data=NULL, inames=NULL, qwidth=6, qpoints=49, method="pearson",
			    .twotier=TRUE, .parallel=TRUE) {
	if (length(list(...)) > 0) {
		stop(paste("Remaining parameters must be passed by name", deparse(list(...))))
	}

  if (is.null(colnames(grp$param))) stop("Item parameter columns must be named")

  if (missing(data)) {
      data <- grp$data
  }
	if (!missing(qwidth) || !missing(qpoints)) complainAboutQuadSpec()

  if (method != "pearson" && method != "lr") stop(paste("Unknown method", method))
  if (missing(inames)) {
    inames <- colnames(grp$param)
  }
  if (length(inames) < 2) stop("At least 2 items are required")

  spec <- grp$spec
  if (length(spec) < dim(grp$param)[2]) {
      if (dim(grp$param)[2] %% length(spec) != 0) stop("Length of spec must match # of items")
      rep <- dim(grp$param)[2] %/% length(spec)
      while (rep > 1) {
          spec <- c(spec, grp$spec)
          rep <- rep - 1
      }
  }

	if (!is.data.frame(data)) {
		data <- as.data.frame(data)  #safe? TODO
	}
	dataMap <- match(colnames(grp$param), colnames(data))

	items <- match(inames, colnames(grp$param))

	# If we move this whole loop into C then we can avoid
	# repeated set up of the quadrature
	pairs <- list()
	for (iter1 in 2:length(items)) {
		for (iter2 in 1:(iter1-1)) {
			pairs[[ 1+length(pairs) ]] <- c(iter1, iter2)
		}
	}
	.la <- ifelse(.parallel, mclapply, lapply)
	detail <- .la(pairs, function(pair) {
		iter1 <- pair[1]
		iter2 <- pair[2]
		i1 <- items[iter1]
		i2 <- items[iter2]
		obpair <- data[,dataMap[c(i1,i2)]]
		rowWeight <- c()
		if (!is.null(grp[['weightColumn']])) {
			rowWeight <- data[[ grp[['weightColumn']] ]]
		}
		if (!is.null(grp[['freqColumn']])) {
      fc <- data[[ grp[['freqColumn']] ]]
			if (length(rowWeight) == 0) rowWeight <- fc
      else rowWeight <- rowWeight * fc
		}
		observed <- tableWithWeights(obpair, rowWeight)
		N <- sum(observed)

		expected <- N * pairwiseExpected(grp, c(i1, i2), .twotier)
		if (any(dim(observed) != dim(expected))) {
			if (dim(observed)[1] != dim(expected)[1]) {
				bad <- i1
				margin <- 1
			} else {
				bad <- i2
				margin <- 2
			}
			Eoutcomes <- dim(expected)[margin]
			lev <- dimnames(observed)[[margin]]
			stop(paste(colnames(grp$param)[bad], " has ", Eoutcomes,
				   " outcomes in the model but the data has ", length(lev), " (",
				   paste(lev, collapse=", "),")", sep=""))
		}
		dimnames(expected) <- dimnames(observed)
		info <- list(orig.observed=observed, orig.expected=expected)
		info <- CT1997Internal1(info, method)
		info
	})

	lapply(detail, function(d1) {
		if (inherits(d1, "try-error")) stop(d1)
	})

	names(detail) <- sapply(pairs, function(pair) {
		paste(inames[pair[1]], inames[pair[2]], sep=":")
	})

	retobj <- CT1997Internal2(inames, detail)
	retobj$inames <- inames
	retobj$method <- method
	retobj
}

# deprecated name
chen.thissen.1997 <- ChenThissen1997

"+.summary.ChenThissen1997" <- function(e1, e2) {
	e2name <- deparse(substitute(e2))
	if (!inherits(e2, "summary.ChenThissen1997")) {
		stop("Don't know how to add ", e2name, " to a ChenThissen1997",
		     call. = FALSE)
	}
	if (length(e1$detail) != length(e2$detail)) {
		stop("Cannot combine two groups with a different number of items")
	}
	if (any(names(e1$detail) != names(e2$detail))) {
		stop("Cannot combine two groups with a different items")
	}
	method <- e1$method
	detail <- mapply(function(i1, i2) {
		ii <- list(orig.observed = i1$orig.observed + i2$orig.observed,
			   orig.expected = i1$orig.expected + i2$orig.expected)
		ii <- CT1997Internal1(ii, method)
	}, e1$detail, e2$detail, SIMPLIFY=FALSE)

	retobj <- CT1997Internal2(e1$inames, detail)
	retobj$inames <- e1$inames
	retobj$method <- method
	retobj
}

print.summary.ChenThissen1997 <- function(x,...) {
	cat("Chen & Thissen (1997) local dependence test\n")
	cat("  Magnitudes larger than abs(log(.01))=4.6 are significant at the p=.01 level\n")
	cat("  A positive (negative) sign indicates more (less) observed correlation than expected\n\n")
	print(round(x$pval,2))
}

##' Monte-Carlo test for cross-tabulation tables
##'
##' \lifecycle{experimental}
##' This is for developers.
##'
##' @param ob observed table
##' @param ex expected table
##' @param trials number of Monte-Carlo trials
crosstabTest <- function(ob, ex, trials) {
	if (missing(trials)) trials <- 10000
	.Call('_rpf_crosstabTest_cpp', ob, ex, trials)
}

pairwiseExpected <- function(grp, items, .twotier=FALSE) {
	.Call('_rpf_pairwiseExpected_cpp', grp, items - 1L, .twotier)
}

CaiHansen2012 <- function(grp, method, .twotier = FALSE) {
	.Call('_rpf_CaiHansen2012_cpp', grp, method, .twotier)
}

##' Multinomial fit test
##'
##' For degrees of freedom, we use the number of observed statistics
##' (incorrect) instead of the number of possible response patterns
##' (correct) (see Bock, Giibons, & Muraki, 1998, p. 265). This is not
##' a huge problem because this test is becomes poorly calibrated when
##' the multinomial table is sparse. For more accurate p-values, you
##' can conduct a Monte-Carlo simulation study (see examples).
##'
##' Rows with missing data are ignored.
##'
##' The full information test is described in Bartholomew & Tzamourani
##' (1999, Section 3).
##'
##' For CFI and TLI, you must provide an \code{independenceGrp}.
##'
##' @template detail-group
##'
##' @template arg-grp
##' @param independenceGrp the independence group
##' @template arg-dots
##' @param method lr (default) or pearson
##' @param log whether to report p-value in log units
##' @param .twotier whether to use the two-tier optimization (default TRUE)
##' @references Bartholomew, D. J., & Tzamourani, P. (1999). The
##' goodness-of-fit of latent trait models in attitude
##' measurement. \emph{Sociological Methods and Research, 27}(4), 525-546.
##'
##' Bock, R. D., Gibbons, R., & Muraki, E. (1988). Full-information
##' item factor analysis. \emph{Applied Psychological Measurement, 12}(3),
##' 261-280.
##' @family diagnostic
##' @examples
##' # Create an example IFA group
##' grp <- list(spec=list())
##' grp$spec[1:10] <- rpf.grm()
##' grp$param <- sapply(grp$spec, rpf.rparam)
##' colnames(grp$param) <- paste("i", 1:10, sep="")
##' grp$mean <- 0
##' grp$cov <- diag(1)
##' grp$uniqueFree <- sum(grp$param != 0)
##' grp$data <- rpf.sample(1000, grp=grp)
##'
##' # Monte-Carlo simulation study
##' mcReps <- 3    # increase this to 10,000 or so
##' stat <- rep(NA, mcReps)
##' for (rx in 1:mcReps) {
##'    t1 <- grp
##'    t1$data <- rpf.sample(grp=grp)
##'    stat[rx] <- multinomialFit(t1)$statistic
##' }
##' sum(multinomialFit(grp)$statistic > stat)/mcReps   # better p-value

multinomialFit <- function(grp, independenceGrp, ..., method="lr", log=TRUE, .twotier=TRUE) {
	if (length(list(...)) > 0) {
		stop(paste("Remaining parameters must be passed by name", deparse(list(...))))
	}
	todo <- list(grp)
	if (!missing(independenceGrp)) todo <- list(grp, independenceGrp)
	todo <- lapply(todo, function(gx) {
		if (is.null(gx$weightColumn)) {
			wc <- "freq"
			gx$data <- compressDataFrame(gx$data, wc, .asNumeric=TRUE)
			gx$weightColumn <- wc
			gx$observedStats <- nrow(gx$data)
		}
		if (is.null(gx$uniqueFree)) {
			warning("Number of free parameters not available; assuming 0")
			gx$uniqueFree <- 0
		}
		if (is.null(gx$observedStats)) {
			warning("Number of observed statistics unknown; assuming the number of possible response patterns")
			gx$observedStats <- prod(sapply(gx$spec, function(sp) sp$outcomes))
		}
		gx
	})
	got <- lapply(todo, CaiHansen2012, method, .twotier)
	stat <- got[[1]]$stat
	df <- todo[[1]]$observedStats - todo[[1]]$uniqueFree
	out <- list()
	out <- c(out, list(statistic=stat, df=df))
	out$pval <- pchisq(stat, out$df, lower.tail=FALSE, log.p=log)
	out$log <- log
	out$method <- method
	out$n <- got[[1]]$n
	out$omitted <- grp$omitted
	if (length(todo) == 1) {
		fi <- computeFitStatistics(stat, df, out$n, NA, NA)
	} else {
		fi <- computeFitStatistics(stat, df, out$n,
					   got[[2]]$stat, todo[[2]]$observedStats - todo[[2]]$uniqueFree)
	}
	for (k in names(fi)) out[[k]] <- fi[[k]]
	class(out) <- "summary.multinomialFit"
	out
}

print.summary.multinomialFit <- function(x,...) {
	cat("Full information multinomial fit test\n")
	part1 <- paste("n = ", x$n, ", ", x$method, "(", x$df, ") = ", round(x$statistic, 2), sep="")
	if (x$log) {
		part2 <- paste("log(p) = ", round(x$pval,2), sep="")
	} else {
		part2 <- paste("p = ", round(x$pval,4), sep="")
	}
	cat(paste(part1, ", ", part2, sep=""), fill=TRUE)
	catFitStatistics(x)
	if (!is.null(x$omitted)) {
		cat(paste("omitted: ", paste(x$omitted, collapse=", "), "\n", sep=""))
	}
}

suppressWarnings(RNGversion("3.5"))
library(testthat)
library(rpf)
library(mvtnorm)

options(mc.cores=1L) #otherwise not picked up by covr

context("chen & thissen 1997")

test_that("collapseCategoricalCells", {
  O = matrix(c(7,31,42,20,0), 1,5,
             dimnames=list('a',letters[1:5]))
  E = matrix(c(3,39,50,8,0), 1,5,
             dimnames=list('a',letters[1:5]))
  got <- collapseCategoricalCells(O,E,9)
  expect_equal(O['a','a'],7)
  expect_true(is.na(got$O['a','a']))
  expect_equal(got$collapsed, 2)
  expect_equal(E[,letters[2:3]], got$E[,letters[2:3]])
})

mxSimplify2Array <- function(x, higher=FALSE) {
	if (higher) {
		stop("higher=TRUE is not implemented. Consider using simplify2array")
	}
  len <- sapply(x, length)
  biggest <- which(len == max(len))[1]
  out <- matrix(NA, nrow=max(len), ncol=length(x))
  for (iter in 1:length(x)) {
	  if (len[iter] == 0) next
    out[1:len[iter],iter] <- x[[iter]]
  }
  colnames(out) <- names(x)
  rownames(out) <- names(x[[biggest]])
  out
}

test_that("ct1997", {
  set.seed(1)
  
  spec <- list()
  spec[1:5] <- list(rpf.grm(factors=2))
  spec[6]   <- list(rpf.drm(factors=2))
  gen.param <- mxSimplify2Array(lapply(spec, rpf.rparam, version=1))
  colnames(gen.param) <- paste("i", 1:ncol(gen.param), sep="")
  gen.param[2,] <- c(0,0,.5,.5,1,1)
  
  theta <- rmvnorm(1000, c(0,0), diag(2))
  resp <- rpf.sample(t(theta), spec, gen.param)
  
  # hide latent factor that we don't know about
  tspec <- list()
  tspec[1:length(spec)] <- lapply(spec, rpf.modify, factors=1)
  
  grp <- list(spec=tspec, param=gen.param[-2,], mean=c(0), cov=diag(1), data=resp)
  
  got <- ChenThissen1997(grp)
  #cat(deparse(round(got$pval[!is.na(got$pval)], 2)))
  expect_equal(got$pval[!is.na(got$pval)], 
               c(-1.31, -0.35, 3.15, 5.7, -5.66, 0.54, 3.43, -5.45, -0.57,
                 7.5,  8.11, 1.78, 15.37, 2.79, 5.68), tolerance=.001)
  #cat(deparse(round(got$gamma[!is.na(got$gamma)], 3)))
  expect_equal(got$gamma[!is.na(got$gamma)],
               c(-0.064, -0.002, 0.058, 0.062, -0.197, 0.023, 0.055, -0.008,
                 -0.025, 0.158, 0.13, 0.081, 0.232, 0.02, 0.052), tolerance=.01)
})

test_that("ct1997 2tier", {
  set.seed(1)
  require(rpf)
	spec <- list()
	spec[1:5] <- list(rpf.drm(factors=3))
	gen.param <- sapply(spec, rpf.rparam)
  gen.param['a2', 1:2] <- 0
  gen.param['a3', 3] <- 0
  gen.param[c('a2','a3'), 4:5] <- 0
  colnames(gen.param) <- paste("i", 1:ncol(gen.param), sep="")
	resp <- rpf.sample(500, spec, gen.param)
	grp <- list(spec=spec, param=gen.param, mean=runif(3, 0, 1),
	            cov=diag(runif(3,1,2)), data=resp,
	            qpoints=13L, qwidth=4)
	slow <- ChenThissen1997(grp, .twotier=FALSE)
	fast <- ChenThissen1997(grp, .twotier=TRUE)
  expect_equal(slow$raw[!is.na(slow$raw)],
               fast$raw[!is.na(fast$raw)], tolerance=.001)
  
  grp$data <- rpf.sample(200, spec, gen.param, mcar=.2)
  fast <- ChenThissen1997(grp)
  got <- fast$pval[,'i1']
  names(got) <- NULL
#  cat(deparse(round(fast$pval[,'i1'],2)))
  expect_equal(got, c(1.49, 1.34, 2.79, -1.54), tolerance=.1)
})

mxSimplify2Array <- function(x) {
  par <- x
  len <- sapply(par, length)
  biggest <- which(len == max(len))[1]
  out <- matrix(NA, nrow=max(len), ncol=length(par))
  for (x in 1:length(par)) {
    out[1:len[x],x] <- par[[x]]
  }
  rownames(out) <- names(par[[biggest]])
  out
}

test_that("ct1997 permutations", {
  require(rpf)
  set.seed(1)
  grp <- list(spec=list())
  grp$spec[1:10] <- lapply(sample.int(6, 10, replace=TRUE), function(o) rpf.grm(outcomes=1+o))
  grp$param <- mxSimplify2Array(lapply(grp$spec, rpf.rparam))
  colnames(grp$param) <- paste("i", 1:10, sep="")
  grp$mean <- 0
  grp$cov <- diag(1)
  grp$free <- !is.na(grp$param) & grp$param != 0
  grp$data <- rpf.sample(500, grp=grp)
  grp$data <- grp$data[,colnames(grp$data)[sample.int(10)]]
  
  ChenThissen1997(grp, inames = colnames(grp$param)[sample.int(10, 9)])
  # will stop if something is wrong
  
  grp1 <- grp
  grp1$data <- grp$data[1:250,]
  grp2 <- grp
  grp2$data <- grp$data[251:500,]

  r1 <- ChenThissen1997(grp)
  r2 <- ChenThissen1997(grp1) + ChenThissen1997(grp2)
  expect_equal(r1$pval, r2$pval)
  expect_equal(r1$gamma, r2$gamma)
})

drawRandomProportion <- function(expected) {
  total <- sum(expected)
  prob <- expected / total
  sim <- rep(NA, length(expected))
  rowSim <- sample.int(length(expected), size=total, prob=prob, replace=TRUE)
  sim <- tabulate(rowSim, length(expected))
  sim
}

if (0) {
  spec <- list()
  spec[1:6] <- rpf.grm(factors=1)
  gen.param <- sapply(spec, rpf.rparam)
  grp <- list(spec=spec, param=gen.param, mean=c(0), cov=diag(1))

  pair <- c(1L,2L)
  numPeople <- 185
  E <- (numPeople * pairwiseExpected(grp, pair))

  ms <- pairwiseItemDistribution(grp, pair)
  hist(ms)
  quantile(ms, c(.95, .99))   # 25 38
  
  trials <- 10000
  ms <- rep(NA, trials)
  for (tx in 1:trials) {
    O <- drawPairwiseSample(grp, pair, numPeople)
    ms[tx] <- sum((c(E) - O)^2)
  }

  hist(ms)
  quantile(ms, c(.95, .99))   # 25 38
}

pearson.gof <- function(observed, expected, df) {
  x2 <- sum((observed - expected)^2/expected)
  if (missing(df)) {
    df <- (dim(observed)[1]-1) * (dim(observed)[2]-1)
  }
  pchisq(x2, df, lower.tail=FALSE)
}

if (0) {
  spec <- list()
  spec[1:6] <- rpf.grm(factors=1)
  gen.param <- sapply(spec, rpf.rparam)
  grp <- list(spec=spec, param=gen.param, mean=c(0), cov=diag(1))

  pair <- c(1L,2L)
  numPeople <- 500
  E <- (numPeople * pairwiseExpected(grp, pair))

  trials <- 50
  got <- expand.grid(trial=1:trials, method=c("pearson","rms"), pval=NA)
  for (rep in 1:trials) {
    O <- drawPairwiseSample(grp, pair, 50, qpts=11L, qwidth=4)  # doesn't work!
    O <- O * numPeople / 50
    got[got$trial==rep,'pval'] <- c(pearson.gof(O, E),
                                    pairwiseItemTest(grp, pair, O, qpts=11, qwidth=4))
#                                    ptw2011.gof.test(O, E)
   print(rep)
  }
  #
  require(ggplot2)
 mask <- got$method=="rms"
  #mask <- rep(TRUE, nrow(got))
 pval <- got[mask,'pval']
 got[mask,'pval'] <- 1 / (1+exp(-(logit(pval) - 2.8)))
  
  tbl <- expand.grid(alpha=seq(0,1,.01), method=c("pearson","rms"), emp=NA)
  tbl[tbl$method=="rms",'emp'] <-
    Vectorize(function(x) sum(got[got$method=="rms",'pval'] < x))(seq(0,1,.01))
  tbl[tbl$method=="pearson",'emp'] <-
    Vectorize(function(x) sum(got[got$method=="pearson",'pval'] < x))(seq(0,1,.01))
  tbl$emp <- tbl$emp / trials
  
  ggplot(tbl, aes(alpha, emp, color=method)) + geom_line() +
    geom_abline(slope=1, color="yellow")+ coord_fixed()
}

library(testthat)
library(rpf)

context("ordinal.gamma")

# Example data from Agresti (1990, p. 21)
jobsat <- matrix(c(20,22,13,7,24,38,28,18,80,104,81,54,82,125,113,92), nrow=4, ncol=4)
expect_equal(ordinal.gamma(jobsat), .1265, tolerance=.001)

#options(error = utils::recover)
library(testthat)
library(rpf)

context("outfit/infit")

test_that("kct", {
  data(kct)
  responses <- kct.people[,paste("V",2:19, sep="")]
  rownames(responses) <- kct.people$NAME
  colnames(responses) <- kct.items$NAME

  scores <- kct.people$MEASURE
  params <- cbind(1, kct.items$MEASURE, logit(0), logit(1))
  rownames(params) <- kct.items$NAME
  items<-list()
  items[1:18] <- list(rpf.drm())
  params[,2] <- -params[,2]

  expect_warning(fit <- rpf.1dim.fit(items, t(params), responses, scores, 2, wh.exact=TRUE),
                 "Excluding response Helen")

  expect_equal(fit$infit, kct.items$IN.MSQ[4:17], tolerance=.002)
  expect_equal(fit$infit.z, kct.items$IN.ZSTD[4:17], tolerance=.01)
  expect_equal(fit$outfit, kct.items$OUT.MSQ[4:17], tolerance=.002)
  expect_equal(fit$outfit.z, kct.items$OUT.ZSTD[4:17], tolerance=.01)

  expect_warning(fit <- rpf.1dim.fit(items, t(params), responses, scores, 1, wh.exact=TRUE),
                 "Excluding item 1= 1-4")
  
  expect_equal(fit$infit, kct.people$IN.MSQ[1:34], tolerance=.002)
  expect_equal(fit$infit.z, kct.people$IN.ZSTD[1:34], tolerance=.005)
  expect_equal(fit$outfit, kct.people$OUT.MSQ[1:34], tolerance=.002)
  expect_equal(fit$outfit.z, kct.people$OUT.ZSTD[1:34], tolerance=.005)
})

test_that("freq", {
  data(kct)
  responses <- kct.people[,paste("V",2:19, sep="")]
  colnames(responses) <- kct.items$NAME
  
  params <- cbind(1, kct.items$MEASURE, logit(0), logit(1))
  rownames(params) <- kct.items$NAME
  params[,2] <- -params[,2]
  grp <- list(spec=rep(list(rpf.drm()),18),
              param=t(params),
              data=responses,
              scores=data.frame(skill=kct.people$MEASURE))
  
  fit1 <- suppressWarnings(rpf.1dim.fit(group=grp, margin=1, wh.exact = TRUE))
  expect_equal(as.integer(fit1$name), 1:34)
  
  grp$data <- grp$data[c(rep(1,10),2:34),]
  grp$scores <- grp$scores[c(rep(1,10),2:34),,drop=FALSE]
  fit2 <- suppressWarnings(rpf.1dim.fit(group=grp, margin=1, wh.exact = TRUE))
  fit2 <- fit2[-c(2:10),]
  expect_true(all(fit1 == fit2))
})

plot.icc <- function(ii, ii.p, width=7) {
  require(ggplot2)
  require(reshape2)
  grid <- expand.grid(theta=seq(-width,width,.1))
  grid <- cbind(grid, t(rpf.prob(ii, ii.p, grid$theta)))
  grid2 <- melt(grid, id.vars=c("theta"), variable.name="category", value.name="p")
  ggplot(grid2, aes(theta, p, color=category)) + geom_line() +
    ylim(0,1) + xlim(-width, width)
}

test_that("sf", {
  data(science)
  spec <- list()
  spec[1:25] <- list(rpf.nrm(outcomes=3, T.c = lower.tri(diag(2),TRUE) * -1))
  
  param <- rbind(a=1, alf1=1, alf2=0,
        gam1=sfif$MEASURE + sfsf[sfsf$CATEGORY==1,"Rasch.Andrich.threshold.MEASURE"],
        gam2=sfif$MEASURE + sfsf[sfsf$CATEGORY==2,"Rasch.Andrich.threshold.MEASURE"])
  colnames(param) <- sfif$NAME
  
  iorder <- match(sfif$NAME, colnames(sfpf))
  responses <- sfpf[,iorder]
  rownames(responses) <- sfpf$NAME
  
  expect_warning(fit <- rpf.1dim.fit(spec, param, responses, sfpf$MEASURE, 2, wh.exact=TRUE),
                 "Excluding item GO TO MUSEUM")
  
  expect_equal(fit$infit, sfif$IN.MSQ[-12], tolerance=.002)
  expect_equal(fit$infit.z, sfif$IN.ZSTD[-12], tolerance=.005)
  expect_equal(fit$outfit, sfif$OUT.MSQ[-12], tolerance=.002)
  expect_equal(fit$outfit.z, sfif$OUT.ZSTD[-12], tolerance=.005)

  expect_warning(fit <- rpf.1dim.fit(spec, param, responses, sfpf$MEASURE, 1, wh.exact=TRUE),
                 "Excluding response ROSSNER, LAWRENCE")
  
  expect_equal(fit$infit, sfpf$IN.MSQ[-2], tolerance=.02)
  expect_equal(fit$infit.z, sfpf$IN.ZSTD[-2], tolerance=.05)
  expect_equal(fit$outfit, sfpf$OUT.MSQ[-2], tolerance=.05)
  expect_equal(fit$outfit.z, sfpf$OUT.ZSTD[-2], tolerance=.075)
})

library(testthat)
library(rpf)

context("goodness of fit")

# traditional Pearson X^2 goodness of fit test
pearson.gof <- function(observed, expected, df) {
  x2 <- sum((observed - expected)^2/expected)
  if (missing(df)) {
    df <- (dim(observed)[1]-1) * (dim(observed)[2]-1)
  }
  pchisq(x2, df, lower.tail=FALSE)
}

ms <- function(observed, expected, draws) {
  draws * sum((observed - expected)^2)
}

mc.gof.test <- function(observed, expected) {
  crosstabTest(matrix(observed, nrow=1), matrix(c(expected), nrow=1), 10000)
}

geolen <- function(v) sqrt(sum(v * v))
angle <- function(v1, v2) acos(sum(v1 * v2) / (geolen(v1) * geolen(v2)))

test_that("mc", {
  set.seed(1)
  bins <- 8L
  v1 <- matrix(runif(bins), nrow=2)
  v2 <- matrix(runif(bins), nrow=2)
  v1 <- v1 / sum(v1)
  v2 <- v2 / sum(v2)
  draws <- 300  # ptw2011 accuracy is proportional to draws
  
  got <- data.frame(gap=seq(.5, .05, -.025))
  for (xx in 1:dim(got)[1]) {
    gap <- got[xx,'gap']
    v3 <- v1 * (1-gap) + v2 * gap
    got$angle[xx] <- angle(v1, v3)
    observed <- v1 * draws
    expected <- v3 * draws
    got$mc[xx] <- mc.gof.test(observed, expected)
    got$ptw[xx] <- ptw2011.gof.test(observed, expected)
    got$x2[xx] <- pearson.gof(observed, expected, bins-1)
  }

  expect_equal(got$mc, got$ptw, .01)
#  print(max(abs(got$x2 - got$mc)))
  expect_true(max(abs(got$x2 - got$mc)) > .15)
})

test_that("crazy1", {
  # can obtain less than 0 without check
  observed <- structure(c(0L, 0L, 0L, 0L, 0L, 158L, 35L, 51L, 22L, 40L), .Dim = c(5L,  2L))
  expected <- structure(c(78.8628, 5.0351, 2.835, 0.5783, 0.4605, 108.4319,  23.5408, 35.5775, 17.2235, 33.4546), .Dim = c(5L, 2L))
  expect_true(ptw2011.gof.test(observed, expected) >= 0)
})

drawRandomProportion <- function(expected) {
  total <- sum(expected)
  prob <- expected / total
  sim <- rep(NA, length(expected))
  rowSim <- sample.int(length(expected), size=total, prob=prob, replace=TRUE)
  sim <- tabulate(rowSim, length(expected))
  sim
}

drawRandomCrosstab <- function(expected) {
  rowTotals <- apply(expected, 1, sum)
  sim <- matrix(NA, nrow(expected), ncol(expected))
  for (rx in 1:nrow(expected)) {
    rowSim <- sample.int(ncol(expected), size=rowTotals[rx],
                         prob=c(expected[rx,] / rowTotals[rx]), replace=TRUE)
    sim[rx,] <- as.numeric(tabulate(rowSim, ncol(expected)))
  }
  sim
}

if (0) {
  E = structure(c(2.849, 13.056, 19.17, 28.498, 41.552, 41.064, 56.528,  65.796, 62.687, 66.283, 56.303, 38.329, 49.191, 40.029, 31.637,  19.376, 11.155, 5.057, 2.257, 0.347, 0.151, 0.944, 1.83, 3.502,  6.448, 7.936, 13.472, 19.204, 22.313, 28.717, 29.697, 24.671,  38.809, 38.971, 38.363, 29.624, 21.845, 12.943, 7.743, 1.653), .Dim = c(20L,  2L))
  trials <- 150
  got <- rep(NA, trials)
  for (rep in 1:trials) {
    got[rep] <- crosstabTest(drawRandomCrosstab(E),E, 1000)
  }
  
  require(ggplot2)
  qplot(c(0, 1), stat = "function", fun = Vectorize(function(x) sum(got < x)/length(got)), geom = "line") +
    geom_abline(slope=1, color="red")+ coord_fixed()
}

##' Create monotonic polynomial generalized partial credit (GPC-MP) model
##'
##' This model is a polytomous model proposed by Falk & Cai (2016) and is based
##' on the generalized partial credit model (Muraki, 1992).
##'
##' The GPC-MP replaces the linear predictor part of the
##' generalized partial credit model with a monotonic polynomial,
##' \eqn{m(\theta;\omega,\xi,\mathbf{\alpha},\mathbf{\tau})}{m(theta; omega, alpha, tau)}.
##' The response function for category k is:
##'
##' \deqn{\mathrm P(\mathrm{pick}=k|\omega,\xi,\alpha,\tau,\theta)
##' = \frac{\exp(\sum_{v=0}^k (\xi_k + m(\theta;\omega,\xi,\mathbf{\alpha},\mathbf{\tau})))}{\sum_{u=0}^{K-1}\exp(\sum_{v=0}^u (\xi_u + m(\theta;\omega,\xi,\mathbf{\alpha},\mathbf{\tau})))}
##' }{\frac{exp(sum_{v=0}^k (xi_k + m(theta;omega,xi,alpha,tau)))}{sum_{u=0}^{K-1}exp(sum_{v=0}^u (xi_u + m(theta;omega,xi,alpha,tau)))}}
##'
##' where \eqn{\mathbf{\alpha}}{alpha} and \eqn{\mathbf{\tau}}{tau} are vectors
##' of length q. The GPC-MP uses the same parameterization for the polynomial
##' as described for the logistic function of a monotonic polynomial (LMP).
##' See also (\code{\link{rpf.lmp}}).
##'
##' The order of the polynomial is always odd and is controlled by
##' the user specified non-negative integer, q. The model contains
##' 1+(outcomtes-1)+2*q parameters and are used as input to the \code{\link{rpf.prob}}
##' function in the following order:
##' \eqn{\omega}{omega} - natural log of the slope of the item model when q=0,
##' \eqn{\xi}{xi} - a (outcomes-1)-length vector of intercept parameters,
##' \eqn{\alpha}{alpha} and \eqn{\tau}{tau} - two parameters that control bends in
##' the polynomial. These latter parameters are repeated in the same order for
##' models with q>0. For example, a q=2 polynomial with 3 categories will have an item
##' parameter vector of: \eqn{\omega, \xi_1, \xi_2, \alpha_1, \tau_1, \alpha_2, \tau_2}{
##' omega, xi1, xi2, alpha1, tau1, alpha2, tau2}.
##'
##' Note that the GPC-MP reduces to the LMP when the number of
##' categories is 2, and the GPC-MP reduces to the generalized partial
##' credit model when the order of the polynomial is 1 (i.e., q=0).
##'
##' @param outcomes The number of possible response categories.
##' @param q a non-negative integer that controls the order of the
##' polynomial (2q+1) with a default of q=0 (1st order polynomial = generalized partial credit model).
##' @param multidimensional whether to use a multidimensional model.
##' Defaults to \code{FALSE}. The multidimensional version is not yet
##' available.
##' @return an item model
##' @references Falk, C. F., & Cai, L. (2016). Maximum marginal likelihood
##' estimation of a monotonic polynomial generalized partial credit model with
##' applications to multiple group analysis. \emph{Psychometrika, 81}, 434-460.
##' \doi{10.1007/s11336-014-9428-7}
##' @references Muraki, E. (1992). A generalized partial credit model: Application of an EM algorithm. \emph{Applied Psychological Measurement, 16,} 159–176.
##'
##' @family response model
##'
##' @examples
##' spec <- rpf.gpcmp(5,2) # 5-category, 3rd order polynomial
##' theta<-seq(-3,3,.1)
##' p<-rpf.prob(spec, c(1.02,3.48,2.5,-.25,-1.64,.89,-8.7,-.74,-8.99),theta)
##'

rpf.gpcmp <- function(outcomes=2, q=0, multidimensional=FALSE) {
  if(!(q%%1==0)){
    stop("q must be an integer >= 0")
  }
  if(multidimensional){
      stop("Multidimensional gpcmp model is not yet supported")
  }
  m <- NULL
  id <- -1
  id <- rpf.id_of("gpcmp")
  m <- new("rpf.1dim.gpcmp",
           outcomes=outcomes,
           factors=1)
  m@spec <- c(id, m@outcomes, m@factors, q)
  m
}

setMethod("rpf.rparam", signature(m="rpf.1dim.gpcmp"),
          function(m, version) {
            n <- 1
            q<-m$spec[4] ## ok to hardcode this index?
            ret<-c(omega=rnorm(n, 0, .5)) # random overall slope
            # randomly generate xi
            xi <- rnorm(m@outcomes-1)
            xi <- xi[order(xi)]
            ret<-c(ret,xi=xi)
            if(q>0){
                for(i in 1:q){
                    ret<-c(ret,runif(n,-1,1),log(runif(n,.0001,1)))
                    names(ret)[(m@outcomes+1+(i-1)*2):(m@outcomes+(i*2))]<-c(paste("alpha",i,sep=""),paste("tau",i,sep=""))
                }
            }
            ret
        })

suppressWarnings(RNGversion("3.5"))
library(testthat)
library(rpf)
library(numDeriv)
#options(error = utils::recover)

context("dTheta")

items <- list(rpf.drm(),
              rpf.drm(factors=2),
              rpf.grm(outcomes=3, factors=2),
              rpf.nrm(outcomes=4, factors=2, T.a="random", T.c="random"),
              rpf.nrm(outcomes=4, factors=3, T.a="random", T.c="random"),
              rpf.lmp(q=0),
              rpf.lmp(q=1),
              rpf.gpcmp(q=0),
              rpf.gpcmp(q=1))

triSize <- function(sz) sz*(sz + 1) / 2

for (ii in items) {
  test_that(class(ii), {
    dir <- runif(ii$factors)
    dir <- dir / sqrt(sum(dir^2))

    ii.p <- rpf.rparam(ii)
    at <- rnorm(ii$factors)
    analytic <- rpf.dTheta(ii, ii.p, at, dir)
    numDeriv <- ii$factors + triSize(ii$factors)
    deriv <- matrix(NA, numDeriv, ii$outcomes)
    for (ox in 1:ii$outcomes) {
      got <- genD(function(param) {
        rpf.prob(ii, ii.p, param)[ox]
      }, at, method.args=list(eps=0.01, d=0.01, r=2))
      deriv[,ox] <- got$D
    }
    if (ii$factors == 1) {
      expect_equal(analytic$gradient, c(deriv[1,]), tolerance=1e-6)
      expect_equal(analytic$hessian, c(deriv[2,]), tolerance=1e-3)
    } else {
      expect_equal(analytic$gradient,
                   c(dir %*% deriv[c(1:ii$factors),]), tolerance=1e-6)
      expect_equal(analytic$hessian,
                   c(dir %*% deriv[c(ii$factors+triSize(1:ii$factors)),]), tolerance=1e-3)
    }
  })
}

##' Create a multiple-choice response model
##'
##' \lifecycle{experimental}
##'
##' This function instantiates a multiple-choice response
##' model.
##'
##' @param outcomes the number of possible outcomes
##' @param numChoices the number of choices available
##' @param factors the number of factors
##' @return an item model
##' @family response model
##' @author Jonathan Weeks <weeksjp@@gmail.com>
rpf.mcm <- function(outcomes=2, numChoices=5, factors=1) {
  stop("Not implemented")
  guess.weight <- 20
  guessing <- (1/numChoices)
  new("rpf.mdim.mcm",
      outcomes=outcomes, factors=factors,
      a.prior.sdlog=.5,
      c.prior.alpha=guess.weight*guessing+1,
      c.prior.beta=guess.weight*(1-guessing)+1)
}

### mdim

setMethod("rpf.prob", signature(m="rpf.mdim.mcm", param="numeric",
                                theta="matrix"),
          function(m, param, theta) {
            den <- NULL

            a1 <- param[1:(m@factors*m@outcomes)]
            b1 <- param[(m@factors*m@outcomes+1):
                        ((m@factors+1)*m@outcomes)]
            c1 <- param[-1:-((m@factors+1)*m@outcomes)]

            ##   Compute the denominator
            for (k in 1:m@outcomes) {
              tmp <- (k-1)*m@factors
              tmp1 <- tmp+m@factors
              d <- exp((theta %*% a1[(tmp+1):tmp1])+b1[k])
              den <- cbind(den, d)
            }
            den <- apply(den,1,sum)

            numPersons <- dim(theta)[1]

            p <- array(dim=c(numPersons, m@outcomes))

            for (k in 1:m@outcomes) {
              tmp <- (k-1)*m@factors
              tmp1 <- tmp+m@factors
              if (k==1) {
                cp <- (exp((theta %*% a1[(tmp+1):tmp1])+b1[k]))/den
              } else {
                cp <- (exp((theta %*% a1[(tmp+1):tmp1])+b1[k])+c1[k-1]*(exp((theta %*% a1[1:m@factors])+b1[1])))/den
              }
              p[,k] <- cp
            }
            return(p)
          })

setMethod("rpf.rparam", signature(m="rpf.mdim.mcm"),
          function(m) {
              a <- rlnorm(m@outcomes * m@factors,
                          meanlog=0, sdlog=m@a.prior.sdlog)
              b <- sort(rnorm(m@outcomes))
              c <- rbeta(m@outcomes-1, shape1=m@c.prior.alpha-2,
                         shape2=m@c.prior.beta-2)
              c(a=a,b=b,c=c)
          })

##' Create a graded response model
##'
##' For outcomes k in 0 to K, slope vector a, intercept vector c, and latent ability vector theta,
##' the response probability function is
##' \deqn{\mathrm P(\mathrm{pick}=0|a,c,\theta) = 1- \mathrm P(\mathrm{pick}=1|a,c_1,\theta)
##' }{P(pick=0|a,c,th) = 1-P(pick=1|a,c_1,th)}
##' \deqn{\mathrm P(\mathrm{pick}=k|a,c,\theta) = \frac{1}{1+\exp(-(a\theta + c_k))} - \frac{1}{1+\exp(-(a\theta + c_{k+1}))}
##' }{P(pick=k|a,c,th) = 1/(1+exp(-(a th + c_k))) - 1/(1+exp(-(a th + c_(k+1))))}
##' \deqn{\mathrm P(\mathrm{pick}=K|a,c,\theta) = \frac{1}{1+\exp(-(a\theta + c_K))}
##' }{P(pick=K|a,c,th) = 1/(1+exp(-(a th + c_K)))}
##'
##' The graded response model was designed for a item with a series of
##' dependent parts where a higher score implies that easier parts of
##' the item were surmounted. If there is any chance your polytomous
##' item has independent parts then consider \code{\link{rpf.nrm}}.
##' If your categories cannot cross then the graded response model
##' provides a little more information than the nominal model.
##' Stronger a priori assumptions offer provide more power at the cost
##' of flexibility.
##'
##' @param outcomes The number of choices available
##' @param factors the number of factors
##' @param multidimensional whether to use a multidimensional model.
##' Defaults to \code{TRUE}.
##' @return an item model
##' @family response model
##' @examples
##' spec <- rpf.grm()
##' rpf.prob(spec, rpf.rparam(spec), 0)
rpf.grm <- function(outcomes=2, factors=1, multidimensional=TRUE) {
  if (!multidimensional && factors > 1) {
    stop("More than 1 dimension must use a multidimensional model")
  }
  m <- NULL
  id <- -1
  if (!multidimensional) {
    stop("The old parameterization is no longer available")
  } else {
    id <- rpf.id_of("grm")
    m <- new("rpf.mdim.grm",
             outcomes=outcomes,
             factors=factors)
  }
  m@spec <- c(id, m@outcomes, m@factors)
  m
}

### mdim

setMethod("rpf.modify", signature(m="rpf.mdim.graded", factors="numeric"),
          function(m, factors) {
              rpf.grm(m@outcomes, factors)
          })

setMethod("rpf.rparam", signature(m="rpf.mdim.graded"),
          function(m, version) {
		  if (m@factors == 0) {
			  b <- rnorm(m@outcomes-1)
			  b <- b[order(-b)]
			  return(c(b=b))
		  }
		  if (version == 1L) {
			  a <- rlnorm(m@factors, meanlog=0, sdlog=.5)
			  b <- rnorm(m@outcomes-1)
			  b <- b[order(-b)]
			  c(a=a,b=b)
		  } else {
			  a <- rlnorm(m@factors, meanlog=log(4+m@outcomes)-1.7, sdlog=.5)
			  width <- runif(1, 1, 2)
			  mid <- rnorm(1)
			  thresh <- seq(-width,width,length.out=m@outcomes-1)
			  tvar <- runif(1, .01, .1)
			  b <- mid + thresh + rnorm(length(thresh), sd=sqrt(tvar))
			  b <- b[order(-b)] * sqrt(sum(a^2))
			  c(a=a,b=b)
		  }
          })

# See Schilling & Bock (2005, pp. 8-9)

library(testthat)
library(rpf)

context("rescale")

genOrthogonal<-function(dim) { 
  Q<-MOrthogonal(runif(dim))
  return(Q)
}

# Construct an orthogonal matrix whose first few columns are standardized 'M'
# where columns of 'M' are orthogonal.
# Here "standardized 'M'" means each its columns has length 1.
MOrthogonal<-function(M)
{
  # can set the parameter "tol" of "qr" to decide how small value should be 0
  tmp<-qr(M)
  Q<-qr.Q(tmp,complete=TRUE)
  if(is.vector(M)) { if(Q[1]*M[1]<0) Q<- -Q }
  else { if(Q[1,1]*M[1,1]<0) Q<- - Q }
  return(Q)
}

# adapted from clusterGeneration 1.3.1 by Weiliang Qiu, Harry Joe
genPositiveDefMat <- function(dim, low=-1.4, upp=1.4) {
  u<-matrix(0, dim,dim)
  egvalues <- exp(runif(dim,min=low,max=upp))
  diag(u)<-egvalues #the diagonal elements of u are positive
  Sigma<-u
  if(dim>1)
  { Q<-genOrthogonal(dim) # generate an orthogonal matrix 
    Sigma<-Q%*%u%*%t(Q) # the final positive definite matrix
  }
  Sigma
}

for (dims in 1:3) {
  test_that(paste(dims, "dims"), {
    spec <- list()
    spec[[1]] <- rpf.drm(factors=dims, multidimensional=TRUE)
    spec[[2]] <- rpf.grm(factors=dims, outcomes = 3, multidimensional=TRUE)
    spec[[3]] <- rpf.nrm(factors=dims, T.a="random", T.c="random")
    numItems <- length(spec)
    
    test.point <- rnorm(dims)
    param <- list()
    prob <- list()
    for (ix in 1:numItems) {
      param[[ix]] <- rpf.rparam(spec[[ix]])
      prob[[ix]] <- rpf.prob(spec[[ix]], param[[ix]], test.point)
    }
    
    for (ix in 1:numItems) {
      info <- paste("  (While testing item model", class(spec[[ix]]), ")")
      
      nomove <- rep(0, dims)
      padj <- rpf.rescale(spec[[ix]], param[[ix]], nomove, diag(dims))
      prob.adj <- rpf.prob(spec[[ix]], padj, test.point)
      expect_equal(prob.adj, prob[[ix]], 1e-3, label="Unmoved params",
                   info=info)
      
      move <- rnorm(dims)
      padj <- rpf.rescale(spec[[ix]], param[[ix]], move, diag(dims))
      prob.adj <- rpf.prob(spec[[ix]], padj, test.point-move)
      expect_equal(prob.adj, prob[[ix]], 1e-3, label="Moved params", info=info)
      
      cov <- genPositiveDefMat(dims) * lower.tri(diag(dims), TRUE)
      Icov <- t(solve(cov))
      
      padj <- rpf.rescale(spec[[ix]], param[[ix]], nomove, cov)
      prob.adj <- rpf.prob(spec[[ix]], padj, t(test.point %*% Icov))
      expect_equal(prob.adj, prob[[ix]], 1e-3, label="Covariance scaled params", info=info)
      
      padj <- rpf.rescale(spec[[ix]], param[[ix]], move, cov)
      prob.adj <- rpf.prob(spec[[ix]], padj, t(t(test.point-move) %*% Icov))
      expect_equal(prob.adj, prob[[ix]], 1e-3,
                   label="Moved and covariance scaled params", info=info)
    }
  })
}

if (0) {
  sum((param[[ix]][1:2]) * test.point)
  sum((param[[ix]][1:2] %*% cov) * (test.point %*% t(solve(cov))))
}

library(rpf)
library(testthat)

context("extremes")

test_that("param info", {
  ans1 <- structure(list("slope", NA_real_, 1e-06,
                         "slope", NA_real_, 1e-06,
                         "intercept", NA_real_, NA_real_,
                         "bound", NA_real_, NA_real_,
                         "bound", NA_real_, NA_real_),
                    .Dim = c(3L,  5L), .Dimnames = list(c("type", "upper", "lower"), NULL))
  expect_identical(rpf.paramInfo(rpf.drm(factors=2)), ans1)
  
  ans2 <- structure(list("slope", NA_real_, 1e-06, "slope", NA_real_, 1e-06,
                         "intercept", NA_real_, NA_real_, "intercept", NA_real_, NA_real_),
                    .Dim = 3:4, .Dimnames = list(     c("type", "upper", "lower"), NULL))
  expect_identical(rpf.paramInfo(rpf.grm(outcomes=3, factors=2)), ans2)
  
  ans3 <- structure(list("slope", NA_real_, 1e-06, "slope", NA_real_, 1e-06, "slope",
                         NA_real_, NA_real_, "slope", NA_real_, NA_real_, "slope", NA_real_,
                         NA_real_, "intercept", NA_real_, NA_real_, "intercept", NA_real_, NA_real_,
                         "intercept", NA_real_, NA_real_),
                    .Dim = c(3L, 8L), .Dimnames = list(     c("type", "upper", "lower"), NULL))
  expect_identical(rpf.paramInfo(rpf.nrm(outcomes=4, factors=2)), ans3)
})

spec <- list()
param <- list()
# repair the poor version of drm TODO
#spec [[length(spec) +1]] <- rpf.drm(poor=TRUE)
#param[[length(param)+1]] <- c(1, 0, 0)
spec [[length(spec) +1]] <- rpf.drm()
param[[length(param)+1]] <- c(1, 0, logit(.05), logit(.95))

spec [[length(spec) +1]] <- rpf.grm(3)
param[[length(param)+1]] <- c(1, 1, -1)

spec [[length(spec) +1]] <- rpf.nrm(3)
param[[length(param)+1]] <- c(1,  .5, .6, 0, -.6)

# To debug, set breakpoint on Rf_error 
where <- seq(-1000, 1000, 100)
for (ix in 1:length(spec)) {
  ispec <- spec[[ix]]
  iparam <- param[[ix]]
  test_that(paste("extreme values in", class(ispec)), {
    for (wh in where) {
      v <- rpf.prob(ispec, iparam, wh)
      expect_equal(sum(v), 1)
      if (wh != 0) {
        sep <- sort(-v)[1:2]
        expect_true(abs(sep[1] - sep[2]) > .89,
                    info=paste(c(wh, sep), collapse=" "))
      }
    }
    for (wh in where) {
      v <- rpf.logprob(ispec, iparam, wh)
      if (all(v > -35 & v < 35)) {
        expect_equal(sum(exp(v)), 1)
      }
    }
    w <- rchisq(ispec$outcomes, df=6)
    for (wh in where) rpf.dLL(ispec, iparam, wh, w)
  })
}

for (ix in 1:length(spec)) {
  ispec <- spec[[ix]]
  iparam <- param[[ix]]
  test_that(paste("score=NA", class(ispec)), {
    v <- rpf.prob(ispec, iparam, as.numeric(c(NA,NA)))
    expect_true(all(is.na(v)))
    v <- rpf.logprob(ispec, iparam, as.numeric(c(NA,NA)))
    expect_true(all(is.na(v)))
  })
}

##' Create a dichotomous response model
##'
##' For slope vector a, intercept c, pseudo-guessing parameter g,
##' upper bound u, and latent ability vector theta, the response probability
##' function is
##' \deqn{\mathrm P(\mathrm{pick}=0|a,c,g,u,\theta) = 1- \mathrm P(\mathrm{pick}=1|a,c,g,u,\theta)
##' }{P(pick=0|a,c,g,u,th) = 1-P(pick=1|a,c,g,u,th)}
##' \deqn{\mathrm P(\mathrm{pick}=1|a,c,g,u,\theta) = g+(u-g)\frac{1}{1+\exp(-(a\theta + c))}
##' }{P(pick=1|a,c,g,u,th) = g+(u-g)/(1+exp(-(a th + c)))}
##'
##' The pseudo-guessing and upper bound parameter are specified in
##' logit units (see \code{\link{logit}}).
##'
##' For discussion on the choice of priors see Cai, Yang, and
##' Hansen (2011, p. 246).
##'
##' @param factors the number of factors
##' @param multidimensional whether to use a multidimensional model.
##' Defaults to \code{TRUE}.
##' @param poor if TRUE, use the traditional parameterization of
##' the 1d model instead of the slope-intercept parameterization
##' @return an item model
##' @family response model
##' @references Cai, L., Yang, J. S., & Hansen, M. (2011). Generalized
##' Full-Information Item Bifactor Analysis.  \emph{Psychological
##' Methods, 16}(3), 221-248.
##' @examples
##' spec <- rpf.drm()
##' rpf.prob(spec, rpf.rparam(spec), 0)
rpf.drm <- function(factors=1, multidimensional=TRUE, poor=FALSE) {
  if (!multidimensional && factors > 1) {
    stop("More than 1 dimension must use a multidimensional model")
  }
  m <- NULL
  id <- -1
  if (!multidimensional) {
    if (!poor) stop("The old parameterization is no longer available")
    id <- rpf.id_of("drm1-")
    m <- new("rpf.1dim.drm",
             outcomes=2,
             factors=1)
  } else {
    id <- rpf.id_of("drm")
    m <- new("rpf.mdim.drm",
             outcomes=2,
             factors=factors)
  }
  m@spec <- c(id, 2, m@factors)
  m
}

### 1dim

setMethod("rpf.rparam", signature(m="rpf.1dim.drm"),
          function(m, version) {
            n <- 1
            c(a=rlnorm(n, meanlog=0, sdlog=.5),
              b=rnorm(n),
              g=rbeta(1, 5,17),
              u=rbeta(1, 17,5))
          })

### mdim

setMethod("rpf.modify", signature(m="rpf.mdim.drm", factors="numeric"),
          function(m, factors) {
              rpf.drm(factors)
          })

##' Transform from [0,1] to the reals
##'
##' The logit function is a standard transformation from [0,1] (such
##' as a probability) to the real number line. This function is
##' exactly the same as qlogis.
##'
##' @param p a number between 0 and 1
##' @param location see qlogis
##' @param scale see qlogis
##' @param lower.tail see qlogis
##' @param log.p see qlogis
##' @examples
##' logit(.5)  # 0
##' logit(.25) # -1.098
##' logit(0)   # -Inf
##' @seealso
##' qlogis, plogis
logit <- qlogis

setMethod("rpf.rparam", signature(m="rpf.mdim.drm"),
          function(m, version) {
		  if (m@factors == 0) {
			  return(c(b=rnorm(1)))
		  }
		  if (version == 1L) {
			  c(a=rlnorm(m@factors, meanlog=0, sdlog=.5),
			    b=rnorm(1),
			    g=logit(rbeta(1, 5,17)),
			    u=logit(rbeta(1, 17,5)))
		  } else {
			  a <- rlnorm(m@factors, meanlog=0, sdlog=.5)
			  b <- rnorm(1) * sqrt(sum(a^2))
			  g <- logit(rbeta(1, 5,17))
			  u <- logit(rbeta(1, 17,5))
			  c(a=a, b=b, g=g, u=u)
		  }
          })

# Not sure if this is correct because of rotation
## as.loadings <- function(m, param) {
##   loading <- vector(mode="numeric", m@factors)
##   for (d in 1:m@factors) {
##     loading[d] <- param[d] / sqrt(1+sum(param[d:m@factors]^2))
##   }
##   loading
## }

library(testthat)
library(rpf)
#options(error = utils::recover)

context("lmp ICC")

test_that("LMP", {

  ## Check vs. 2PL from same package
  lmp.k0<-rpf.lmp(q=0)
  twopl<-rpf.drm()
  par<-c(.7,-.1)
  par2<-c(exp(.7),-.1,logit(0),logit(1))
  expect_equal(rpf.prob(lmp.k0, par, c(-1,0,1)), rpf.prob(twopl, par2, c(-1,0,1)))

  ## 3rd order polynomial
  lmp.k1 <- rpf.lmp(q=1)

  ## Response probabilities manually copied from code used for Falk & Cai
  lmp.k1.values<-matrix(c(.86737623446,.52497918748,.00005929256,
                          .1326238,.4750208,.99994070744),
                        byrow=TRUE,nrow=2)
  par<-c(.7,-.1,-2,.5)

  ## Since not obtaining above from external code, there will be some minor
  ## differences if we compare to too many decimal places
  expect_equal(lmp.k1.values,rpf.prob(lmp.k1, par, c(-1,0,1)), tolerance=1e-7)

  ## 5th order polynomial
  lmp.k2 <- rpf.lmp(q=2)

  ## Response probabilities manually copied from code used for Falk & Cai
  lmp.k2.values<-matrix(c(.9998936,.6224593,.4339060,
                          .0001063994,.3775406677,.5660940386),
                        byrow=TRUE,nrow=2)
  par<-c(.6,-.5,.4,-.2,1,-1)

  ## Since not obtaining above from external code, there will be some minor
  ## differences if we compare to too many decimal places
  expect_equal(lmp.k2.values,rpf.prob(lmp.k2, par, c(-1,0,1)), tolerance=1e-7)

})

library(rpf)
library(ggplot2)
library(reshape2)
library(testthat)
#options(digits=22)
#options(width=200)

# need to integrate TODO
inspect.gradient <- function(spec, param) {
  w <- seq(1, spec@numOutcomes) * 5.0
  samples <- 50
  got <- list()
  for (px in 1:length(param)) {
    orig <- param[px]
    df1 <- data.frame(grid=orig + seq(-.5, .5, length.out=samples+1))
    for (gx in 1:length(df1$grid)) {
      param[px] <- df1$grid[gx]
      df1$LL[gx] <- sum(w * rpf.logprob(spec, param, loc))
    }
    df2 <- data.frame(empirical=diff(df1$LL) / diff(df1$grid),
                      grid=df1$grid[1:samples] + diff(df1$grid)/2)
    for (gx in 1:length(df2$grid)) {
      param[px] <- df2$grid[gx]
      df2$analytic[gx] <- rpf.gradient(spec, param, loc, w)[px]
    }
    df2$px <- px
    param[px] <- orig
    got <- rbind(got, df2)
  }
  got$px <- factor(got$px)
  as.data.frame(got)
}

plot.gradient <- function(df, px) {
  df2 <- df[df$px == px,]
  df2 <- melt(df2, id.vars=c('px', 'grid'), value.name="dLL", variable.name="type")
  spot <- df2[quantile(1:dim(df2)[[1]], probs=seq(0,1,.2)),]
  
  ggplot(df2, aes(grid, dLL, color=type)) + geom_line() +
    geom_point(data=spot, aes(x=grid,y=dLL), alpha=.5, size=3, color='yellow')
}

plot.gradients <- function(df, px) {
  df2 <- df[df$px == px,]
  df2 <- df2[,c('empirical', 'grid')]
  colnames(df2)[[1]] <- "dLL"
  df2$name <- paste("empirical", px, sep="")
  df3 <- df[,c('analytic', 'grid', 'px')]
  colnames(df3)[[1]] <- "dLL"
  df3$name <- paste("analytic", df3$px, sep="")
  df3$px <- NULL
  plot.data <- rbind(df2, df3)
  plot.data$name <- factor(plot.data$name)
  spot <- df2[quantile(1:dim(df2)[[1]], probs=seq(0,1,.2)),]
  ggplot(plot.data, aes(grid, dLL, color=name)) + geom_line() +
    geom_point(data=spot, aes(x=grid,y=dLL), alpha=.5, size=3, color='yellow')
}

spec <- list()
param <- list()
spec [[length(spec) +1]] <- rpf.drm(poor=TRUE)
param[[length(param)+1]] <- c(1, 0, 0)
spec [[length(spec) +1]] <- rpf.drm(multidimensional=TRUE)
param[[length(param)+1]] <- c(1, 0, 0)
spec [[length(spec) +1]] <- rpf.gpcm(3)
param[[length(param)+1]] <- c(1, -10, 10)

#for (mx in 1:length(spec)) {
  mx<-3
  ispec <- spec[[mx]]
  iparam <- param[[mx]]
  df <- inspect.gradient(ispec, iparam)
  expect_equal(df$empirical, df$analytic, tolerance = 1e-4)
#}

# diagnostic tools
if (0) {
  plot.gradient(df, 2)
  plot.gradients(df, 1)
}

##' Convert an OpenMx MxModel object into an IFA group
##'
##' When \dQuote{minItemsPerScore} is passed, EAP scores will be computed
##' from the data and stored. Scores are required for some diagnostic
##' tests. See discussion of \dQuote{minItemsPerScore} in \link{EAPscores}.
##'
##' @template detail-group
##' @param mxModel MxModel object
##' @param data observed data (otherwise the data will be taken from the mxModel)
##' @param container an MxModel in which to search for the latent distribution matrices
##' @template arg-dots
##' @param minItemsPerScore minimum number of items required to compute a score (also see description)
##' @return a groups with item parameters and latent distribution
##' @seealso \href{https://cran.r-project.org/package=ifaTools}{ifaTools}
as.IFAgroup <- function(mxModel, data=NULL, container=NULL, ..., minItemsPerScore=NULL) {
	if (length(list(...)) > 0) {
		stop(paste("Remaining parameters must be passed by name", deparse(list(...))))
	}
	expectation <- mxModel$expectation
  if (!is(expectation, "MxExpectationBA81")) {
    stop(paste("Don't know how to create an IFA group from",
               class(expectation)))
  }
	if (missing(container)) {
		container <- mxModel
	}

  mat <- expectation$item
  if (length(grep("\\.", mat))) {
    stop(paste("Don't know how to obtain the item matrix", mat))
  }
  itemMat <- mxModel[[mat]]
  if (is.null(itemMat)) {
    stop(paste("Item matrix", mat, "not found"))
  }

  ret <- list(spec = expectation$ItemSpec,
              param = itemMat$values,
              free = itemMat$free,
	      labels = itemMat$labels,
	      # TODO maybe should include free variables in latent distribution?
	      uniqueFree = length(unique(itemMat$labels[itemMat$free], incomparables=NA)),
              qpoints = expectation$qpoints,
              qwidth = expectation$qwidth)

  mat <- expectation$mean
  if (length(grep("\\.", mat))) {
    meanMat <- eval(substitute(mxEval(theExpression, container),
			       list(theExpression = parse(text = mat)[[1]])))
    if (is.null(meanMat)) {
      stop(paste("Don't know how to obtain the mean matrix", mat))
    }
    ret$mean <- meanMat
  } else {
	  mxMat <- mxModel[[mat]]
	  if (!is.null(mxMat)) {
		  ret$mean <- mxMat$values
	  }
  }

  mat <- expectation$cov
  if (length(grep("\\.", mat))) {
    covMat <- eval(substitute(mxEval(theExpression, container),
				   list(theExpression = parse(text = mat)[[1]])))
    if (is.null(covMat)) {
      stop(paste("Don't know how to obtain the cov matrix", mat))
    }
    ret$cov <- covMat
  } else {
	  mxMat <- mxModel[[mat]]
	  if (!is.null(mxMat)) {
		  ret$cov <- mxMat$values
	  }
  }

  if (!missing(data)) {
    ret$data <- data
  } else if (!is.null(mxModel$data)) {
    mxData <- mxModel$data
    if (mxData$type != "raw") {
      stop(paste("Not sure how to handle data of type", mxData$type))
    }
    if (!is.null(mxData$frequency) && !is.na(mxData$frequency)) {
	    ret$freqColumn <- mxData$frequency
    }
    if (!is.null(mxData$weight) && !is.na(mxData$weight)) {
	    ret$weightColumn <- mxData$weight
    }
    ret$data <- mxData$observed
  }

  if (!is.na(expectation$weightColumn)) {
    ret$weightColumn <- expectation$weightColumn
    ret$observedStats <- nrow(ret$data) - 1
  } else {
	  if (ncol(ret$param) == ncol(ret$data)) {
		  freq <- tabulateRows(ret$data[orderCompletely(ret$data),])
		  ret$observedStats <- length(freq) - 1L
	  }
  }

  if (max(sapply(ret$spec, function(s) s$factors)) > 0 && !missing(minItemsPerScore)) {
    ret$minItemsPerScore <- minItemsPerScore
    ret$score <- EAPscores(ret, compressed=TRUE)
  }

  ret
}

#' Strip data and scores from an IFA group
#'
#' In addition, the freqColumn and weightColumn are reset to NULL.
#'
#' @template arg-grp
#' @template detail-group
#' @return
#' The same group without associated data.
#' @examples
#' spec <- list()
#' spec[1:3] <- list(rpf.grm(outcomes=3))
#' param <- sapply(spec, rpf.rparam)
#' data <- rpf.sample(5, spec, param)
#' colnames(param) <- colnames(data)
#' grp <- list(spec=spec, param=param, data=data, minItemsPerScore=1L)
#' grp$score <- EAPscores(grp)
#' str(grp)
#' grp <- stripData(grp)
#' str(grp)
stripData <- function(grp) {
	grp$data <- NULL
	grp$score <- NULL
	grp$weightColumn <- NULL
	grp$freqColumn <- NULL
	grp
}

# Copied from flexmirt.R
Tnom.trend <- function(nc) {
  T <- matrix(0,nc,nc-1)
  for (i in 1:nc) {
    T[i,1] <- i-1
  }
  for (k in 2:(nc-1)) {
    for (i in 2:(nc-1)) {
      T[k,i] <- sin(pi*(i-1)*(k-1)/(nc-1))
    }
  }
  return(T[-1,])
}

# Copied from flexmirt.R
Tnom.ida <- function(nc) {
  T <- matrix(0,nc,nc-1)
  T[nc,1] <- nc-1
  T[2:(nc-1),2:(nc-1)] <- diag(nc-2)
  return(T[-1,])
}

# Copied from flexmirt.R
Tnom.idc <- function(nc) {
  T <- matrix(0,nc,nc-1)
  T[2:nc,] <- diag(nc-1)
  return(T[-1,])
}

build.T <- function(outcomes, got, type) {
	if (is.null(got)) stop("rpf.nrm: T matrix specified as NULL")
  if (!is.matrix(got)) {
    if (got == "id") {
	    if (type == 'a') {
		    got <- Tnom.ida(outcomes)
	    } else {
		    got <- Tnom.idc(outcomes)
	    }
    } else if (got == "trend") {
      got <- Tnom.trend(outcomes)
    } else if (got == "random") {
      while (1) {
        side <- outcomes-1
        got <- matrix(rnorm(side*side), side, side)
        invertible <- try(solve(got), silent=TRUE)
        if (!inherits(invertible, "try-error")) break
      }
    } else {
      stop(paste("T matrix", deparse(got), "not recognized"))
    }
  }
  if (all(dim(got) == c(outcomes,outcomes-1))) {
    if (any(got[1,] != 0)) warning("Non-zero T[1,] will be ignored")
    got <- got[-1,]
  }
  if (all(dim(got) != rep(outcomes-1, 2))) {
    stop(paste("T matrix must be of dimension",
               paste(rep(outcomes-1, 2), collapse="x"),
               "not", paste(dim(got), collapse="x")))
  }
  got
}

##' Create a nominal response model
##'
##' This function instantiates a nominal response model.
##'
##' The transformation matrices T.a and T.c are chosen by the analyst
##' and not estimated.  The T matrices must be invertible square
##' matrices of size outcomes-1. As a shortcut, either T matrix
##' can be specified as "trend" for a Fourier basis or as "id" for an
##' identity basis. The response probability function is
##'
##' \deqn{a = T_a \alpha}
##' \deqn{c = T_c \gamma}
##' \deqn{\mathrm P(\mathrm{pick}=k|s,a_k,c_k,\theta) = C\ \frac{1}{1+\exp(-(s \theta a_k + c_k))}}
##'
##' where \eqn{a_k} and \eqn{c_k} are the result of multiplying two vectors
##' of free parameters \eqn{\alpha} and \eqn{\gamma} by fixed matrices \eqn{T_a} and \eqn{T_c}, respectively;
##' \eqn{a_0} and \eqn{c_0} are fixed to 0 for identification;
##' and \eqn{C} is a normalizing factor to ensure that \eqn{\sum_k \mathrm P(\mathrm{pick}=k) = 1}.
##'
##' @param outcomes The number of choices available
##' @param factors the number of factors
##' @param T.a the T matrix for slope parameters
##' @param T.c the T matrix for intercept parameters
##' @return an item model
##' @family response model
##' @references Thissen, D., Cai, L., & Bock, R. D. (2010). The
##' Nominal Categories Item Response Model. In M. L. Nering &
##' R. Ostini (Eds.), \emph{Handbook of Polytomous Item Response
##' Theory Models} (pp. 43--75). Routledge.
##' @examples
##' spec <- rpf.nrm()
##' rpf.prob(spec, rpf.rparam(spec), 0)
##' # typical parameterization for the Generalized Partial Credit Model
##' gpcm <- function(outcomes) rpf.nrm(outcomes, T.c=lower.tri(diag(outcomes-1),TRUE) * -1)
##' spec <- gpcm(4)
##' rpf.prob(spec, rpf.rparam(spec), 0)
rpf.nrm <- function(outcomes=3, factors=1, T.a="trend", T.c="trend") {
  if (outcomes < 3) stop("Minimum number of outcomes is 3")
  T.a <- build.T(outcomes, T.a, 'a')
  T.c <- build.T(outcomes, T.c, 'c')
  id <- rpf.id_of("nominal")
  m <- new("rpf.mdim.nrm",
           outcomes=outcomes,
           factors=factors)
  m@spec <- c(id, outcomes, factors, T.a, T.c, solve(T.a), solve(T.c))
  m
}

getT <- function(m, tx) {
  Tsize <- (m@outcomes-1L)^2
  offset <- 4 + tx * Tsize
  matrix(m@spec[offset:(offset+Tsize-1)], m@outcomes-1L, m@outcomes-1L)
}

setMethod("rpf.rparam", signature(m="rpf.mdim.nrm"),
          function(m, version) {
		  if (m@factors == 0) {
			  return(c(gam=ck <- sort(rnorm(m@outcomes-1))))
		  }
            a <- rlnorm(m@factors, sdlog=.5)
            ak <- abs(rnorm(m@outcomes-1, mean=1, sd=.25))
            ck <- sort(rnorm(m@outcomes-1))
            c(a=a,
              alf=getT(m,2) %*% ak,
              gam=getT(m,3) %*% ck)
          })

setMethod("rpf.modify", signature(m="rpf.mdim.nrm", factors="numeric"),
          function(m, factors) {
              rpf.nrm(m@outcomes, factors, T.a=getT(m,0), T.c=getT(m,1))
          })

#' Description of LSAT7 data
#'
#' Data from Bock & Lieberman (1970); contains 5 dichotomously scored
#' items obtained from the Law School Admissions Test, section 7.
#'
#' @name LSAT7
#' @docType data
#' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
#' @references
#' Bock, R. D., & Lieberman, M. (1970). Fitting a response model for \emph{n}
#' dichotomously scored items. \emph{Psychometrika, 35}(2), 179-197.
#'
#' @keywords data
#' @examples
#' data(LSAT7)
NULL

#' Description of LSAT6 data
#'
#' Data from Thissen (1982); contains 5 dichotomously scored
#' items obtained from the Law School Admissions Test, section 6.
#'
#' @name LSAT6
#' @docType data
#' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
#' @references
#' Thissen, D. (1982). Marginal maximum likelihood estimation for the one-parameter logistic model.
#' \emph{Psychometrika, 47}, 175-186.
#'
#' @keywords data
#' @examples
#' data(LSAT6)
NULL

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

findIdenticalRowsData <- function(data, missing, defvars, skipMissingness, skipDefvars) {
    .Call(`_rpf_findIdenticalRowsData`, data, missing, defvars, skipMissingness, skipDefvars)
}

ot2000 <- function(robj, iPlusOne, alter, mask, twoTier) {
    .Call(`_rpf_ot2000`, robj, iPlusOne, alter, mask, twoTier)
}

ssEAP_wrapper <- function(robj, Rmask, twoTier) {
    .Call(`_rpf_ssEAP_wrapper`, robj, Rmask, twoTier)
}

pairwiseExpected_cpp <- function(robj, items, twoTier) {
    .Call(`_rpf_pairwiseExpected_cpp`, robj, items, twoTier)
}

collapse <- function(r_observed_orig, r_expected_orig, r_min) {
    .Call(`_rpf_collapse`, r_observed_orig, r_expected_orig, r_min)
}

fast_tableWithWeights <- function(Ritem1, Ritem2, Rweight) {
    .Call(`_rpf_fast_tableWithWeights`, Ritem1, Ritem2, Rweight)
}

observedSumScore_cpp <- function(Rgrp, Rmask) {
    .Call(`_rpf_observedSumScore_cpp`, Rgrp, Rmask)
}

itemOutcomeBySumScore_cpp <- function(Rgrp, Rmask, interestPlusOne) {
    .Call(`_rpf_itemOutcomeBySumScore_cpp`, Rgrp, Rmask, interestPlusOne)
}

gamma_cor <- function(mat) {
    .Call(`_rpf_gamma_cor`, mat)
}

crosstabTest_cpp <- function(Robserved, Rexpected, trials) {
    .Call(`_rpf_crosstabTest_cpp`, Robserved, Rexpected, trials)
}

eap_wrapper <- function(Rgrp) {
    .Call(`_rpf_eap_wrapper`, Rgrp)
}

get_model_id <- function(str) {
    .Call(`_rpf_get_model_id`, str)
}

numSpec <- function(spec) {
    .Call(`_rpf_numSpec`, spec)
}

numParam <- function(spec) {
    .Call(`_rpf_numParam`, spec)
}

paramInfo <- function(spec, pnum) {
    .Call(`_rpf_paramInfo`, spec, pnum)
}

prob <- function(spec, r_param, r_theta) {
    .Call(`_rpf_prob`, spec, r_param, r_theta)
}

logprob <- function(spec, r_param, r_theta) {
    .Call(`_rpf_logprob`, spec, r_param, r_theta)
}

dLL <- function(spec, r_param, r_where, r_weight) {
    .Call(`_rpf_dLL`, spec, r_param, r_where, r_weight)
}

dTheta <- function(spec, r_param, r_where, r_dir) {
    .Call(`_rpf_dTheta`, spec, r_param, r_where, r_dir)
}

rescale <- function(spec, r_param, r_mean, r_cov) {
    .Call(`_rpf_rescale`, spec, r_param, r_mean, r_cov)
}

has_openmp <- function() {
    .Call(`_rpf_has_openmp`)
}

setNumberOfCores <- function(num) {
    .Call(`_rpf_setNumberOfCores`, num)
}

registerCCallable <- function() {
    invisible(.Call(`_rpf_registerCCallable`))
}

CaiHansen2012_cpp <- function(Rgrp, Rmethod, twotier) {
    .Call(`_rpf_CaiHansen2012_cpp`, Rgrp, Rmethod, twotier)
}

# copied from OpenMx R/MxSummary.R

pChiSqFun <- function(x, val, degf, goal){
	goal - pchisq(val, degf, ncp=x)
}

rmseaConfidenceIntervalHelper <- function(chi.squared, df, N, lower, upper){
	# Lower confidence interval
	if( pchisq(chi.squared, df=df, ncp=0) >= upper){ #sic
		lower.lam <- uniroot(f=pChiSqFun, interval=c(1e-10, 1e4), val=chi.squared, degf=df, goal=upper)$root
		# solve pchisq(ch, df=df, ncp=x) == upper for x
	} else{
		lower.lam <- 0
	}
	# Upper confidence interval
	if( pchisq(chi.squared, df=df, ncp=0) >= lower){ #sic
		upper.lam <- uniroot(f=pChiSqFun, interval=c(1e-10, 1e4), val=chi.squared, degf=df, goal=lower)$root
		# solve pchisq(ch, df=df, ncp=x) == lower for x
	} else{
		upper.lam <- 0
	}
	lower.rmsea <- sqrt(lower.lam/(N*df))
	upper.rmsea <- sqrt(upper.lam/(N*df))
	return(c(lower=lower.rmsea, upper=upper.rmsea))
}

computeFitStatistics <- function(likelihood, DoF, numObs,
				 independence, indDoF, saturated=0, satDoF=0) {
	chi <- likelihood - saturated
	chiDoF <- DoF - satDoF # DoF = obsStat-model.ep; satDoF = obsStat-sat.ep; So sat.ep-model.ep == DoF-satDoF
	CFI <- (independence - indDoF - likelihood + DoF)/(independence - indDoF - saturated + satDoF)
	TLI <- 1
	rmseaSquared <- 0
	RMSEA <- 0
	RMSEACI <- c(lower=NA, upper=NA)
	if (!is.na(chiDoF) && chiDoF > 0) {
		TLI <- ((independence-saturated)/(indDoF-satDoF) - (chi)/(DoF-satDoF))/((independence-saturated)/(indDoF-satDoF) - 1)
					# Here we use N in the denominator as given in the original
					# RMSEA paper. The difference between N and N-1 is negligible
					# for sample sizes over 30. RMSEA should not be taken seriously
					# such small samples anyway.
		rmseaSquared <- (chi / (chiDoF) - 1) / numObs
		RMSEACI <- c(lower=NA, upper=NA)
		if (length(rmseaSquared) == 0 || is.na(rmseaSquared) || 
		    is.nan(rmseaSquared)) { 
					# || (rmseaSquared < 0)) { # changed so 'rmseaSquared < 0' yields zero with comment
			RMSEA <- NA
		} else if (rmseaSquared < 0) {
			RMSEA <- 0.0
		} else {
			RMSEA <- sqrt(rmseaSquared)
			ci <- try(rmseaConfidenceIntervalHelper(chi, chiDoF, numObs, .025, .975))
			if (!inherits(ci, "try-error")) RMSEACI <- ci
		}
	}
	list(CFI=CFI, TLI=TLI, RMSEA=RMSEA, RMSEASquared=rmseaSquared, RMSEACI=RMSEACI)
}

catFitStatistics <- function(x) {
	cat("CFI:", x$CFI, '\n')
	cat("TLI:", x$TLI, '\n')
	if (length(x$RMSEASquared) == 1 && !is.na(x$RMSEASquared) && x$RMSEASquared < 0.0) {
		cat("RMSEA: ", x$RMSEA, '*(Non-centrality parameter is negative)', '\n')
	} else {
		cat("RMSEA:  ", x$RMSEA, "  [95% CI (", x$RMSEACI[1], ", ", x$RMSEACI[2], ")]", '\n', sep="")
	}
}

##' Create monotonic polynomial graded response (GR-MP) model
##'
##' The GR-MP model replaces the linear predictor of the graded
##' response model (Samejima, 1969, 1972) with a monotonic polynomial (Falk, conditionally accepted).
##'
##' Given its relationship to the graded response model, the GR-MP
##' is constructed in an analogous way:
##'
##' \deqn{\mathrm P(\mathrm{pick}=0|\lambda,\alpha,\tau,\theta) = 1- \frac{1}{1+\exp(-(\xi_1 + m(\theta;\lambda,\mathbf{\alpha},\mathbf{\tau})))}
##' }{P(pick=0|a,c,theta) = 1-1/(1+exp(-(xi_1 + m(theta;lambda,alpha,tau)))}
##' \deqn{\mathrm P(\mathrm{pick}=k|\lambda,\alpha,\tau,\theta) = \frac{1}{1+\exp(-(\xi_k + m(\theta;\lambda,\mathbf{\alpha},\mathbf{\tau})))} - \frac{1}{1+\exp(-(\xi_{k+1} + m(\theta,\lambda,\mathbf{\alpha},\mathbf{\tau}))}
##' }{P(pick=k|a,c,theta) = 1/(1+exp(-(xi_k + m(theta;lambda,alpha,tau))) - 1/(1+exp(-(xi_(k+1) + m(theta;lambda,alpha,tau))))}
##' \deqn{\mathrm P(\mathrm{pick}=K|\lambda,\alpha,\tau,\theta) = \frac{1}{1+\exp(-(\xi_K + m(\theta;\lambda,\mathbf{\alpha},\mathbf{\tau}))}
##' }{P(pick=K|\lambda,\alpha,\tau,theta) = 1/(1+exp(-(xi_K + m(theta;lambda,alpha,tau))))}
##'
##' The order of the polynomial is always odd and is controlled by
##' the user specified non-negative integer, q. The model contains
##' 1+(outcomtes-1)+2*q parameters and are used as input to the \code{\link{rpf.prob}}
##' or \code{\link{rpf.dTheta}} functions in the following order:
##' \eqn{\lambda}{lambda} - slope of the item model when q=0,
##' \eqn{\xi}{xi} - a (outcomes-1)-length vector of intercept parameters,
##' \eqn{\alpha}{alpha} and \eqn{\tau}{tau} - two parameters that control bends in
##' the polynomial. These latter parameters are repeated in the same order for
##' models with q>0.  For example, a q=2 polynomial with 3 categories will have an item
##' parameter vector of: \eqn{\lambda, \xi_1, \xi_2, \alpha_1, \tau_1, \alpha_2, \tau_2}{
##' lambda, xi1, xi2, alpha1, tau1, alpha2, tau2}.
##'
##' As with other monotonic polynomial-based item models
##' (e.g., \code{\link{rpf.lmp}}), the polynomial looks like the
##' following:
##'
##' \deqn{m(\theta;\lambda,\alpha,\tau) = b_1\theta + b_2\theta^2 + \dots + b_{2q+1}\theta^{2q+1}
##' }{m(theta) = b_1*theta + b_2*theta^2 + \dots + b_(2q+1)*theta^{2q+1}}
##'
##' However, the coefficients, b, are not directly estimated, but are a function of the
##' item parameters, and the parameterization of the GR-MP is different than
##' that currently appearing for the logistic function of a monotonic
##' polynomial (LMP; \code{\link{rpf.lmp}}) and monotonic polynomial generalized partial credit
##' (GPC-MP; \code{\link{rpf.gpcmp}}) models. In particular, the polynomial is
##' parameterized such that boundary descrimination functions for the GR-MP will
##' be all monotonically increasing or decreasing for any given item. This allows
##' the possibility of items that load either negatively or positively on the latent
##' trait, as is common with reverse-worded items in non-cognitive tests (e.g., personality).
##'
##' The derivative \eqn{m'(\theta;\lambda,\alpha,\tau)}{m'(theta;lambda,alpha,tau)} is
##' parameterized in the following way:
##'
##'\deqn{ m'(\theta;\lambda,\alpha,\tau) = \left\{\begin{array}{ll}\lambda \prod_{u=1}^q(1-2\alpha_{u}\theta + (\alpha_{u}^2 + \exp(\tau_{u}))\theta^2) &  \mbox{if } q > 0 \\
##'  \lambda & \mbox{if } q = 0\end{array} \right.}{m'(theta) = m'(theta;lambda,alpha,tau) = lambda \prod_{u=1}^q (1-2*alpha_u*theta + (alpha_u^2 + exp(tau_u))*theta^2) (if q > 0) \\
##'  lambda (if q = 0)}
##'
##' Note that the only difference between the GR-MP and these other models
##' is that \eqn{\lambda}{lambda} is not re-parameterized and may take on
##' negative values. When \eqn{\lambda}{lambda} is negative, it is analogous
##' to having a negative loading or a monotonically decreasing function.
##'
##' @param outcomes The number of possible response categories. When equal to 2, the model reduces
##' to the logistic function of a monotonic polynomial (LMP).
##' @param q a non-negative integer that controls the order of the
##' polynomial (2q+1) with a default of q=0 (1st order polynomial = graded response model).
##' @param multidimensional whether to use a multidimensional model.
##' Defaults to \code{FALSE}. The multidimensional version is not yet
##' available.
##' @return an item model
##' @references Falk, C. F. (conditionally accepted). The monotonic polynomial
##' graded response model: Implementation and a comparative study. \emph{Applied Psychological
##' Measurement}.
##' @references Samejima, F. (1969). Estimation of latent ability using a response pattern of graded scores. \emph{Psychometric Monographs}, 17.
##' @references Samejima, F. (1972). A general model of free-response data. \emph{Psychometric Monographs}, 18.
##'
##' @family response model
##'
##' @examples
##' spec <- rpf.grmp(5,2) # 5-category, 3rd order polynomial
##' theta<-seq(-3,3,.1)
##' p<-rpf.prob(spec, c(2.77,2,1,0,-1,.89,-8.7,-.74,-8.99),theta)

rpf.grmp <- function(outcomes=2, q=0, multidimensional=FALSE) {
  if(!(q%%1==0)){
    stop("q must be an integer >= 0")
  }
  if(multidimensional){
      stop("Multidimensional grmp model is not yet supported")
  }
  m <- NULL
  id <- -1
  id <- rpf.id_of("grmp")
  m <- new("rpf.1dim.grmp",
           outcomes=outcomes,
           factors=1)
  m@spec <- c(id, m@outcomes, m@factors, q)
  m
}

setMethod("rpf.rparam", signature(m="rpf.1dim.grmp"),
          function(m, version) {
            n <- 1
            q<-m$spec[4] ## ok to hardcode this index?
            ret<-c(lambda=rlnorm(n, 0, .5)) # random overall slope
            # randomly generate xi
            xi <- rnorm(m@outcomes-1)
            xi <- xi[order(-xi)]
            ret<-c(ret,xi=xi)
            if(q>0){
                for(i in 1:q){
                    ret<-c(ret,runif(n,-1,1),log(runif(n,.0001,1)))
                    names(ret)[(m@outcomes+1+(i-1)*2):(m@outcomes+(i*2))]<-c(paste("alpha",i,sep=""),paste("tau",i,sep=""))
                }
            }
            ret
        })

##' Create logistic function of a monotonic polynomial (LMP) model
##'
##' This model is a dichotomous response model originally proposed by
##' Liang (2007) and is implemented using the parameterization by
##' Falk & Cai (2016).
##'
##' The LMP model replaces the linear predictor part of the
##' two-parameter logistic function with a monotonic polynomial,
##' \eqn{m(\theta,\omega,\xi,\mathbf{\alpha},\mathbf{\tau})}{m(theta; omega, alpha, tau)},
##'
##' \deqn{\mathrm P(\mathrm{pick}=1|\omega,\xi,\mathbf{\alpha},\mathbf{\tau},\theta)
##' = \frac{1}{1+\exp(-(\xi + m(\theta;\omega,\mathbf{\alpha},\mathbf{\tau})))}
##' }{P(pick=1|omega,xi,alpha,tau,th) = 1/(1+exp(-(xi + m(theta;omega,alpha,tau))))}
##'
##' where \eqn{\mathbf{\alpha}}{alpha} and \eqn{\mathbf{\tau}}{tau} are vectors
##' of length q.
##'
##' The order of the polynomial is always odd and is controlled by
##' the user specified non-negative integer, q. The model contains
##' 2+2*q parameters and are used in conjunction with the \code{\link{rpf.prob}}
##' or \code{\link{rpf.dTheta}} function in the following order:
##' \eqn{\omega}{omega} - the natural log of the slope of the item model when q=0,
##' \eqn{\xi}{xi} - the intercept,
##' \eqn{\alpha}{alpha} and \eqn{\tau}{tau} - two parameters that control bends in
##' the polynomial. These latter parameters are repeated in the same order for
##' models with q>0. For example, a q=2 polynomial with have an item
##' parameter vector of: \eqn{\omega, \xi, \alpha_1, \tau_1, \alpha_2, \tau_2}{
##' omega, xi, alpha1, tau1, alpha2, tau2}.
##'
##' In general, the polynomial looks like the following:
##'
##' \deqn{m(\theta;\omega,\alpha,\tau) = b_1\theta + b_2\theta^2 + \dots + b_{2q+1}\theta^{2q+1}
##' }{m(theta;omega,alpha,tau) = b_1*theta + b_2*theta^2 + \dots + b_(2q+1)*theta^{2q+1}}
##'
##' However, the coefficients, b, are not directly estimated, but are a function of the
##' item parameters. In particular, the derivative \eqn{m'(\theta;\omega,\alpha,\tau)}{m'(theta;omega,alpha,tau)} is
##' parameterized in the following way:
##'
##'\deqn{m'(\theta;\omega,\alpha,\tau) = \left\{\begin{array}{ll}\exp(\omega) \prod_{u=1}^q(1-2\alpha_{u}\theta + (\alpha_{u}^2 + \exp(\tau_{u}))\theta^2) &  \mbox{if } q > 0 \\
##'  \exp(\omega) & \mbox{if } q = 0\end{array} \right.}{m'(theta) = m'(theta;omega,alpha,tau) = exp(omega) \prod_{u=1}^q (1-2*alpha_u*theta + (alpha_u^2 + exp(tau_u))*theta^2) (if q > 0) \\
##'  exp(omega) (if q = 0)}
##'
##'
##' See Falk & Cai (2016) for more details as to how the polynomial is constructed.
##' At the lowest order polynomial (q=0) the model reduces to the
##' two-parameter logistic (2PL) model. However, parameterization of the
##' slope parameter, \eqn{\omega}{omega}, is currently different than
##' the 2PL (i.e., slope = exp(\eqn{\omega}{omega})). This parameterization
##' ensures that the response function is always monotonically increasing
##' without requiring constrained optimization.
##'
##' For an alternative parameterization that releases constraints
##' on \eqn{\omega}{omega}, allowing for monotonically decreasing functions,
##' see \code{\link{rpf.grmp}}. And for polytomous items, see both
##' \code{\link{rpf.grmp}} and \code{\link{rpf.gpcmp}}.
##'
##' @param q a non-negative integer that controls the order of the
##' polynomial (2q+1) with a default of q=0 (1st order polynomial = 2PL).
##' @param multidimensional whether to use a multidimensional model.
##' Defaults to \code{FALSE}. The multidimensional version is not yet
##' available.
##' @return an item model
##' @references Falk, C. F., & Cai, L. (2016). Maximum marginal likelihood
##' estimation of a monotonic polynomial generalized partial credit model with
##' applications to multiple group analysis. \emph{Psychometrika, 81}, 434-460.
##' \doi{10.1007/s11336-014-9428-7}
##'
##' Liang (2007). \emph{A semi-parametric approach to estimating item response
##' functions}. Unpublished doctoral dissertation, Department of Psychology,
##' The Ohio State University.
##' @family response model
##' @examples
##' spec <- rpf.lmp(1) # 3rd order polynomial
##' theta<-seq(-3,3,.1)
##' p<-rpf.prob(spec, c(-.11,.37,.24,-.21),theta)
##'
##' spec <- rpf.lmp(2) # 5th order polynomial
##' p<-rpf.prob(spec, c(.69,.71,-.5,-8.48,.52,-3.32),theta)

rpf.lmp <- function(q=0, multidimensional=FALSE) {
  if(!(q%%1==0)){
    stop("q must be an integer >= 0")
  }
  if(multidimensional){
      stop("Multidimensional LMP model is not yet supported")
  }
  m <- NULL
  id <- -1
  id <- rpf.id_of("lmp")
  m <- new("rpf.1dim.lmp",
           outcomes=2,
           factors=1)
  m@spec <- c(id, 2, m@factors, q)
  m
}

setMethod("rpf.rparam", signature(m="rpf.1dim.lmp"),
          function(m, version) {
            n <- 1
            q<-m$spec[4] ## ok to hardcode this index?
            ret<-c(omega=rnorm(n, 0, .5),xi=rnorm(n, 0, .75))
            if(q>0){
                for(i in 1:q){
                    ret<-c(ret,runif(n,-1,1),log(runif(n,.0001,1)))
                    names(ret)[(3+(i-1)*2):(2+(i*2))]<-c(paste("alpha",i,sep=""),paste("tau",i,sep=""))
                }
            }
            ret
        })

##' Order a data.frame by missingness and all columns
##'
##' Completely order all rows in a data.frame.
##' 
##' @param observed a data.frame holding ordered factors in every column
##' @return the sorted order of the rows
##' @examples
##' df <- as.data.frame(matrix(c(sample.int(2, 30, replace=TRUE)), 10, 3))
##' mask <- matrix(c(sample.int(3, 30, replace=TRUE)), 10, 3) == 1
##' df[mask] <- NA
##' df[orderCompletely(df),]
orderCompletely <- function(observed) {
	observedNames <- colnames(observed)
	nacount <- sapply(observedNames, function(x) { sum(is.na(observed[,x])) })
	observedNames <- observedNames[order(nacount, decreasing=TRUE)]
	othervectorsNA <- lapply(observedNames, function(x) {!is.na(observed[,x]) })
	othervectors <- lapply(observedNames, function(x) {observed[,x] })
	args <- c(othervectorsNA, othervectors, 'na.last'=FALSE)
	do.call('order', args)
}

##' Tabulate data.frame rows
##'
##' Like \code{tabulate} but entire rows are the unit of tabulation.
##' The data.frame is not sorted, but must be sorted already.
##'
##' @param observed a sorted data.frame holding ordered factors in every column
##' @seealso \code{\link{orderCompletely}}
##' @examples
##' df <- as.data.frame(matrix(c(sample.int(2, 30, replace=TRUE)), 10, 3))
##' df <- df[orderCompletely(df),]
##' tabulateRows(df)
tabulateRows <- function(observed) {
	selectMissing <- rep(0L, nrow(observed))
	selectDefvars <- rep(0L, nrow(observed))
	threeVectors <- .Call('_rpf_findIdenticalRowsData', observed,
			      selectMissing, selectDefvars, TRUE, TRUE)
	dups <- threeVectors[[1]]
	result <- rep(NA, sum(dups==1L))
	dx <- 1L
	rx <- 1L
	while (dx <= length(dups)) {
		result[rx] <- dups[dx]
		rx <- rx + 1L
		dx <- dx + dups[dx]
	}
	result
}

#' Expand summary table of patterns and frequencies
#'
#' Expand a summary table of unique response patterns to a full sized
#' data-set.
#'
#' @param tabdata An object of class \code{data.frame} with the unique response patterns and the number of frequencies
#' @param freqName Column name containing the frequencies
#' @return Returns a data frame with all the response patterns
#' @author Based on code by Phil Chalmers \email{rphilip.chalmers@@gmail.com}
#' @examples
#' data(LSAT7)
#' expandDataFrame(LSAT7, freqName="freq")
expandDataFrame <- function(tabdata, freqName=NULL) {
	if (is.null(colnames(tabdata))) stop("colnames are required")

    if (missing(freqName)) {
        freqCol <- ncol(tabdata)
	warning(paste("Assuming column", colnames(tabdata)[freqCol], "contains frequencies"))
    } else {
        freqCol <- which(freqName == colnames(tabdata))
        if (length(freqCol) != 1) {
            stop(paste("Frequency column", freqName, "not found"))
        }
    }

    rows <- sum(tabdata[,freqCol])
    indexVector <- rep(NA, rows)
    rx <- 1L
    ix <- 1L
    while (rx <= nrow(tabdata)) {
        indexVector[ix:(ix + tabdata[rx,freqCol] - 1)] <- rx
	ix <- ix + tabdata[rx,freqCol]
        rx <- rx + 1L
    }
    tabdata[indexVector,-freqCol]
}

#' Compress a data frame into unique rows and frequencies
#'
#' Compress a data frame into unique rows and frequency counts.
#'
#' @param tabdata An object of class \code{data.frame}
#' @param freqColName Column name to contain the frequencies
#' @param .asNumeric logical. Whether to cast the frequencies to the numeric type
#' @return Returns a compressed data frame
#' @examples
#' df <- as.data.frame(matrix(c(sample.int(2, 30, replace=TRUE)), 10, 3))
#' compressDataFrame(df)
compressDataFrame <- function(tabdata, freqColName="freq", .asNumeric=FALSE) {
	if (!is.na(match(freqColName, colnames(tabdata)))) {
		# Might be nice to recompress instead of stopping.
		# There might be rows to collapse due to removal
		# of columns.
		stop(paste("Frequency column", freqColName, "already appears as a column:",
			   paste(colnames(tabdata), collapse=", ")))
	}
	tabdata <- tabdata[orderCompletely(tabdata),,drop=FALSE]
	freq <- tabulateRows(tabdata)
	if (.asNumeric) freq <- as.numeric(freq)
	tabdata <- unique(tabdata)
	tabdata <- cbind(tabdata, freq)
	colnames(tabdata)[ncol(tabdata)] <- freqColName
	tabdata
}

library(testthat)
library(rpf)

context("dataframe")

test_that("df basics", {
	df <- as.data.frame(matrix(sample.int(2, 5 * 100, replace=TRUE), 100, 5))
	cdf <- compressDataFrame(df)
	expect_true(nrow(cdf) < nrow(df))
	expect_equal(sum(cdf$freq), nrow(df))
	df2 <- expandDataFrame(cdf, "freq")
	expect_equal(nrow(df2), 100)
	expect_true(all(df2 == df[orderCompletely(df),]))
})

df <- as.data.frame(matrix(as.numeric(sample(2, 5 * 100, replace=TRUE)), 100, 5))
mask <- matrix(runif(5*100) < .1, ncol=5)
df[mask] <- NA
cdf <- compressDataFrame(df)
expect_equal(sum(cdf$freq), nrow(df))
df2 <- expandDataFrame(cdf, "freq")
expect_equal(nrow(df2), 100)
expect_true(all(df2 == df[orderCompletely(df),], na.rm=TRUE))
expect_true(all(is.na(df2) == is.na(df[orderCompletely(df),])))

.onLoad <- function(libname, pkgname) {
	if ("package:parallel" %in% search()) {
		cores <- as.integer(Sys.getenv("OMP_NUM_THREADS"))
		if (is.na(cores)) {
      cores <- detectCores(logical=TRUE)
      if(is.na(cores)) cores <- 1L
			else cores <- 2L
    }
		.Call(`_rpf_setNumberOfCores`, cores)
		#packageStartupMessage(paste("OpenMP will use", cores, "cores"))
	}
}

.onAttach <- function(libname, pkgname) {
	if (! .Call(`_rpf_has_openmp`)) {
		packageStartupMessage("RPF is not compiled to take advantage of computers with multiple cores.")
	}
}

# Register entry points for exported C++ functions
methods::setLoadAction(function(ns) {
    .Call('_rpf_registerCCallable', PACKAGE = 'rpf')
})

##' rpf - Response Probability Functions
##'
##' Factor out logic and math common
##' to Item Factor Analysis fitting, diagnostics, and analysis.  It is
##' envisioned as core support code suitable for more specialized IFA
##' packages to build upon.
##'
##' This package provides optimized, low-level functions to map
##' parameters to response probabilities for dichotomous (1PL, 2PL and
##' 3PL) \code{\link{rpf.drm}} and polytomous (graded response
##' \code{\link{rpf.grm}}, partial credit/generalized partial credit
##' (via the nominal model), and nominal \code{\link{rpf.nrm}} items.
##'
##' Item model parameters are passed around as a numeric vector. A 1D
##' matrix is also acceptable. Regardless of model, parameters are
##' always ordered as follows: discrimination/slope ("a"),
##' difficulty/intercept ("b"), and pseudo guessing/upper-bound ("g"/"u"). If
##' person ability ranges from negative to positive then
##' probabilities are output from incorrect to correct. That is, a low
##' ability person (e.g., ability = -2) will be more likely to get an
##' item incorrect than correct. For example, a dichotomous model that
##' returns [.25, .75] indicates a probability of .25 for incorrect
##' and .75 for correct.  A polytomous model will have the most
##' incorrect probability at index 1 and the most correct probability
##' at the maximum index.
##'
##' All models are always in the logistic metric. To obtain normal
##' ogive discrimination parameters, divide slope parameters by
##' \code{\link{rpf.ogive}}. Item models are estimated in
##' slope-intercept form. Input/output matrices arranged in the way
##' most convenient for low-level processing in C. The maximum
##' absolute logit is 35 because f(x) := 1-exp(x) loses accuracy around f(-35)
##' and equals 1 at f(-38) due to the limited accuracy of double
##' precision floating point.
##'
##' This package could also accrete functions to support plotting (but
##' not the actual plot functions).
##'
##' @docType package
##' @aliases rpf-package
##' @rdname rpf.introduction
##' @name An introduction
##' @references
##' Pritikin, J. N., Hunter, M. D., & Boker, S. M.
##' (2015). Modular open-source software for Item Factor Analysis.
##' Educational and Psychological Measurement, 75(3), 458-474
##'
##' Thissen, D. and Steinberg, L. (1986). A taxonomy of
##' item response models. \emph{Psychometrika 51}(4), 567-577.
##' @seealso
##' See \code{\link{rpf.rparam}} to create item parameters.
NULL

##' The base class for response probability functions.
##'
##' Item specifications should not be modified after creation.
##'
##' @name Class rpf.base
##' @rdname rpf.base-class
##' @aliases rpf.base-class
##' $,rpf.base-method
##' $<-,rpf.base-method
setClass("rpf.base",
         representation(spec="numeric",
                        outcomes="numeric",
                        factors="numeric",
                        "VIRTUAL"))

imxExtractSlot <- function(x, name) {
	if (!.hasSlot(x, name)) {
		return(NULL)
	} else {
		return(slot(x, name))
	}
}

setMethod("$", "rpf.base", imxExtractSlot)

setReplaceMethod("$", "rpf.base", function(x, name, value) {
    stop("Slots are read-only")
})

##' The base class for 1 dimensional response probability functions.
##' @name Class rpf.1dim
##' @rdname rpf.1dim-class
##' @aliases rpf.1dim-class
setClass("rpf.1dim", contains='rpf.base',
         representation("VIRTUAL"))

##' The base class for multi-dimensional response probability functions.
##' @name Class rpf.mdim
##' @rdname rpf.mdim-class
##' @aliases rpf.mdim-class
setClass("rpf.mdim", contains='rpf.base',
         representation("VIRTUAL"))

##' Length of the item model vector
##' @param m item model
##' @aliases
##' rpf.numSpec,rpf.base-method
##' rpf_numSpec_wrapper
##' @examples
##' rpf.numSpec(rpf.grm(outcomes=3))
##' rpf.numSpec(rpf.nrm(outcomes=3))
setGeneric("rpf.numSpec", function(m) standardGeneric("rpf.numSpec"))

setMethod("rpf.numSpec", signature(m="rpf.base"),
          function(m) {
            if (length(m@spec)==0) {
              stop("Not implemented")
            } else {
              .Call(`_rpf_numSpec`, m@spec)
            }
          })

##' Length of the item parameter vector
##'
##' @param m item model
##' @aliases
##' rpf.numParam,rpf.base-method
##' rpf_numParam_wrapper
##' @examples
##' rpf.numParam(rpf.grm(outcomes=3))
##' rpf.numParam(rpf.nrm(outcomes=3))
setGeneric("rpf.numParam", function(m) standardGeneric("rpf.numParam"))

setMethod("rpf.numParam", signature(m="rpf.base"),
          function(m) {
            .Call('_rpf_numParam', m@spec)
          })

##' Create a similar item specification with the given number of factors
##'
##' @param m item model
##' @param factors the number of factors/dimensions
##' @aliases
##' rpf.modify,rpf.mdim.drm,numeric-method
##' rpf.modify,rpf.mdim.graded,numeric-method
##' rpf.modify,rpf.mdim.nrm,numeric-method
##' @examples
##' s1 <- rpf.grm(factors=3)
##' rpf.rparam(s1)
##' s2 <- rpf.modify(s1, 1)
##' rpf.rparam(s2)
setGeneric("rpf.modify", function(m, factors) standardGeneric("rpf.modify"))

##' Retrieve a description of the given parameter
##' @param m item model
##' @param num vector of parameters (defaults to all)
##' @return a list containing the type, upper bound, and lower bound
##' @aliases
##' rpf.paramInfo,rpf.base-method
##' rpf_paramInfo_wrapper
##' @examples
##' rpf.paramInfo(rpf.drm())
setGeneric("rpf.paramInfo", function(m, num=NULL) standardGeneric("rpf.paramInfo"))

setMethod("rpf.paramInfo", signature(m="rpf.base"),
          function(m, num=NULL) {
            if (missing(num)) {
              num <- 1:rpf.numParam(m)
            }
            if (length(num) == 0) {
              stop("Which parameter?")
            } else if (length(num) == 1) {
              .Call('_rpf_paramInfo', m@spec, num-1)
            } else {
              sapply(num, function (px) .Call('_rpf_paramInfo', m@spec, px-1))
            }
          })

##' Item parameter derivatives
##'
##' Evaluate the partial derivatives of the log likelihood with
##' respect to each parameter at \code{where} with \code{weight}.
##'
##' It is not easy to write an example for this function. To evaluate
##' the derivative, you need to sum the derivatives across a
##' quadrature. You also need response outcome weights at each
##' quadrature point. It is not anticipated that this function will be
##' often used in R code. It's mainly to expose a C-level function for
##' occasional debugging.
##'
##' @param m item model
##' @param param item parameters
##' @param where location in the latent space
##' @param weight per outcome weights (typically derived by observation)
##' @return first and second order partial derivatives of the log
##' likelihood evaluated at \code{where}. For p parameters, the first
##' p values are the first derivative and the next p(p+1)/2 columns
##' are the lower triangle of the second derivative.
##' @seealso
##' The numDeriv package.
##' @aliases
##' rpf.dLL,rpf.base,numeric,numeric,numeric-method
##' rpf.dLL,rpf.base,numeric,NULL,numeric-method
##' _rpf_dLL
setGeneric("rpf.dLL", function(m, param, where, weight) standardGeneric("rpf.dLL"))

setMethod("rpf.dLL", signature(m="rpf.base", param="numeric",
                                 where="numeric", weight="numeric"),
          function(m, param, where, weight) {
            if (length(m@spec)==0) {
              stop("Not implemented")
            } else {
              .Call('_rpf_dLL', m@spec, param, where, weight)
            }
          })

setMethod("rpf.dLL", signature(m="rpf.base", param="numeric",
                                 where="NULL", weight="numeric"),
          function(m, param, where, weight) {
            if (length(m@spec)==0) {
              stop("Not implemented")
            } else {
              .Call('_rpf_dLL', m@spec, param, where, weight)
            }
          })

##' Item derivatives with respect to the location in the latent space
##'
##' Evaluate the partial derivatives of the response probability with
##' respect to ability. See \link{rpf.info} for an application.
##'
##' @param m item model
##' @param param item parameters
##' @param where location in the latent distribution
##' @param dir if more than 1 factor, a basis vector
##' @aliases
##' _rpf_dTheta
##' rpf.dTheta,rpf.base,numeric,numeric,numeric-method
##' rpf.dTheta,rpf.base,numeric,matrix,numeric-method
setGeneric("rpf.dTheta", function(m, param, where, dir) standardGeneric("rpf.dTheta"))

setMethod("rpf.dTheta", signature(m="rpf.base", param="numeric",
                                  where="numeric", dir="numeric"),
          function(m, param, where, dir) {
            if (length(m@spec)==0) {
              stop("Not implemented")
            } else {
              .Call('_rpf_dTheta', m@spec, param, where, dir)
            }
          })

setMethod("rpf.dTheta", signature(m="rpf.base", param="numeric",
                                  where="matrix", dir="numeric"),
          function(m, param, where, dir) {
            dP.raw <- apply(where, 2, function(w) rpf.dTheta(m, param, w, dir))
            list(gradient=sapply(dP.raw, function(deriv) deriv$gradient),
                 hessian=sapply(dP.raw, function(deriv) deriv$hessian))
          })

##' Rescale item parameters
##'
##' Adjust item parameters for changes in mean and covariance of the
##' latent distribution.
##'
##' @param m item model
##' @param param item parameters
##' @param mean vector of means
##' @param cov covariance matrix
##' @aliases
##' _rpf_rescale
##' rpf.rescale,rpf.base,numeric,numeric,matrix-method
##' @examples
##' spec <- rpf.grm()
##' p1 <- rpf.rparam(spec)
##' testPoint <- rnorm(1)
##' move <- rnorm(1)
##' cov <- as.matrix(rlnorm(1))
##' Icov <- solve(cov)
##' padj <- rpf.rescale(spec, p1, move, cov)
##' pr1 <- rpf.prob(spec, padj, (testPoint-move) %*% Icov)
##' pr2 <- rpf.prob(spec, p1, testPoint)
##' abs(pr1 - pr2) < 1e9
setGeneric("rpf.rescale", function(m, param, mean, cov) standardGeneric("rpf.rescale"))

setMethod("rpf.rescale", signature(m="rpf.base", param="numeric",
                                   mean="numeric", cov="matrix"),
          function(m, param, mean, cov) {
            if (length(m@spec)==0) {
              stop("Not implemented")
            } else {
              .Call('_rpf_rescale', m@spec, param, mean, cov)
            }
          })

##' Map an item model, item parameters, and person trait score into a
##' probability vector
##'
##' This function is known by many names in the literature. When
##' plotted against latent trait, it is often called a traceline, item
##' characteristic curve, or item response function. Sometimes the word
##' 'category' or 'outcome' is used in place of 'item'. For example,
##' 'item response function' might become 'category response
##' function'. All these terms refer to the same thing.
##'
##' @param m an item model
##' @param param item parameters
##' @param theta the trait score(s)
##' @return a vector of probabilities. For dichotomous items,
##' probabilities are returned in the order incorrect, correct.
##' Although redundent, both incorrect and correct probabilities are
##' returned in the dichotomous case for API consistency with
##' polytomous item models.
##' @docType methods
##' @aliases
##' rpf.prob,rpf.1dim,numeric,numeric-method
##' rpf.prob,rpf.mdim,numeric,NULL-method
##' rpf.prob,rpf.mdim,numeric,numeric-method
##' rpf.prob,rpf.mdim,numeric,matrix-method
##' rpf.prob,rpf.base,data.frame,numeric-method
##' rpf.prob,rpf.base,matrix,numeric-method
##' rpf.prob,rpf.base,matrix,matrix-method
##' rpf.prob,rpf.1dim,numeric,matrix-method
##' rpf.prob,rpf.1dim.grm,numeric,numeric-method
##' rpf.prob,rpf.mdim.grm,numeric,numeric-method
##' rpf.prob,rpf.mdim.nrm,numeric,matrix-method
##' rpf.prob,rpf.mdim.mcm,numeric,matrix-method
##' rpf.prob,rpf.mdim.grm,numeric,matrix-method
##' _rpf_prob
##' @examples
##' i1 <- rpf.drm()
##' i1.p <- rpf.rparam(i1)
##' rpf.prob(i1, c(i1.p), -1)   # low trait score
##' rpf.prob(i1, c(i1.p), c(0,1))    # average and high trait score
setGeneric("rpf.prob", function(m, param, theta) standardGeneric("rpf.prob"))

##' Map an item model, item parameters, and person trait score into a
##' probability vector
##'
##' Note that in general, exp(rpf.logprob(..)) != rpf.prob(..) because
##' the range of logits is much wider than the range of probabilities
##' due to limitations of floating point numerical precision.
##'
##' @param m an item model
##' @param param item parameters
##' @param theta the trait score(s)
##' @return a vector of probabilities. For dichotomous items,
##' probabilities are returned in the order incorrect, correct.
##' Although redundent, both incorrect and correct probabilities are
##' returned in the dichotomous case for API consistency with
##' polytomous item models.
##' @docType methods
##' @aliases
##' rpf.logprob,rpf.1dim,numeric,numeric-method
##' rpf.logprob,rpf.1dim,numeric,matrix-method
##' rpf.logprob,rpf.mdim,numeric,matrix-method
##' rpf.logprob,rpf.mdim,numeric,numeric-method
##' rpf.logprob,rpf.mdim,numeric,NULL-method
##' _rpf_logprob
##' @examples
##' i1 <- rpf.drm()
##' i1.p <- rpf.rparam(i1)
##' rpf.logprob(i1, c(i1.p), -1)   # low trait score
##' rpf.logprob(i1, c(i1.p), c(0,1))    # average and high trait score
setGeneric("rpf.logprob", function(m, param, theta) standardGeneric("rpf.logprob"))

setMethod("rpf.logprob", signature(m="rpf.1dim", param="numeric", theta="numeric"),
          function(m, param, theta) {
            if (length(m@spec)==0) {
              stop("Not implemented")
            } else {
              .Call('_rpf_logprob', m@spec, param, theta)
            }
          })

setMethod("rpf.logprob", signature(m="rpf.mdim", param="numeric", theta="matrix"),
          function(m, param, theta) {
            if (length(m@spec)==0) {
              stop("Not implemented")
            } else {
              .Call('_rpf_logprob', m@spec, param, theta)
            }
          })

setMethod("rpf.logprob", signature(m="rpf.mdim", param="numeric", theta="numeric"),
          function(m, param, theta) {
            rpf.logprob(m, param, as.matrix(theta))
          })

setMethod("rpf.logprob", signature(m="rpf.mdim", param="numeric", theta="NULL"),
          function(m, param, theta) {
            if (length(m@spec)==0) {
              stop("Not implemented")
            } else {
              .Call('_rpf_logprob', m@spec, param, theta)
            }
          })

setMethod("rpf.logprob", signature(m="rpf.1dim", param="numeric", theta="matrix"),
          function(m, param, theta) {
            rpf.logprob(m, param, as.numeric(theta))
          })

setMethod("rpf.prob", signature(m="rpf.1dim", param="numeric", theta="numeric"),
          function(m, param, theta) {
            if (length(m@spec)==0) {
              exp(rpf.logprob(m, param, theta))
            } else {
              .Call('_rpf_prob', m@spec, param, theta)
            }
          })

setMethod("rpf.prob", signature(m="rpf.mdim", param="numeric", theta="NULL"),
          function(m, param, theta) {
            if (length(m@spec)==0) {
              exp(rpf.logprob(m, param, theta))
            } else {
              .Call('_rpf_prob', m@spec, param, theta)
            }
          })

setMethod("rpf.prob", signature(m="rpf.mdim", param="numeric", theta="matrix"),
          function(m, param, theta) {
            if (length(m@spec)==0) {
              exp(rpf.logprob(m, param, theta))
            } else {
              .Call('_rpf_prob', m@spec, param, theta)
            }
          })

setMethod("rpf.prob", signature(m="rpf.base", param="data.frame", theta="numeric"),
          function(m, param, theta) {
            exp(rpf.logprob(m, as.numeric(param), theta))
          })

setMethod("rpf.prob", signature(m="rpf.base", param="matrix", theta="numeric"),
          function(m, param, theta) {
            rpf.prob(m, as.numeric(param), theta)
          })

setMethod("rpf.prob", signature(m="rpf.base", param="matrix", theta="matrix"),
          function(m, param, theta) {
            rpf.prob(m, as.numeric(param), theta)
          })

setMethod("rpf.prob", signature(m="rpf.1dim", param="numeric", theta="matrix"),
          function(m, param, theta) {
            rpf.prob(m, param, as.numeric(theta))
          })

setMethod("rpf.prob", signature(m="rpf.mdim", param="numeric", theta="numeric"),
          function(m, param, theta) {
            rpf.prob(m, param, as.matrix(theta))
          })

##' Map an item model, item parameters, and person trait score into a
##' information vector
##'
##' @param ii an item model
##' @param ii.p item parameters
##' @param where the location in the latent distribution
##' @param basis if more than 1 factor, a positive basis vector
##' @return Fisher information
##' @examples
##' i1 <- rpf.drm()
##' i1.p <- c(.6,1,.1,.95)
##' theta <- seq(0,3,.05)
##' plot(theta, rpf.info(i1, i1.p, t(theta)), type="l")
##' @references Dodd, B. G., De Ayala, R. J. & Koch,
##' W. R. (1995). Computerized adaptive testing with polytomous items.
##' \emph{Applied psychological measurement 19}(1), 5-22.
rpf.info <- function(ii, ii.p, where, basis=1) {
  if (any(basis < 0)) warning("All components of the basis vector should be positive")
  if (!missing(basis)) {
    basis <- basis/sqrt(sum(basis^2))
  }
  P <- rpf.prob(ii, ii.p, where)
  dP <- rpf.dTheta(ii, ii.p, where, basis)
  colSums(dP$gradient^2 / P - dP$hessian)
}

##' Generates item parameters
##'
##' This function generates random item parameters. The version
##' argument is available if you are writing a test that depends on
##' reproducable random parameters (using \code{set.seed}).
##'
##' @param m an item model
##' @param version the version of random parameters
##' @return item parameters
##' @docType methods
##' @aliases
##' rpf.rparam,rpf.1dim.drm-method
##' rpf.rparam,rpf.mdim.drm-method
##' rpf.rparam,rpf.1dim.graded-method
##' rpf.rparam,rpf.mdim.graded-method
##' rpf.rparam,rpf.mdim.nrm-method
##' rpf.rparam,rpf.mdim.mcm-method
##' rpf.rparam,rpf.1dim.lmp-method
##' rpf.rparam,rpf.1dim.grmp-method
##' rpf.rparam,rpf.1dim.gpcmp-method
##' @examples
##' i1 <- rpf.drm()
##' rpf.rparam(i1)
setGeneric("rpf.rparam", function(m, version=2L) standardGeneric("rpf.rparam"))

##' The ogive constant
##'
##' The ogive constant can be multiplied by the discrimination
##' parameter to obtain a response curve very similar to the Normal
##' cumulative distribution function (Haley, 1952; Molenaar, 1974).
##' Recently, Savalei (2006) proposed a new constant of 1.749 based on
##' Kullback-Leibler information.
##'
##' In recent years, the logistic has grown in favor, and therefore,
##' this package does not offer any special support for this
##' transformation (Baker & Kim, 2004, pp. 14-18).
##'
##' @references Camilli, G. (1994). Teacher's corner: Origin of the
##' scaling constant d=1.7 in Item Response Theory. \emph{Journal of
##' Educational and Behavioral Statistics, 19}(3), 293-295.
##'
##' Baker & Kim (2004). \emph{Item Response Theory: Parameter
##' Estimation Techniques.} Marcel Dekker, Inc.
##'
##' Haley, D. C. (1952). \emph{Estimation of the dosage mortality
##' relationship when the dose is subject to error} (Technical Report
##' No. 15). Stanford University Applied Mathematics and Statistics
##' Laboratory, Stanford, CA.
##'
##' Molenaar, W. (1974). De logistische en de normale kromme [The
##' logistic and the normal curve]. \emph{Nederlands Tijdschrift voor de
##' Psychologie} 29, 415-420.
##'
##' Savalei, V. (2006). Logistic approximation to the normal: The KL
##' rationale. \emph{Psychometrika, 71}(4), 763--767.
rpf.ogive <- 1.702

##' The base class for 1 dimensional graded response probability functions.
##'
##' This class contains methods common to both the generalized partial
##' credit model and the graded response model.
##'
##' @name Class rpf.1dim.graded
##' @rdname rpf.1dim.graded-class
##' @aliases rpf.1dim.graded-class
setClass("rpf.1dim.graded", contains='rpf.1dim',
         representation("VIRTUAL"))

##' The base class for multi-dimensional graded response probability
##' functions.
##'
##' This class contains methods common to both the generalized partial
##' credit model and the graded response model.
##'
##' @name Class rpf.mdim.graded
##' @rdname rpf.mdim.graded-class
##' @aliases rpf.mdim.graded-class
setClass("rpf.mdim.graded", contains='rpf.mdim',
         representation("VIRTUAL"))

##' The unidimensional graded response item model.
##'
##' @name Class rpf.1dim.grm
##' @rdname rpf.1dim.grm-class
##' @aliases rpf.1dim.grm-class
setClass("rpf.1dim.grm", contains='rpf.1dim.graded')

##' Unidimensional dichotomous item models (1PL, 2PL, and 3PL).
##'
##' @name Class rpf.1dim.drm
##' @rdname rpf.1dim.drm-class
##' @aliases rpf.1dim.drm-class
setClass("rpf.1dim.drm", contains='rpf.1dim')

##' Multidimensional dichotomous item models (M1PL, M2PL, and M3PL).
##'
##' @name Class rpf.mdim.drm
##' @rdname rpf.mdim.drm-class
##' @aliases rpf.mdim.drm-class
setClass("rpf.mdim.drm", contains='rpf.mdim')

##' The multidimensional graded response item model.
##'
##' @name Class rpf.mdim.grm
##' @rdname rpf.mdim.grm-class
##' @aliases rpf.mdim.grm-class
setClass("rpf.mdim.grm", contains='rpf.mdim.graded')

##' The nominal response item model (both unidimensional and
##' multidimensional models have the same parameterization).
##'
##' @name Class rpf.mdim.nrm
##' @rdname rpf.mdim.nrm-class
##' @aliases rpf.mdim.nrm-class
setClass("rpf.mdim.nrm", contains='rpf.mdim')

##' The multiple-choice response item model (both unidimensional and
##' multidimensional models have the same parameterization).
##'
##' @name Class rpf.mdim.mcm
##' @rdname rpf.mdim.mcm-class
##' @aliases rpf.mdim.mcm-class
setClass("rpf.mdim.mcm", contains='rpf.mdim')

##' Unidimensional logistic function of a monotonic polynomial.
##'
##' @name Class rpf.1dim.lmp
##' @rdname rpf.1dim.lmp-class
##' @aliases rpf.1dim.lmp-class
##'
setClass("rpf.1dim.lmp", contains='rpf.1dim')

##' Unidimensional graded response monotonic polynomial.
##'
##' @name Class rpf.1dim.grmp
##' @rdname rpf.1dim.grmp-class
##' @aliases rpf.1dim.grmp-class
##'
setClass("rpf.1dim.grmp", contains='rpf.1dim')

##' Unidimensional generalized partial credit monotonic polynomial.
##'
##' @name Class rpf.1dim.gpcmp
##' @rdname rpf.1dim.gpcmp-class
##' @aliases rpf.1dim.gpcmp-class
##'
setClass("rpf.1dim.gpcmp", contains='rpf.1dim')

##' Convert an rpf item model name to an ID
##'
##' This is an internal function and should not be used.
##'
##' @param name name of the item model (string)
##' @return the integer ID assigned to the given model
rpf.id_of <- function(name) {
   .Call(`_rpf_get_model_id`, name)
}

<!--
%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Custom Item Models (HTML)}
-->

# Overview

This document describes the process of adding custom
item models to rpf as was done by a contributor to
the rpf package (Carl F. Falk) with assistance from
the package author (Joshua N. Pritikin) for the logistic
function of a monotonic polynomial (LMP)
item model [(Falk & Cai, in press)](https://dx.doi.org/10.1007/s11336-014-9428-7).

This document assumes experience in editing R and C++ code and
installing R packages from source. In addition, use of
versioning control software (e.g., git) is also helpful.
Not mentioned are platform-specific tools or compilers
that need to be in place in order to compile/install
packages from source.

The rpf package is modular in the sense that one
only needs to write the underlying functions specific
to the item model, such as the traceline (ICC, IRF, etc.),
derivatives w.r.t. item parameters and the latent trait(s), and
a few utility functions. As a byproduct, estimation of a
measurement model that utilizes the item model may be
possible via use of an external package
(e.g., [OpenMx](https://openmx.ssri.psu.edu/))

# Procedure

## Outline

Obtain rpf source code from [CRAN](https://cran.r-project.org/package=rpf)
or another source such as [GitHub](https://github.com/cran/rpf).
I used the latter as I wanted to use git to track my own changes
and then later merge with the rpf package.

Most work needs to be done in C++ using the first file
mentioned in the subsequent section. Testing appears to be
easiest from R, however, with some functions in the rpf
package yielding access to the underlying C++ functions.
It is therefore possible to also glean some insight into
what the underyling C++ functions ought to do, and their
input by examining rpf documentation and experimenting
with associated functions.


## Edit src/libifa-rpf.cpp

The bulk of work that needs to be done to implement an item
model takes place in the src/libifa-rpf.cpp file.

For example, ```librpf_model[]``` towards the end of the
file contains available item models with associated C++
functions required to implement each item model. As a
concrete example, the following code chunk
effectively lists all functions used by the
unidimensional LMP item model:

```
{"lmp",
    irt_rpf_1dim_lmp_numSpec,
    irt_rpf_1dim_lmp_numParam,
    irt_rpf_1dim_lmp_paramInfo,
    irt_rpf_1dim_lmp_prob,
    irt_rpf_logprob_adapter,
    irt_rpf_1dim_lmp_deriv1,
    irt_rpf_1dim_lmp_deriv2,
    irt_rpf_1dim_lmp_dTheta,
    irt_rpf_1dim_lmp_rescale,
}
```

To add a novel item model, one would need to add an
analogous chunk of code to ```librpf_model[]``` with
the given functions pertaining to those used by the
novel item model. Each type of function takes input
in the same way regardless of the item model. This
is a feature that helps make rpf modular.

It is easiest to understand the purpose of each function by
focusing on the suffixes for the functions above. These
suffixes are referenced below. The prefixes often tell us
which item model the function pertains to.
Below also effectively contains stubs that could be
used to start constructing a new item model.

### ```numSpec```
Each item model (e.g., when created by R) contains
a vector that contains a specification for the item
model. This will contain information such as the
number of categories, factors, and other item-specific
information. I will discuss how this specification is
constructed from R input at a later section of this
document. I assume the ```numSpec``` function
merely returns the number of elements of this vector.

For example:
```
static int
irt_rpf_1dim_lmp_numSpec(const double *spec)
{ return RPF_ISpecCount; }
```

### ```numParam```
Based only on information in the item specification,
return an integer that indicates how many parameters
the item has. For example, the LMP model has a
user-specified integer *k* that determines the
number of parameters and appears as the last element
in the specification vector.

```
static int
irt_rpf_1dim_lmp_numParam(const double *spec)
{
  int k = spec[RPF_ISpecCount];
  return(2+2*k);
}
```

### ```paramInfo```
Returns information (in pointers) regarding a single
item parameter based on the item specification ```*spec```,
the index of the item parameter ```param```. This generally assumes
that the item parameters always appear in a particular
order for the item model, which can be determined based
only on the item specification. For instance, a
two-parameter logistic item model will always have a
slope and intercept term, which appear in that order,
and the number of slopes depends on the number of factors.

Returned information may include the type of
parameter ```**type``` which is a text description of
the item parameter (e.g., "Slope" or "Intercept"), and
the upper and lower bounds for the item parameter, ```*upper```
and ```*lower```, which may be set to ```nan("unset")``` if
bounds are not applicable. Below is only a stub, not the full
function for the LMP model.

```
static void
irt_rpf_1dim_lmp_paramInfo(const double *spec, const int param,
			   const char **type, double *upper, double *lower)
{
  \\ Code implementing paramInfo goes here
}
```

### ```prob```

The traceline (ICC, IRF, etc.) function for the item model.
This function should compute the probability of response
to each category (stored in ```*out``` in that order),
based on a fixed vector of item parameters,```*param```, and
at a single point of the latent space, ```*th```. That is,
if the item model is dichotomous, ```*out``` will contain
two values, and if the latent trait has three dimensions,
```*th``` will have three values. In unidimensional models,
```*th``` will have only a single value.

```
static void
irt_rpf_1dim_lmp_prob(const double *spec,
		      const double *param, const double *th,
		      double *out)
{
  \\ Code implementing prob goes here
}
```

### ```logprob_adapter``` or ```logprob```

I assume this function computes the log of the traceline,
which in some cases can be done simply by obtaining
information from the ```prob``` function corresponding
to the item model and is what ```logprob_adapter``` does.

```
irt_rpf_logprob_adapter(const double *spec,
			const double *param, const double *th,
			double *out)
{
  \\ Code implementing logprob goes here
}
```

### ```deriv1```

This function computes first *and* second-order complete
data derivatives the negative log-likelihood for a
single item with respect to item parameters.
e.g., for use in optimization at the M-Step of the EM
algorithm.

This function assumes complete data on the latent trait as
is often the case after the E-Step of the EM algorithm
has effectively computed expected counts for each possible
category at each quadrature node.

Note that derivatives are at a single point of the latent
trait, ```*where``` which is analogous to ```*th``` from
the traceline function. This may represent a single
quadrature node, or if complete data were available it
might represent a single respondent's score on the
latent trait.

```*weight``` is a vector that determines how much
weight to give to each response category. If input to
the function were to assume a single respondent, this
vector is effectively an indicator function that will
have a "1" in the proper location indicating the
person's response. In the case of EM, the weight vector
may contain expected counts at this particular quadrature
node for each of the item's possible response categories.

```*out``` will contain the derivatives for use
elsewhere. Note that this vector may not be empty
when entering the function, e.g., in the case that
```deriv1``` is repeatedly called to compute
derivatives summing across multiple
quadrature points or respondents. From the rpf
documentation, the order of elements in ```*out``` is...
"For p parameters, the first p values are the first
derivative and the next p(p+1)/2 columns are the lower
triangle of the second derivative."

```
static void
irt_rpf_1dim_lmp_deriv1(const double *spec,
				  const double *param,
				  const double *where,
				  const double *weight, double *out)
{
  // Code implementing deriv1 goes here

}
```

### ```deriv2```

This function implements derivatives or computations
that may occur once any time derivatives are taken,
regardless of how many quadrature nodes or respondents.
Whereas ```deriv1``` for example, may be called repeatedly
(once for for each quadrature node, ```deriv2``` would
only be called once when computing derivatives across
such quadrature nodes. That is, currently the
```rpf.dLL``` function in R from rpf may do ```deriv1```
multiple times followed by ```deriv2```
before returning derivatives to the user.

```
static void irt_rpf_1dim_lmp_deriv2(const double *spec,
				  const double *param,
				  double *out)
{
   \\ Code implementing deriv2 goes here
}
```

### ```dTheta```

Compute derivatives of the traceline w.r.t. the
latent trait, e.g., for use in computing
item information.

These derivative computations happen at a single point
of the latent trait, ```*where```. Derivatives
for each category response function should appear,
in order from least to gretest, in ```*grad``` (first-order),
and ```*hess``` (second-order).

The vector ```*dir``` is a basis vector used in the
case of multidimensional models.

```
static void irt_rpf_1dim_lmp_dTheta(const double *spec, const double *param,
			const double *where, const double *dir,
			double *grad, double *hess)
{
  \\ Code implementing dTheta goes here

}
```

### ```rescale```

Currently this function is not *yet* used and could go
unimplemented. However, the intuition behind it was based
on [Schilling & Bock (2005)](https://dx.doi.org/10.1007/s11336-003-1141-x), equations 22 and 23 for
implementation of adaptive quadrature.

```
static void
irt_rpf_1dim_lmp_rescale(const double *spec, double *param, const int *paramMask,
			 const double *mean, const double *cov)
{
  error("Rescale for LMP model not implemented");
}
```

## Edit other, mostly R files

### Add R functions to create item specification

The the R/ subfolder, each item model or class of item
models has their own associated R file. For example,
"grm.R" or lmp.R" for the Graded Response model and
the LMP item model.

A function common to each of these files is one that
will construct the item specification based on user input.
An example for the Graded Response Models is:

```
rpf.grm <- function(outcomes=2, factors=1, multidimensional=TRUE) {
  if (!multidimensional && factors > 1) {
    stop("More than 1 dimension must use a multidimensional model")
  }
  m <- NULL
  id <- -1
  if (!multidimensional) {
    stop("The old parameterization is no longer available")
  } else {
    id <- rpf.id_of("grm")
    m <- new("rpf.mdim.grm",
             outcomes=outcomes,
             factors=factors)
  }
  m@spec <- c(id, m@outcomes, m@factors)
  m
}
```

Note that input to the function may be item specific,
however, what the function returns ought to be in a
standard format. Specifically, a list containing the id
of the item model (as determined by ```rpf.id_of```, which
requires a String that matches that added to the end
of ```librpf_model[]```), the number of categories (or
outcomes), and number of factors is required. Additional
optional elements may be appended to the specification
as required by the item model. For example, LMP requires
that *k* be added to the end of the specification to
determine the order of a polynomial and number of item
parameters, and does not currently have a multidimensional
version.

```
rpf.lmp <- function(q=1, multidimensional=FALSE) {
  if(!(q%%1==0)){
    stop("q must be an integer >= 0")
  }
  if(multidimensional){
      stop("Multidimensional LMP model is not yet supported")
  }
  m <- NULL
  id <- -1
  id <- rpf.id_of("lmp")
  m <- new("rpf.1dim.lmp",
           outcomes=2,
           factors=1)
  m@spec <- c(id, 2, m@factors, q)
  m
}
```

Other functions in such an .R file appear to be
optional, but may include those that specify how random
parameters be generated for the item model, e.g.,
```rpf.rparam```, or how an item model may be modified
to create a similar item model ```rpf.modify```

Note also that these functions ought to have documentation
for the item model that is in a format usable by
[roxygen2](https://cran.r-project.org/package=roxygen2)

### Edit DESCRIPTION
Any new files created in the previous step ought to be
added to the "Collate" statement in the DESCRIPTION file.

### Edit R/Classes.R
A class ought to be added to this file for the item model.
This will allow generic functions or wrappers to access
item specific C++ code. It is difficult to provide specific
recommendations on how to add the class other than to
look at other examples. For instance, the following
code snippet defines the LMP dichotomous response model,
"rpf.lmp.drm", as a subclass of the unidimensional
response model superclass, "rpf.1dim".

```
##' Unidimensional logistic function of a monotonic polynomial
##'
##' @export
##' @name Class rpf.1dim.lmp
##' @rdname rpf.1dim.lmp-class
##' @aliases rpf.1dim.lmp-class
##'
setClass("rpf.1dim.lmp", contains='rpf.1dim')
```

Thus, the above is all that was required to add a
class for the LMP item model. Alternative item
models may use the multidimensional superclass,
"rpf.mdim" or a specialized class for graded
response models.

## Putting things together and testing

To avoid headaches in debugging, it may be useful
to write code incrementally and test/debug before
continuing to write more. For instance, after
writing stubs for C++ code, see if the code will
compile, perhaps by using ```R CMD check rpf```
or some such from the command line of the OS.

Not all pieces need to be in place before testing
in R. For instance, it may make practical sense
to implement the traceline function in C++, with
suffix, ```prob```, and test via R (see below)
before continuing to write ```deriv1```
and ```dTheta``` functions and so on.
Examination of the accompanying R functions,
their documentation in the rpf package,
and their output for well-known item models
may also provide insight into how the underlyling
C++ functions operate.

### Installing and testing from R

To test the underlying C++ from R, all
modifications to R files as mentioned in the
previous section ought to be done. However,
only C++ functions that you want to test
need to be implemented. Compile and install
the package for testing, e.g.,
```R CMD INSTALL rpf```.

Fire up R and begin testing.

```{r, message=FALSE}
library(rpf)

lmp.item<-rpf.lmp(q=2) # create item w/ 5th order polynomial
par<-c(.69,.71,-.5,-8.48,.52,-3.32) # item parameters
theta<-seq(-3,3,.1) # grid for latent trait

## Test the traceline or "prob" C++ function
P<-rpf.prob(lmp.item, par, theta)

## Prettier plots than this are of course possible
plot(theta, P[2,], type="l", ylim=c(0,1), xlab="Theta", ylab="P(Theta)")

```

The plot above should look like Example 2 from
[Falk and Cai (in press)](https://dx.doi.org/10.1007/s11336-014-9428-7).

Access to derivatives w.r.t. item parameters and
latent traits is also possible. Note that ```rpf.dLL```
appears to call both ```deriv1``` and ```deriv2```
C++ functions.

```{r}
## Derivatives of negative log-likelihood at arbitrary point with arbitrary weights
## Rounding only for easy reading for tutorial
round(rpf.dLL(lmp.item, par, theta[1], weight=c(5,7)),2)
```

For latent traits, with easy ways of testing
accuracy against numerical derivatives - at
least for a unidimensional model.

```{r}
## Analytical derivatives "deriv1" followed by "deriv2"
rpf.dTheta(lmp.item, par, where=-.5, dir=1)

## Numerical derivatives
library(numDeriv)
dTheta.wrap<-function(theta, spec, par, cat=1){
  rpf.prob(spec, par, theta)[cat]
}

## should match first element of gradient from rpf.dTheta
grad(dTheta.wrap, -.5, spec=lmp.item, par=par)

## should match first element of hessian from rpf.dTheta
hessian(dTheta.wrap, -.5, spec=lmp.item, par=par)

```

One could also test numerical derivatives of the
log-likelihood w.r.t. item parameters in a similar manner.
However, this may take a little extra work, and
note that numerical derivatives may not always
work well, especially when testing during
estimation and when parameter estimates are far
from the MLE.


### Add formal testing files

If any formal testing is done that the
user wishes to be reproducible, which is
highly encouraged and some may argue is
necessary, these formal tests can currently
be added to inst/tests/

### Testing estimation

Forthcoming.

### Generating documentation

Forthcoming.

<!--
%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Basic 1 dimensional plots (HTML)}
-->

```{r, cache=FALSE, include=FALSE}
library(knitr)
library(rpf)
library(ggplot2)
library(reshape2)
library(gridExtra)
opts_chunk$set(echo=FALSE)
```

# Basic 1 dimensional plots

This template is suitable for plotting a single group
of 1 dimensional items.
Refer to the Rmd source code to see how to adapt this
template to your project.

## Load data

Normally you would obtain model parameters from [OpenMx](https://openmx.ssri.psu.edu/)
(or flexMIRT with flexmirt.read), but here are some inline
parameters for demonstration purposes.

```{r,echo=TRUE}
small <- structure(list(param = structure(c(1, 1, 0, 0, -0.5789195, -2.412259,  -1.3471789, 1, 1, 0, 0, 1.0983234, -2.0991327, -2.9482965, 1,  1, 0, 0, 0.4078264, -0.9824549, -1.5905594, 1, 1, 0, 0, -1.0650001,  -0.2100243, -3.2034577), .Dim = c(7L, 4L), .Dimnames = list(NULL,      c("X2", "X6", "X7", "X10"))), mean = 0, cov = structure(9.8238066, .Dim = c(1L,  1L))), .Names = c("param", "mean", "cov"))
small$spec <- list()
small$spec[1:4] <- rpf.nrm(outcomes=4, T.c= lower.tri(diag(3),TRUE) * -1)
```

## Item characteristic curve plots

```{r,fig.height=5}
width <- 5
small$icc <- list()
small$iif <- list()

tcc <- expand.grid(theta=seq(-width,width,.1), score=0)
tic <- expand.grid(theta=seq(-width,width,.1), info=0)

for (ix in 1:length(small$spec)) {
    name <- colnames(small$param)[ix]
    ii <- small$spec[[ix]]
    ii.p <- small$param[,ix]
    grid <- expand.grid(theta=seq(-width,width,.1))
    grid <- cbind(grid, t(rpf.prob(ii, ii.p, grid$theta)))
    tcc$score <- tcc$score + c(0:(ii@outcomes-1) %*% rpf.prob(ii, ii.p, grid$theta))
    colnames(grid) <- c("theta", paste0("k", 0:3))
    grid2 <- melt(grid, id.vars=c("theta"), variable.name="category", value.name="p")

    small$icc[[ix]] <- ggplot(grid2, aes(theta, p, color=category)) + geom_line() +
            ggtitle(paste("Item", name)) + ylim(0,1) + xlim(-width, width)

    grid <- expand.grid(theta=seq(-width,width,.1))
    grid$info <- rpf.info(ii, ii.p, t(grid$theta))
    tic$info <- tic$info + grid$info
    small$iif[[ix]] <- ggplot(grid, aes(theta, info)) + geom_line() +
            ggtitle(paste("Item", name)) + xlim(-width, width)
  }
do.call(grid.arrange, c(small$icc, ncol=2))
```

## Item information curve plots

```{r,fig.height=5}
do.call(grid.arrange, c(small$iif, ncol=2))
```

## Whole test plots

```{r,fig.height=2.5}
tcc.plot <- ggplot(tcc, aes(theta, score)) + geom_line() +
  ggtitle("Test Characteristic Curve") + xlim(-width, width)
tic.plot <- ggplot(tic, aes(theta, info)) + geom_line() +
  ggtitle("Test Information Curve") + xlim(-width, width)
grid.arrange(tcc.plot, tic.plot, ncol=2)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{rpf.modify}
\alias{rpf.modify}
\alias{rpf.modify,rpf.mdim.drm,numeric-method}
\alias{rpf.modify,rpf.mdim.graded,numeric-method}
\alias{rpf.modify,rpf.mdim.nrm,numeric-method}
\title{Create a similar item specification with the given number of factors}
\usage{
rpf.modify(m, factors)
}
\arguments{
\item{m}{item model}

\item{factors}{the number of factors/dimensions}
}
\description{
Create a similar item specification with the given number of factors
}
\examples{
s1 <- rpf.grm(factors=3)
rpf.rparam(s1)
s2 <- rpf.modify(s1, 1)
rpf.rparam(s2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{sumScoreEAP}
\alias{sumScoreEAP}
\title{Compute the sum-score EAP table}
\usage{
sumScoreEAP(grp, ..., qwidth = 6, qpoints = 49L, .twotier = TRUE)
}
\arguments{
\item{grp}{a list containing the model and data. See the details section.}

\item{...}{Not used.  Forces remaining arguments to be specified by name.}

\item{qwidth}{DEPRECATED}

\item{qpoints}{DEPRECATED}

\item{.twotier}{whether to enable the two-tier optimization}
}
\description{
Observed tables cannot be computed when data is
missing. Therefore, you can optionally omit items with the
greatest number of responses missing when conducting the
distribution test.
}
\details{
When two-tier covariance structure is detected, EAP scores are
only reported for primary factors. It is possible to compute EAP
scores for specific factors, but it is not clear why this would be
useful because they are conditional on the specific factor sum
scores. Moveover, the algorithm to compute them efficiently has not been
published yet (as of Jun 2014).
}
\section{Format of a group}{


A model, or group within a model, is represented as a named list.
\describe{
\item{spec}{list of response model objects}
\item{param}{numeric matrix of item parameters}
\item{free}{logical matrix of indicating which parameters are free (TRUE) or fixed (FALSE)}
\item{mean}{numeric vector giving the mean of the latent distribution}
\item{cov}{numeric matrix giving the covariance of the latent distribution}
\item{data}{data.frame containing observed item responses, and optionally,
weights and frequencies}
\item{score}{factors scores with response patterns in rows}
\item{weightColumn}{name of the data column containing the numeric row weights (optional)}
\item{freqColumn}{name of the data column containing the integral row frequencies (optional)}
\item{qwidth}{width of the quadrature expressed in Z units}
\item{qpoints}{number of quadrature points}
\item{minItemsPerScore}{minimum number of non-missing items when estimating factor scores}
}

The \code{param} matrix stores items parameters by column. If a
column has more rows than are required to fully specify a model
then the extra rows are ignored.  The order of the items in
\code{spec} and order of columns in \code{param} are assumed to
match. All items should have the same number of latent dimensions.
Loadings on latent dimensions are given in the first few rows and
can be named by setting rownames. Item names are assigned by
\code{param} colnames.

Currently only a multivariate normal distribution is available,
parameterized by the \code{mean} and \code{cov}. If \code{mean} and
\code{cov} are not specified then a standard normal distribution is
assumed. The quadrature consists of equally spaced points. For
example, \code{qwidth=2} and \code{qpoints=5} would produce points
-2, -1, 0, 1, and 2. The quadrature specification is part of the
group and not passed as extra arguments for the sake of
consistency. As currently implemented, OpenMx uses EAP scores to
estimate latent distribution parameters. By default, the exact same
EAP scores should be produced by \link{EAPscores}.
}

\examples{
# see Thissen, Pommerich, Billeaud, & Williams (1995, Table 2)
 spec <- list()
 spec[1:3] <- list(rpf.grm(outcomes=4))

 param <- matrix(c(1.87, .65, 1.97, 3.14,
                   2.66, .12, 1.57, 2.69,
                   1.24, .08, 2.03, 4.3), nrow=4)
 # fix parameterization
 param <- apply(param, 2, function(p) c(p[1], p[2:4] * -p[1]))

 grp <- list(spec=spec, mean=0, cov=matrix(1,1,1), param=param)
 sumScoreEAP(grp)
}
\seealso{
Other scoring: 
\code{\link{EAPscores}()},
\code{\link{bestToOmit}()},
\code{\link{itemOutcomeBySumScore}()},
\code{\link{observedSumScore}()},
\code{\link{omitItems}()},
\code{\link{omitMostMissing}()}
}
\concept{scoring}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{fromFactorThreshold}
\alias{fromFactorThreshold}
\title{Convert factor thresholds to response function intercepts}
\usage{
fromFactorThreshold(threshold, loading, ogive = rpf.ogive)
}
\arguments{
\item{threshold}{a matrix with items in the columns and thresholds in the rows}

\item{loading}{a matrix with items in the rows and factors in the columns}

\item{ogive}{the ogive constant (default \link{rpf.ogive})}
}
\value{
an item intercept matrix with items in the columns and intercepts in the rows
}
\description{
Convert factor thresholds to response function intercepts
}
\seealso{
Other factor model equivalence: 
\code{\link{fromFactorLoading}()},
\code{\link{toFactorLoading}()},
\code{\link{toFactorThreshold}()}
}
\concept{factor model equivalence}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/science.R
\docType{data}
\name{science}
\alias{science}
\alias{sfif}
\alias{sfpf}
\alias{sfsf}
\alias{sfxf}
\title{Liking for Science dataset}
\description{
These data are from Wright & Masters (1982, p. 18).
}
\details{
All items were fit to a 3 category Partial Credit Model (PCM)
using Ministep 3.75.0.
}
\examples{
data(science)
}
\references{
Wright, B. D. & Masters, G. N. (1982). \emph{Rating Scale
Analysis.} Chicago: Mesa Press.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{Class rpf.base}
\alias{Class rpf.base}
\alias{rpf.base-class}
\alias{$,rpf.base-method}
\alias{$<-,rpf.base-method}
\title{The base class for response probability functions.}
\description{
Item specifications should not be modified after creation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{Class rpf.mdim.graded}
\alias{Class rpf.mdim.graded}
\alias{rpf.mdim.graded-class}
\title{The base class for multi-dimensional graded response probability
functions.}
\description{
This class contains methods common to both the generalized partial
credit model and the graded response model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{data}
\name{rpf.ogive}
\alias{rpf.ogive}
\title{The ogive constant}
\format{
An object of class \code{numeric} of length 1.
}
\usage{
rpf.ogive
}
\description{
The ogive constant can be multiplied by the discrimination
parameter to obtain a response curve very similar to the Normal
cumulative distribution function (Haley, 1952; Molenaar, 1974).
Recently, Savalei (2006) proposed a new constant of 1.749 based on
Kullback-Leibler information.
}
\details{
In recent years, the logistic has grown in favor, and therefore,
this package does not offer any special support for this
transformation (Baker & Kim, 2004, pp. 14-18).
}
\references{
Camilli, G. (1994). Teacher's corner: Origin of the
scaling constant d=1.7 in Item Response Theory. \emph{Journal of
Educational and Behavioral Statistics, 19}(3), 293-295.

Baker & Kim (2004). \emph{Item Response Theory: Parameter
Estimation Techniques.} Marcel Dekker, Inc.

Haley, D. C. (1952). \emph{Estimation of the dosage mortality
relationship when the dose is subject to error} (Technical Report
No. 15). Stanford University Applied Mathematics and Statistics
Laboratory, Stanford, CA.

Molenaar, W. (1974). De logistische en de normale kromme [The
logistic and the normal curve]. \emph{Nederlands Tijdschrift voor de
Psychologie} 29, 415-420.

Savalei, V. (2006). Logistic approximation to the normal: The KL
rationale. \emph{Psychometrika, 71}(4), 763--767.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnose.R
\name{SitemFit1}
\alias{SitemFit1}
\title{Compute the S fit statistic for 1 item}
\usage{
SitemFit1(
  grp,
  item,
  free = 0,
  ...,
  method = "pearson",
  log = TRUE,
  qwidth = 6,
  qpoints = 49L,
  alt = FALSE,
  omit = 0L,
  .twotier = TRUE
)
}
\arguments{
\item{grp}{a list containing the model and data. See the details section.}

\item{item}{the item of interest}

\item{free}{the number of free parameters involved in estimating the item (to adjust the df)}

\item{...}{Not used.  Forces remaining arguments to be specified by name.}

\item{method}{whether to use a pearson or rms test}

\item{log}{whether to return p-values in log units}

\item{qwidth}{\lifecycle{deprecated}}

\item{qpoints}{\lifecycle{deprecated}}

\item{alt}{whether to include the item of interest in the denominator}

\item{omit}{number of items to omit or a character vector with the names of the items to omit when calculating the observed and expected sum-score tables}

\item{.twotier}{whether to enable the two-tier optimization}
}
\description{
Implements the Kang & Chen (2007) polytomous extension to
S statistic of Orlando & Thissen (2000). Rows with
missing data are ignored, but see the \code{omit} option.
}
\details{
This statistic is good at finding a small number of misfitting
items among a large number of well fitting items. However, be
aware that misfitting items can cause other items to misfit.

Observed tables cannot be computed when data is
missing. Therefore, you can optionally omit items with the
greatest number of responses missing relative to the item of
interest.

Pearson is slightly more powerful than RMS in most cases I
examined.

Setting \code{alt} to \code{TRUE} causes the tables to match
published articles. However, the default setting of \code{FALSE}
probably provides slightly more power when there are less than 10
items.

The name of the test, "S", probably stands for sum-score.
}
\section{Format of a group}{


A model, or group within a model, is represented as a named list.
\describe{
\item{spec}{list of response model objects}
\item{param}{numeric matrix of item parameters}
\item{free}{logical matrix of indicating which parameters are free (TRUE) or fixed (FALSE)}
\item{mean}{numeric vector giving the mean of the latent distribution}
\item{cov}{numeric matrix giving the covariance of the latent distribution}
\item{data}{data.frame containing observed item responses, and optionally,
weights and frequencies}
\item{score}{factors scores with response patterns in rows}
\item{weightColumn}{name of the data column containing the numeric row weights (optional)}
\item{freqColumn}{name of the data column containing the integral row frequencies (optional)}
\item{qwidth}{width of the quadrature expressed in Z units}
\item{qpoints}{number of quadrature points}
\item{minItemsPerScore}{minimum number of non-missing items when estimating factor scores}
}

The \code{param} matrix stores items parameters by column. If a
column has more rows than are required to fully specify a model
then the extra rows are ignored.  The order of the items in
\code{spec} and order of columns in \code{param} are assumed to
match. All items should have the same number of latent dimensions.
Loadings on latent dimensions are given in the first few rows and
can be named by setting rownames. Item names are assigned by
\code{param} colnames.

Currently only a multivariate normal distribution is available,
parameterized by the \code{mean} and \code{cov}. If \code{mean} and
\code{cov} are not specified then a standard normal distribution is
assumed. The quadrature consists of equally spaced points. For
example, \code{qwidth=2} and \code{qpoints=5} would produce points
-2, -1, 0, 1, and 2. The quadrature specification is part of the
group and not passed as extra arguments for the sake of
consistency. As currently implemented, OpenMx uses EAP scores to
estimate latent distribution parameters. By default, the exact same
EAP scores should be produced by \link{EAPscores}.
}

\references{
Kang, T. and Chen, T. T. (2007). An investigation of
the performance of the generalized S-Chisq item-fit index for
polytomous IRT models. ACT Research Report Series.

Orlando, M. and Thissen, D. (2000). Likelihood-Based
Item-Fit Indices for Dichotomous Item Response Theory Models.
\emph{Applied Psychological Measurement, 24}(1), 50-64.
}
\seealso{
Other diagnostic: 
\code{\link{ChenThissen1997}()},
\code{\link{SitemFit}()},
\code{\link{multinomialFit}()},
\code{\link{rpf.1dim.fit}()},
\code{\link{sumScoreEAPTest}()}
}
\concept{diagnostic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{rpf.rescale}
\alias{rpf.rescale}
\alias{_rpf_rescale}
\alias{rpf.rescale,rpf.base,numeric,numeric,matrix-method}
\title{Rescale item parameters}
\usage{
rpf.rescale(m, param, mean, cov)
}
\arguments{
\item{m}{item model}

\item{param}{item parameters}

\item{mean}{vector of means}

\item{cov}{covariance matrix}
}
\description{
Adjust item parameters for changes in mean and covariance of the
latent distribution.
}
\examples{
spec <- rpf.grm()
p1 <- rpf.rparam(spec)
testPoint <- rnorm(1)
move <- rnorm(1)
cov <- as.matrix(rlnorm(1))
Icov <- solve(cov)
padj <- rpf.rescale(spec, p1, move, cov)
pr1 <- rpf.prob(spec, padj, (testPoint-move) \%*\% Icov)
pr2 <- rpf.prob(spec, p1, testPoint)
abs(pr1 - pr2) < 1e9
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LSAT.R
\docType{data}
\name{LSAT7}
\alias{LSAT7}
\title{Description of LSAT7 data}
\description{
Data from Bock & Lieberman (1970); contains 5 dichotomously scored
items obtained from the Law School Admissions Test, section 7.
}
\examples{
data(LSAT7)
}
\references{
Bock, R. D., & Lieberman, M. (1970). Fitting a response model for \emph{n}
dichotomously scored items. \emph{Psychometrika, 35}(2), 179-197.
}
\author{
Phil Chalmers \email{rphilip.chalmers@gmail.com}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnose.R
\name{rpf.mean.info1}
\alias{rpf.mean.info1}
\title{Find the point where an item provides mean maximum information}
\usage{
rpf.mean.info1(spec, iparam, grain = 0.1)
}
\arguments{
\item{spec}{an item spec}

\item{iparam}{an item parameter vector}

\item{grain}{the step size for numerical integration (optional)}
}
\description{
\lifecycle{experimental}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{Class rpf.mdim.nrm}
\alias{Class rpf.mdim.nrm}
\alias{rpf.mdim.nrm-class}
\title{The nominal response item model (both unidimensional and
multidimensional models have the same parameterization).}
\description{
The nominal response item model (both unidimensional and
multidimensional models have the same parameterization).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grm.R
\name{rpf.grm}
\alias{rpf.grm}
\title{Create a graded response model}
\usage{
rpf.grm(outcomes = 2, factors = 1, multidimensional = TRUE)
}
\arguments{
\item{outcomes}{The number of choices available}

\item{factors}{the number of factors}

\item{multidimensional}{whether to use a multidimensional model.
Defaults to \code{TRUE}.}
}
\value{
an item model
}
\description{
For outcomes k in 0 to K, slope vector a, intercept vector c, and latent ability vector theta,
the response probability function is
\deqn{\mathrm P(\mathrm{pick}=0|a,c,\theta) = 1- \mathrm P(\mathrm{pick}=1|a,c_1,\theta)
}{P(pick=0|a,c,th) = 1-P(pick=1|a,c_1,th)}
\deqn{\mathrm P(\mathrm{pick}=k|a,c,\theta) = \frac{1}{1+\exp(-(a\theta + c_k))} - \frac{1}{1+\exp(-(a\theta + c_{k+1}))}
}{P(pick=k|a,c,th) = 1/(1+exp(-(a th + c_k))) - 1/(1+exp(-(a th + c_(k+1))))}
\deqn{\mathrm P(\mathrm{pick}=K|a,c,\theta) = \frac{1}{1+\exp(-(a\theta + c_K))}
}{P(pick=K|a,c,th) = 1/(1+exp(-(a th + c_K)))}
}
\details{
The graded response model was designed for a item with a series of
dependent parts where a higher score implies that easier parts of
the item were surmounted. If there is any chance your polytomous
item has independent parts then consider \code{\link{rpf.nrm}}.
If your categories cannot cross then the graded response model
provides a little more information than the nominal model.
Stronger a priori assumptions offer provide more power at the cost
of flexibility.
}
\examples{
spec <- rpf.grm()
rpf.prob(spec, rpf.rparam(spec), 0)
}
\seealso{
Other response model: 
\code{\link{rpf.drm}()},
\code{\link{rpf.gpcmp}()},
\code{\link{rpf.grmp}()},
\code{\link{rpf.lmp}()},
\code{\link{rpf.mcm}()},
\code{\link{rpf.nrm}()}
}
\concept{response model}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{rpf.numSpec}
\alias{rpf.numSpec}
\alias{rpf.numSpec,rpf.base-method}
\alias{rpf_numSpec_wrapper}
\title{Length of the item model vector}
\usage{
rpf.numSpec(m)
}
\arguments{
\item{m}{item model}
}
\description{
Length of the item model vector
}
\examples{
rpf.numSpec(rpf.grm(outcomes=3))
rpf.numSpec(rpf.nrm(outcomes=3))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{Class rpf.1dim.graded}
\alias{Class rpf.1dim.graded}
\alias{rpf.1dim.graded-class}
\title{The base class for 1 dimensional graded response probability functions.}
\description{
This class contains methods common to both the generalized partial
credit model and the graded response model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataframe.R
\name{tabulateRows}
\alias{tabulateRows}
\title{Tabulate data.frame rows}
\usage{
tabulateRows(observed)
}
\arguments{
\item{observed}{a sorted data.frame holding ordered factors in every column}
}
\description{
Like \code{tabulate} but entire rows are the unit of tabulation.
The data.frame is not sorted, but must be sorted already.
}
\examples{
df <- as.data.frame(matrix(c(sample.int(2, 30, replace=TRUE)), 10, 3))
df <- df[orderCompletely(df),]
tabulateRows(df)
}
\seealso{
\code{\link{orderCompletely}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nrm.R
\name{rpf.nrm}
\alias{rpf.nrm}
\title{Create a nominal response model}
\usage{
rpf.nrm(outcomes = 3, factors = 1, T.a = "trend", T.c = "trend")
}
\arguments{
\item{outcomes}{The number of choices available}

\item{factors}{the number of factors}

\item{T.a}{the T matrix for slope parameters}

\item{T.c}{the T matrix for intercept parameters}
}
\value{
an item model
}
\description{
This function instantiates a nominal response model.
}
\details{
The transformation matrices T.a and T.c are chosen by the analyst
and not estimated.  The T matrices must be invertible square
matrices of size outcomes-1. As a shortcut, either T matrix
can be specified as "trend" for a Fourier basis or as "id" for an
identity basis. The response probability function is

\deqn{a = T_a \alpha}
\deqn{c = T_c \gamma}
\deqn{\mathrm P(\mathrm{pick}=k|s,a_k,c_k,\theta) = C\ \frac{1}{1+\exp(-(s \theta a_k + c_k))}}

where \eqn{a_k} and \eqn{c_k} are the result of multiplying two vectors
of free parameters \eqn{\alpha} and \eqn{\gamma} by fixed matrices \eqn{T_a} and \eqn{T_c}, respectively;
\eqn{a_0} and \eqn{c_0} are fixed to 0 for identification;
and \eqn{C} is a normalizing factor to ensure that \eqn{\sum_k \mathrm P(\mathrm{pick}=k) = 1}.
}
\examples{
spec <- rpf.nrm()
rpf.prob(spec, rpf.rparam(spec), 0)
# typical parameterization for the Generalized Partial Credit Model
gpcm <- function(outcomes) rpf.nrm(outcomes, T.c=lower.tri(diag(outcomes-1),TRUE) * -1)
spec <- gpcm(4)
rpf.prob(spec, rpf.rparam(spec), 0)
}
\references{
Thissen, D., Cai, L., & Bock, R. D. (2010). The
Nominal Categories Item Response Model. In M. L. Nering &
R. Ostini (Eds.), \emph{Handbook of Polytomous Item Response
Theory Models} (pp. 43--75). Routledge.
}
\seealso{
Other response model: 
\code{\link{rpf.drm}()},
\code{\link{rpf.gpcmp}()},
\code{\link{rpf.grmp}()},
\code{\link{rpf.grm}()},
\code{\link{rpf.lmp}()},
\code{\link{rpf.mcm}()}
}
\concept{response model}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{Class rpf.1dim.lmp}
\alias{Class rpf.1dim.lmp}
\alias{rpf.1dim.lmp-class}
\title{Unidimensional logistic function of a monotonic polynomial.}
\description{
Unidimensional logistic function of a monotonic polynomial.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{rpf.dTheta}
\alias{rpf.dTheta}
\alias{_rpf_dTheta}
\alias{rpf.dTheta,rpf.base,numeric,numeric,numeric-method}
\alias{rpf.dTheta,rpf.base,numeric,matrix,numeric-method}
\title{Item derivatives with respect to the location in the latent space}
\usage{
rpf.dTheta(m, param, where, dir)
}
\arguments{
\item{m}{item model}

\item{param}{item parameters}

\item{where}{location in the latent distribution}

\item{dir}{if more than 1 factor, a basis vector}
}
\description{
Evaluate the partial derivatives of the response probability with
respect to ability. See \link{rpf.info} for an application.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drm.R
\name{logit}
\alias{logit}
\title{Transform from [0,1] to the reals}
\usage{
logit(p, location = 0, scale = 1, lower.tail = TRUE, log.p = FALSE)
}
\arguments{
\item{p}{a number between 0 and 1}

\item{location}{see qlogis}

\item{scale}{see qlogis}

\item{lower.tail}{see qlogis}

\item{log.p}{see qlogis}
}
\description{
The logit function is a standard transformation from [0,1] (such
as a probability) to the real number line. This function is
exactly the same as qlogis.
}
\examples{
logit(.5)  # 0
logit(.25) # -1.098
logit(0)   # -Inf
}
\seealso{
qlogis, plogis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{collapseCategoricalCells}
\alias{collapseCategoricalCells}
\title{Collapse small sample size categorical frequency counts}
\usage{
collapseCategoricalCells(observed, expected, minExpected = 1)
}
\arguments{
\item{observed}{the observed frequency table}

\item{expected}{the expected frequency table}

\item{minExpected}{the minimum expected cell frequency

Pearson's X^2 test requires some minimum frequency per cell to
avoid an inflated false positive rate. This function will merge
cells with the lowest frequency counts until all the counts are
above the minimum threshold. Cells that have been merged are filled
with NAs. The resulting tables and number of merged cells is
returned.}
}
\description{
Collapse small sample size categorical frequency counts
}
\examples{
O = matrix(c(7,31,42,20,0), 1,5)
E = matrix(c(3,39,50,8,0), 1,5)
collapseCategoricalCells(O,E,9)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drm.R
\name{rpf.drm}
\alias{rpf.drm}
\title{Create a dichotomous response model}
\usage{
rpf.drm(factors = 1, multidimensional = TRUE, poor = FALSE)
}
\arguments{
\item{factors}{the number of factors}

\item{multidimensional}{whether to use a multidimensional model.
Defaults to \code{TRUE}.}

\item{poor}{if TRUE, use the traditional parameterization of
the 1d model instead of the slope-intercept parameterization}
}
\value{
an item model
}
\description{
For slope vector a, intercept c, pseudo-guessing parameter g,
upper bound u, and latent ability vector theta, the response probability
function is
\deqn{\mathrm P(\mathrm{pick}=0|a,c,g,u,\theta) = 1- \mathrm P(\mathrm{pick}=1|a,c,g,u,\theta)
}{P(pick=0|a,c,g,u,th) = 1-P(pick=1|a,c,g,u,th)}
\deqn{\mathrm P(\mathrm{pick}=1|a,c,g,u,\theta) = g+(u-g)\frac{1}{1+\exp(-(a\theta + c))}
}{P(pick=1|a,c,g,u,th) = g+(u-g)/(1+exp(-(a th + c)))}
}
\details{
The pseudo-guessing and upper bound parameter are specified in
logit units (see \code{\link{logit}}).

For discussion on the choice of priors see Cai, Yang, and
Hansen (2011, p. 246).
}
\examples{
spec <- rpf.drm()
rpf.prob(spec, rpf.rparam(spec), 0)
}
\references{
Cai, L., Yang, J. S., & Hansen, M. (2011). Generalized
Full-Information Item Bifactor Analysis.  \emph{Psychological
Methods, 16}(3), 221-248.
}
\seealso{
Other response model: 
\code{\link{rpf.gpcmp}()},
\code{\link{rpf.grmp}()},
\code{\link{rpf.grm}()},
\code{\link{rpf.lmp}()},
\code{\link{rpf.mcm}()},
\code{\link{rpf.nrm}()}
}
\concept{response model}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{Class rpf.1dim.drm}
\alias{Class rpf.1dim.drm}
\alias{rpf.1dim.drm-class}
\title{Unidimensional dichotomous item models (1PL, 2PL, and 3PL).}
\description{
Unidimensional dichotomous item models (1PL, 2PL, and 3PL).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataframe.R
\name{expandDataFrame}
\alias{expandDataFrame}
\title{Expand summary table of patterns and frequencies}
\usage{
expandDataFrame(tabdata, freqName = NULL)
}
\arguments{
\item{tabdata}{An object of class \code{data.frame} with the unique response patterns and the number of frequencies}

\item{freqName}{Column name containing the frequencies}
}
\value{
Returns a data frame with all the response patterns
}
\description{
Expand a summary table of unique response patterns to a full sized
data-set.
}
\examples{
data(LSAT7)
expandDataFrame(LSAT7, freqName="freq")
}
\author{
Based on code by Phil Chalmers \email{rphilip.chalmers@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{toFactorThreshold}
\alias{toFactorThreshold}
\title{Convert response function intercepts to factor thresholds}
\usage{
toFactorThreshold(intercept, slope, ogive = rpf.ogive)
}
\arguments{
\item{intercept}{a matrix with items in the columns and intercepts in the rows}

\item{slope}{a matrix with items in the columns and slopes in the rows}

\item{ogive}{the ogive constant (default \link{rpf.ogive})}
}
\value{
a factor threshold matrix with items in the columns and factor thresholds in the rows
}
\description{
Convert response function intercepts to factor thresholds
}
\seealso{
Other factor model equivalence: 
\code{\link{fromFactorLoading}()},
\code{\link{fromFactorThreshold}()},
\code{\link{toFactorLoading}()}
}
\concept{factor model equivalence}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnose.R
\name{ordinal.gamma}
\alias{ordinal.gamma}
\title{Compute the ordinal gamma association statistic}
\usage{
ordinal.gamma(mat)
}
\arguments{
\item{mat}{a cross tabulation matrix}
}
\description{
Compute the ordinal gamma association statistic
}
\examples{
# Example data from Agresti (1990, p. 21)
jobsat <- matrix(c(20,22,13,7,24,38,28,18,80,104,81,54,82,125,113,92), nrow=4, ncol=4)
ordinal.gamma(jobsat)
}
\references{
Agresti, A. (1990). Categorical data analysis. New York: Wiley.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{Class rpf.mdim}
\alias{Class rpf.mdim}
\alias{rpf.mdim-class}
\title{The base class for multi-dimensional response probability functions.}
\description{
The base class for multi-dimensional response probability functions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kct.R
\docType{data}
\name{kct}
\alias{kct}
\alias{kct.items}
\alias{kct.people}
\title{Knox Cube Test dataset}
\description{
These data from Wright & Stone (1979, p. 31) were fit with
Winsteps 3.73 using a 1PL model (slope fixed to 1).
}
\examples{
data(kct)
}
\references{
Wright, B. D. & Stone, M. H. (1979). \emph{Best Test Design: Rasch Measurement.} Univ of Chicago Social Research.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnose.R
\name{rpf.mean.info}
\alias{rpf.mean.info}
\title{Find the point where an item provides mean maximum information}
\usage{
rpf.mean.info(spec, param, grain = 0.1)
}
\arguments{
\item{spec}{list of item specs}

\item{param}{list or matrix of item parameters}

\item{grain}{the step size for numerical integration (optional)}
}
\description{
\lifecycle{experimental}
This is a point estimate of the mean difficulty of items that do
not offer easily interpretable parameters such as the Generalized
PCM. Since the information curve may not be unimodal, this
function integrates across the latent space.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{methods}
\name{rpf.rparam}
\alias{rpf.rparam}
\alias{rpf.rparam,rpf.1dim.drm-method}
\alias{rpf.rparam,rpf.mdim.drm-method}
\alias{rpf.rparam,rpf.1dim.graded-method}
\alias{rpf.rparam,rpf.mdim.graded-method}
\alias{rpf.rparam,rpf.mdim.nrm-method}
\alias{rpf.rparam,rpf.mdim.mcm-method}
\alias{rpf.rparam,rpf.1dim.lmp-method}
\alias{rpf.rparam,rpf.1dim.grmp-method}
\alias{rpf.rparam,rpf.1dim.gpcmp-method}
\title{Generates item parameters}
\usage{
rpf.rparam(m, version = 2L)
}
\arguments{
\item{m}{an item model}

\item{version}{the version of random parameters}
}
\value{
item parameters
}
\description{
This function generates random item parameters. The version
argument is available if you are writing a test that depends on
reproducable random parameters (using \code{set.seed}).
}
\examples{
i1 <- rpf.drm()
rpf.rparam(i1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnose.R
\name{ChenThissen1997}
\alias{ChenThissen1997}
\alias{chen.thissen.1997}
\title{Computes local dependence indices for all pairs of items}
\usage{
ChenThissen1997(
  grp,
  ...,
  data = NULL,
  inames = NULL,
  qwidth = 6,
  qpoints = 49,
  method = "pearson",
  .twotier = TRUE,
  .parallel = TRUE
)
}
\arguments{
\item{grp}{a list containing the model and data. See the details section.}

\item{...}{Not used.  Forces remaining arguments to be specified by name.}

\item{data}{data \lifecycle{deprecated}}

\item{inames}{a subset of items to examine}

\item{qwidth}{\lifecycle{deprecated}}

\item{qpoints}{\lifecycle{deprecated}}

\item{method}{method to use to calculate P values. The default is the
Pearson X^2 statistic. Use "lr" for the similar likelihood ratio statistic.}

\item{.twotier}{whether to enable the two-tier optimization}

\item{.parallel}{whether to take advantage of multiple CPUs (default TRUE)}
}
\value{
a list with raw, pval and detail. The pval matrix is a
lower triangular matrix of log P values with the sign
determined by relative association between the observed and
expected tables (see \code{\link{ordinal.gamma}})
}
\description{
Item Factor Analysis makes two assumptions: (1) that the latent
distribution is reasonably approximated by the multivariate Normal
and (2) that items are conditionally independent. This test
examines the second assumption. The presence of locally dependent
items can inflate the precision of estimates causing a test to
seem more accurate than it really is.
}
\details{
Statically significant entries suggest that the item pair has
local dependence. Since log(.01)=-4.6, an absolute magitude of 5
is a reasonable cut-off. Positive entries indicate that the two
item residuals are more correlated than expected. These items may share an
unaccounted for latent dimension. Consider a redesign of the items
or the use of testlets for scoring. Negative entries indicate that
the two item residuals are less correlated than expected.
}
\section{Format of a group}{


A model, or group within a model, is represented as a named list.
\describe{
\item{spec}{list of response model objects}
\item{param}{numeric matrix of item parameters}
\item{free}{logical matrix of indicating which parameters are free (TRUE) or fixed (FALSE)}
\item{mean}{numeric vector giving the mean of the latent distribution}
\item{cov}{numeric matrix giving the covariance of the latent distribution}
\item{data}{data.frame containing observed item responses, and optionally,
weights and frequencies}
\item{score}{factors scores with response patterns in rows}
\item{weightColumn}{name of the data column containing the numeric row weights (optional)}
\item{freqColumn}{name of the data column containing the integral row frequencies (optional)}
\item{qwidth}{width of the quadrature expressed in Z units}
\item{qpoints}{number of quadrature points}
\item{minItemsPerScore}{minimum number of non-missing items when estimating factor scores}
}

The \code{param} matrix stores items parameters by column. If a
column has more rows than are required to fully specify a model
then the extra rows are ignored.  The order of the items in
\code{spec} and order of columns in \code{param} are assumed to
match. All items should have the same number of latent dimensions.
Loadings on latent dimensions are given in the first few rows and
can be named by setting rownames. Item names are assigned by
\code{param} colnames.

Currently only a multivariate normal distribution is available,
parameterized by the \code{mean} and \code{cov}. If \code{mean} and
\code{cov} are not specified then a standard normal distribution is
assumed. The quadrature consists of equally spaced points. For
example, \code{qwidth=2} and \code{qpoints=5} would produce points
-2, -1, 0, 1, and 2. The quadrature specification is part of the
group and not passed as extra arguments for the sake of
consistency. As currently implemented, OpenMx uses EAP scores to
estimate latent distribution parameters. By default, the exact same
EAP scores should be produced by \link{EAPscores}.
}

\references{
Chen, W.-H. & Thissen, D. (1997). Local dependence
indexes for item pairs using Item Response Theory. \emph{Journal
of Educational and Behavioral Statistics, 22}(3), 265-289.

Thissen, D., Steinberg, L., & Mooney, J. A. (1989). Trace lines for testlets: A use
of multiple-categorical-response models. \emph{Journal of Educational Measurement,
26} (3), 247--260.

Wainer, H. & Kiely, G. L. (1987). Item clusters and computerized
adaptive testing: A case for testlets.  \emph{Journal of
Educational measurement, 24}(3), 185--201.
}
\seealso{
\href{https://github.com/jpritikin/ifaTools}{ifaTools}

Other diagnostic: 
\code{\link{SitemFit1}()},
\code{\link{SitemFit}()},
\code{\link{multinomialFit}()},
\code{\link{rpf.1dim.fit}()},
\code{\link{sumScoreEAPTest}()}
}
\concept{diagnostic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnose.R
\docType{methods}
\name{rpf.1dim.moment}
\alias{rpf.1dim.moment}
\title{Calculate cell central moments}
\usage{
rpf.1dim.moment(spec, params, scores, m)
}
\arguments{
\item{spec}{list of item models}

\item{params}{data frame of item parameters, 1 per row}

\item{scores}{model derived person scores}

\item{m}{which moment}
}
\value{
moment matrix
}
\description{
Popular central moments include 2 (variance) and 4 (kurtosis).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LSAT.R
\docType{data}
\name{LSAT6}
\alias{LSAT6}
\title{Description of LSAT6 data}
\description{
Data from Thissen (1982); contains 5 dichotomously scored
items obtained from the Law School Admissions Test, section 6.
}
\examples{
data(LSAT6)
}
\references{
Thissen, D. (1982). Marginal maximum likelihood estimation for the one-parameter logistic model.
\emph{Psychometrika, 47}, 175-186.
}
\author{
Phil Chalmers \email{rphilip.chalmers@gmail.com}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gpcmp.R
\name{rpf.gpcmp}
\alias{rpf.gpcmp}
\title{Create monotonic polynomial generalized partial credit (GPC-MP) model}
\usage{
rpf.gpcmp(outcomes = 2, q = 0, multidimensional = FALSE)
}
\arguments{
\item{outcomes}{The number of possible response categories.}

\item{q}{a non-negative integer that controls the order of the
polynomial (2q+1) with a default of q=0 (1st order polynomial = generalized partial credit model).}

\item{multidimensional}{whether to use a multidimensional model.
Defaults to \code{FALSE}. The multidimensional version is not yet
available.}
}
\value{
an item model
}
\description{
This model is a polytomous model proposed by Falk & Cai (2016) and is based
on the generalized partial credit model (Muraki, 1992).
}
\details{
The GPC-MP replaces the linear predictor part of the
generalized partial credit model with a monotonic polynomial,
\eqn{m(\theta;\omega,\xi,\mathbf{\alpha},\mathbf{\tau})}{m(theta; omega, alpha, tau)}.
The response function for category k is:

\deqn{\mathrm P(\mathrm{pick}=k|\omega,\xi,\alpha,\tau,\theta)
= \frac{\exp(\sum_{v=0}^k (\xi_k + m(\theta;\omega,\xi,\mathbf{\alpha},\mathbf{\tau})))}{\sum_{u=0}^{K-1}\exp(\sum_{v=0}^u (\xi_u + m(\theta;\omega,\xi,\mathbf{\alpha},\mathbf{\tau})))}
}{\frac{exp(sum_{v=0}^k (xi_k + m(theta;omega,xi,alpha,tau)))}{sum_{u=0}^{K-1}exp(sum_{v=0}^u (xi_u + m(theta;omega,xi,alpha,tau)))}}

where \eqn{\mathbf{\alpha}}{alpha} and \eqn{\mathbf{\tau}}{tau} are vectors
of length q. The GPC-MP uses the same parameterization for the polynomial
as described for the logistic function of a monotonic polynomial (LMP).
See also (\code{\link{rpf.lmp}}).

The order of the polynomial is always odd and is controlled by
the user specified non-negative integer, q. The model contains
1+(outcomtes-1)+2*q parameters and are used as input to the \code{\link{rpf.prob}}
function in the following order:
\eqn{\omega}{omega} - natural log of the slope of the item model when q=0,
\eqn{\xi}{xi} - a (outcomes-1)-length vector of intercept parameters,
\eqn{\alpha}{alpha} and \eqn{\tau}{tau} - two parameters that control bends in
the polynomial. These latter parameters are repeated in the same order for
models with q>0. For example, a q=2 polynomial with 3 categories will have an item
parameter vector of: \eqn{\omega, \xi_1, \xi_2, \alpha_1, \tau_1, \alpha_2, \tau_2}{
omega, xi1, xi2, alpha1, tau1, alpha2, tau2}.

Note that the GPC-MP reduces to the LMP when the number of
categories is 2, and the GPC-MP reduces to the generalized partial
credit model when the order of the polynomial is 1 (i.e., q=0).
}
\examples{
spec <- rpf.gpcmp(5,2) # 5-category, 3rd order polynomial
theta<-seq(-3,3,.1)
p<-rpf.prob(spec, c(1.02,3.48,2.5,-.25,-1.64,.89,-8.7,-.74,-8.99),theta)

}
\references{
Falk, C. F., & Cai, L. (2016). Maximum marginal likelihood
estimation of a monotonic polynomial generalized partial credit model with
applications to multiple group analysis. \emph{Psychometrika, 81}, 434-460.
\doi{10.1007/s11336-014-9428-7}

Muraki, E. (1992). A generalized partial credit model: Application of an EM algorithm. \emph{Applied Psychological Measurement, 16,} 159–176.
}
\seealso{
Other response model: 
\code{\link{rpf.drm}()},
\code{\link{rpf.grmp}()},
\code{\link{rpf.grm}()},
\code{\link{rpf.lmp}()},
\code{\link{rpf.mcm}()},
\code{\link{rpf.nrm}()}
}
\concept{response model}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sample.R
\name{rpf.sample}
\alias{rpf.sample}
\title{Randomly sample response patterns given a list of items}
\usage{
rpf.sample(
  theta,
  items,
  params,
  ...,
  prefix = "i",
  mean = NULL,
  cov = NULL,
  mcar = 0,
  grp = NULL
)
}
\arguments{
\item{theta}{either a vector (for 1 dimension) or a matrix (for >1
dimension) of person abilities or the number of response patterns
to generate randomly}

\item{items}{a list of item models}

\item{params}{a list or matrix of item parameters. If omitted, random item
parameters are generated for each item model.}

\item{...}{Not used.  Forces remaining arguments to be specified by name.}

\item{prefix}{Column names are taken from param or items.
If no column names are available, some will be generated using
the given prefix.}

\item{mean}{mean vector of latent distribution (optional)}

\item{cov}{covariance matrix of latent distribution (optional)}

\item{mcar}{proportion of generated data to set to NA (missing completely at random)}

\item{grp}{a list containing the model and data. See the details section.}
}
\value{
Returns a data frame of response patterns
}
\description{
Returns a random sample of response patterns given a list of item
models and parameters. If \code{grp} is given then theta, items, params,
mean, and cov can be omitted.
}
\section{Format of a group}{


A model, or group within a model, is represented as a named list.
\describe{
\item{spec}{list of response model objects}
\item{param}{numeric matrix of item parameters}
\item{free}{logical matrix of indicating which parameters are free (TRUE) or fixed (FALSE)}
\item{mean}{numeric vector giving the mean of the latent distribution}
\item{cov}{numeric matrix giving the covariance of the latent distribution}
\item{data}{data.frame containing observed item responses, and optionally,
weights and frequencies}
\item{score}{factors scores with response patterns in rows}
\item{weightColumn}{name of the data column containing the numeric row weights (optional)}
\item{freqColumn}{name of the data column containing the integral row frequencies (optional)}
\item{qwidth}{width of the quadrature expressed in Z units}
\item{qpoints}{number of quadrature points}
\item{minItemsPerScore}{minimum number of non-missing items when estimating factor scores}
}

The \code{param} matrix stores items parameters by column. If a
column has more rows than are required to fully specify a model
then the extra rows are ignored.  The order of the items in
\code{spec} and order of columns in \code{param} are assumed to
match. All items should have the same number of latent dimensions.
Loadings on latent dimensions are given in the first few rows and
can be named by setting rownames. Item names are assigned by
\code{param} colnames.

Currently only a multivariate normal distribution is available,
parameterized by the \code{mean} and \code{cov}. If \code{mean} and
\code{cov} are not specified then a standard normal distribution is
assumed. The quadrature consists of equally spaced points. For
example, \code{qwidth=2} and \code{qpoints=5} would produce points
-2, -1, 0, 1, and 2. The quadrature specification is part of the
group and not passed as extra arguments for the sake of
consistency. As currently implemented, OpenMx uses EAP scores to
estimate latent distribution parameters. By default, the exact same
EAP scores should be produced by \link{EAPscores}.
}

\examples{
# 1 dimensional items
i1 <- rpf.drm()
i1.p <- rpf.rparam(i1)
i2 <- rpf.nrm(outcomes=3)
i2.p <- rpf.rparam(i2)
rpf.sample(5, list(i1,i2), list(i1.p, i2.p))
}
\seealso{
\code{\link{sample}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{Class rpf.1dim.gpcmp}
\alias{Class rpf.1dim.gpcmp}
\alias{rpf.1dim.gpcmp-class}
\title{Unidimensional generalized partial credit monotonic polynomial.}
\description{
Unidimensional generalized partial credit monotonic polynomial.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcm.R
\name{rpf.mcm}
\alias{rpf.mcm}
\title{Create a multiple-choice response model}
\usage{
rpf.mcm(outcomes = 2, numChoices = 5, factors = 1)
}
\arguments{
\item{outcomes}{the number of possible outcomes}

\item{numChoices}{the number of choices available}

\item{factors}{the number of factors}
}
\value{
an item model
}
\description{
\lifecycle{experimental}
}
\details{
This function instantiates a multiple-choice response
model.
}
\seealso{
Other response model: 
\code{\link{rpf.drm}()},
\code{\link{rpf.gpcmp}()},
\code{\link{rpf.grmp}()},
\code{\link{rpf.grm}()},
\code{\link{rpf.lmp}()},
\code{\link{rpf.nrm}()}
}
\author{
Jonathan Weeks <weeksjp@gmail.com>
}
\concept{response model}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnose.R
\name{crosstabTest}
\alias{crosstabTest}
\title{Monte-Carlo test for cross-tabulation tables}
\usage{
crosstabTest(ob, ex, trials)
}
\arguments{
\item{ob}{observed table}

\item{ex}{expected table}

\item{trials}{number of Monte-Carlo trials}
}
\description{
\lifecycle{experimental}
This is for developers.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmp.R
\name{rpf.grmp}
\alias{rpf.grmp}
\title{Create monotonic polynomial graded response (GR-MP) model}
\usage{
rpf.grmp(outcomes = 2, q = 0, multidimensional = FALSE)
}
\arguments{
\item{outcomes}{The number of possible response categories. When equal to 2, the model reduces
to the logistic function of a monotonic polynomial (LMP).}

\item{q}{a non-negative integer that controls the order of the
polynomial (2q+1) with a default of q=0 (1st order polynomial = graded response model).}

\item{multidimensional}{whether to use a multidimensional model.
Defaults to \code{FALSE}. The multidimensional version is not yet
available.}
}
\value{
an item model
}
\description{
The GR-MP model replaces the linear predictor of the graded
response model (Samejima, 1969, 1972) with a monotonic polynomial (Falk, conditionally accepted).
}
\details{
Given its relationship to the graded response model, the GR-MP
is constructed in an analogous way:

\deqn{\mathrm P(\mathrm{pick}=0|\lambda,\alpha,\tau,\theta) = 1- \frac{1}{1+\exp(-(\xi_1 + m(\theta;\lambda,\mathbf{\alpha},\mathbf{\tau})))}
}{P(pick=0|a,c,theta) = 1-1/(1+exp(-(xi_1 + m(theta;lambda,alpha,tau)))}
\deqn{\mathrm P(\mathrm{pick}=k|\lambda,\alpha,\tau,\theta) = \frac{1}{1+\exp(-(\xi_k + m(\theta;\lambda,\mathbf{\alpha},\mathbf{\tau})))} - \frac{1}{1+\exp(-(\xi_{k+1} + m(\theta,\lambda,\mathbf{\alpha},\mathbf{\tau}))}
}{P(pick=k|a,c,theta) = 1/(1+exp(-(xi_k + m(theta;lambda,alpha,tau))) - 1/(1+exp(-(xi_(k+1) + m(theta;lambda,alpha,tau))))}
\deqn{\mathrm P(\mathrm{pick}=K|\lambda,\alpha,\tau,\theta) = \frac{1}{1+\exp(-(\xi_K + m(\theta;\lambda,\mathbf{\alpha},\mathbf{\tau}))}
}{P(pick=K|\lambda,\alpha,\tau,theta) = 1/(1+exp(-(xi_K + m(theta;lambda,alpha,tau))))}

The order of the polynomial is always odd and is controlled by
the user specified non-negative integer, q. The model contains
1+(outcomtes-1)+2*q parameters and are used as input to the \code{\link{rpf.prob}}
or \code{\link{rpf.dTheta}} functions in the following order:
\eqn{\lambda}{lambda} - slope of the item model when q=0,
\eqn{\xi}{xi} - a (outcomes-1)-length vector of intercept parameters,
\eqn{\alpha}{alpha} and \eqn{\tau}{tau} - two parameters that control bends in
the polynomial. These latter parameters are repeated in the same order for
models with q>0.  For example, a q=2 polynomial with 3 categories will have an item
parameter vector of: \eqn{\lambda, \xi_1, \xi_2, \alpha_1, \tau_1, \alpha_2, \tau_2}{
lambda, xi1, xi2, alpha1, tau1, alpha2, tau2}.

As with other monotonic polynomial-based item models
(e.g., \code{\link{rpf.lmp}}), the polynomial looks like the
following:

\deqn{m(\theta;\lambda,\alpha,\tau) = b_1\theta + b_2\theta^2 + \dots + b_{2q+1}\theta^{2q+1}
}{m(theta) = b_1*theta + b_2*theta^2 + \dots + b_(2q+1)*theta^{2q+1}}

However, the coefficients, b, are not directly estimated, but are a function of the
item parameters, and the parameterization of the GR-MP is different than
that currently appearing for the logistic function of a monotonic
polynomial (LMP; \code{\link{rpf.lmp}}) and monotonic polynomial generalized partial credit
(GPC-MP; \code{\link{rpf.gpcmp}}) models. In particular, the polynomial is
parameterized such that boundary descrimination functions for the GR-MP will
be all monotonically increasing or decreasing for any given item. This allows
the possibility of items that load either negatively or positively on the latent
trait, as is common with reverse-worded items in non-cognitive tests (e.g., personality).

The derivative \eqn{m'(\theta;\lambda,\alpha,\tau)}{m'(theta;lambda,alpha,tau)} is
parameterized in the following way:

\deqn{ m'(\theta;\lambda,\alpha,\tau) = \left\{\begin{array}{ll}\lambda \prod_{u=1}^q(1-2\alpha_{u}\theta + (\alpha_{u}^2 + \exp(\tau_{u}))\theta^2) &  \mbox{if } q > 0 \\
 \lambda & \mbox{if } q = 0\end{array} \right.}{m'(theta) = m'(theta;lambda,alpha,tau) = lambda \prod_{u=1}^q (1-2*alpha_u*theta + (alpha_u^2 + exp(tau_u))*theta^2) (if q > 0) \\
 lambda (if q = 0)}

Note that the only difference between the GR-MP and these other models
is that \eqn{\lambda}{lambda} is not re-parameterized and may take on
negative values. When \eqn{\lambda}{lambda} is negative, it is analogous
to having a negative loading or a monotonically decreasing function.
}
\examples{
spec <- rpf.grmp(5,2) # 5-category, 3rd order polynomial
theta<-seq(-3,3,.1)
p<-rpf.prob(spec, c(2.77,2,1,0,-1,.89,-8.7,-.74,-8.99),theta)
}
\references{
Falk, C. F. (conditionally accepted). The monotonic polynomial
graded response model: Implementation and a comparative study. \emph{Applied Psychological
Measurement}.

Samejima, F. (1969). Estimation of latent ability using a response pattern of graded scores. \emph{Psychometric Monographs}, 17.

Samejima, F. (1972). A general model of free-response data. \emph{Psychometric Monographs}, 18.
}
\seealso{
Other response model: 
\code{\link{rpf.drm}()},
\code{\link{rpf.gpcmp}()},
\code{\link{rpf.grm}()},
\code{\link{rpf.lmp}()},
\code{\link{rpf.mcm}()},
\code{\link{rpf.nrm}()}
}
\concept{response model}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{Class rpf.1dim}
\alias{Class rpf.1dim}
\alias{rpf.1dim-class}
\title{The base class for 1 dimensional response probability functions.}
\description{
The base class for 1 dimensional response probability functions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{methods}
\name{rpf.logprob}
\alias{rpf.logprob}
\alias{rpf.logprob,rpf.1dim,numeric,numeric-method}
\alias{rpf.logprob,rpf.1dim,numeric,matrix-method}
\alias{rpf.logprob,rpf.mdim,numeric,matrix-method}
\alias{rpf.logprob,rpf.mdim,numeric,numeric-method}
\alias{rpf.logprob,rpf.mdim,numeric,NULL-method}
\alias{_rpf_logprob}
\title{Map an item model, item parameters, and person trait score into a
probability vector}
\usage{
rpf.logprob(m, param, theta)
}
\arguments{
\item{m}{an item model}

\item{param}{item parameters}

\item{theta}{the trait score(s)}
}
\value{
a vector of probabilities. For dichotomous items,
probabilities are returned in the order incorrect, correct.
Although redundent, both incorrect and correct probabilities are
returned in the dichotomous case for API consistency with
polytomous item models.
}
\description{
Note that in general, exp(rpf.logprob(..)) != rpf.prob(..) because
the range of logits is much wider than the range of probabilities
due to limitations of floating point numerical precision.
}
\examples{
i1 <- rpf.drm()
i1.p <- rpf.rparam(i1)
rpf.logprob(i1, c(i1.p), -1)   # low trait score
rpf.logprob(i1, c(i1.p), c(0,1))    # average and high trait score
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{Class rpf.1dim.grm}
\alias{Class rpf.1dim.grm}
\alias{rpf.1dim.grm-class}
\title{The unidimensional graded response item model.}
\description{
The unidimensional graded response item model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataframe.R
\name{compressDataFrame}
\alias{compressDataFrame}
\title{Compress a data frame into unique rows and frequencies}
\usage{
compressDataFrame(tabdata, freqColName = "freq", .asNumeric = FALSE)
}
\arguments{
\item{tabdata}{An object of class \code{data.frame}}

\item{freqColName}{Column name to contain the frequencies}

\item{.asNumeric}{logical. Whether to cast the frequencies to the numeric type}
}
\value{
Returns a compressed data frame
}
\description{
Compress a data frame into unique rows and frequency counts.
}
\examples{
df <- as.data.frame(matrix(c(sample.int(2, 30, replace=TRUE)), 10, 3))
compressDataFrame(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataframe.R
\name{orderCompletely}
\alias{orderCompletely}
\title{Order a data.frame by missingness and all columns}
\usage{
orderCompletely(observed)
}
\arguments{
\item{observed}{a data.frame holding ordered factors in every column}
}
\value{
the sorted order of the rows
}
\description{
Completely order all rows in a data.frame.
}
\examples{
df <- as.data.frame(matrix(c(sample.int(2, 30, replace=TRUE)), 10, 3))
mask <- matrix(c(sample.int(3, 30, replace=TRUE)), 10, 3) == 1
df[mask] <- NA
df[orderCompletely(df),]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{rpf.paramInfo}
\alias{rpf.paramInfo}
\alias{rpf.paramInfo,rpf.base-method}
\alias{rpf_paramInfo_wrapper}
\title{Retrieve a description of the given parameter}
\usage{
rpf.paramInfo(m, num = NULL)
}
\arguments{
\item{m}{item model}

\item{num}{vector of parameters (defaults to all)}
}
\value{
a list containing the type, upper bound, and lower bound
}
\description{
Retrieve a description of the given parameter
}
\examples{
rpf.paramInfo(rpf.drm())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{toFactorLoading}
\alias{toFactorLoading}
\title{Convert response function slopes to factor loadings}
\usage{
toFactorLoading(slope, ogive = rpf.ogive)
}
\arguments{
\item{slope}{a matrix with items in the columns and slopes in the rows}

\item{ogive}{the ogive constant (default \link{rpf.ogive})}
}
\value{
a factor loading matrix with items in the rows and factors in the columns
}
\description{
All slopes are divided by the ogive constant. Then the following
transformation is applied to the slope matrix,
}
\details{
\deqn{\frac{\mathrm{slope}}{\left[ 1 + \mathrm{rowSums}(\mathrm{slope}^2) \right]^\frac{1}{2}}}
}
\seealso{
Other factor model equivalence: 
\code{\link{fromFactorLoading}()},
\code{\link{fromFactorThreshold}()},
\code{\link{toFactorThreshold}()}
}
\concept{factor model equivalence}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{Class rpf.mdim.grm}
\alias{Class rpf.mdim.grm}
\alias{rpf.mdim.grm-class}
\title{The multidimensional graded response item model.}
\description{
The multidimensional graded response item model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{package}
\name{An introduction}
\alias{An introduction}
\alias{rpf-package}
\title{rpf - Response Probability Functions}
\description{
Factor out logic and math common
to Item Factor Analysis fitting, diagnostics, and analysis.  It is
envisioned as core support code suitable for more specialized IFA
packages to build upon.
}
\details{
This package provides optimized, low-level functions to map
parameters to response probabilities for dichotomous (1PL, 2PL and
3PL) \code{\link{rpf.drm}} and polytomous (graded response
\code{\link{rpf.grm}}, partial credit/generalized partial credit
(via the nominal model), and nominal \code{\link{rpf.nrm}} items.

Item model parameters are passed around as a numeric vector. A 1D
matrix is also acceptable. Regardless of model, parameters are
always ordered as follows: discrimination/slope ("a"),
difficulty/intercept ("b"), and pseudo guessing/upper-bound ("g"/"u"). If
person ability ranges from negative to positive then
probabilities are output from incorrect to correct. That is, a low
ability person (e.g., ability = -2) will be more likely to get an
item incorrect than correct. For example, a dichotomous model that
returns [.25, .75] indicates a probability of .25 for incorrect
and .75 for correct.  A polytomous model will have the most
incorrect probability at index 1 and the most correct probability
at the maximum index.

All models are always in the logistic metric. To obtain normal
ogive discrimination parameters, divide slope parameters by
\code{\link{rpf.ogive}}. Item models are estimated in
slope-intercept form. Input/output matrices arranged in the way
most convenient for low-level processing in C. The maximum
absolute logit is 35 because f(x) := 1-exp(x) loses accuracy around f(-35)
and equals 1 at f(-38) due to the limited accuracy of double
precision floating point.

This package could also accrete functions to support plotting (but
not the actual plot functions).
}
\references{
Pritikin, J. N., Hunter, M. D., & Boker, S. M.
(2015). Modular open-source software for Item Factor Analysis.
Educational and Psychological Measurement, 75(3), 458-474

Thissen, D. and Steinberg, L. (1986). A taxonomy of
item response models. \emph{Psychometrika 51}(4), 567-577.
}
\seealso{
See \code{\link{rpf.rparam}} to create item parameters.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmp.R
\name{rpf.lmp}
\alias{rpf.lmp}
\title{Create logistic function of a monotonic polynomial (LMP) model}
\usage{
rpf.lmp(q = 0, multidimensional = FALSE)
}
\arguments{
\item{q}{a non-negative integer that controls the order of the
polynomial (2q+1) with a default of q=0 (1st order polynomial = 2PL).}

\item{multidimensional}{whether to use a multidimensional model.
Defaults to \code{FALSE}. The multidimensional version is not yet
available.}
}
\value{
an item model
}
\description{
This model is a dichotomous response model originally proposed by
Liang (2007) and is implemented using the parameterization by
Falk & Cai (2016).
}
\details{
The LMP model replaces the linear predictor part of the
two-parameter logistic function with a monotonic polynomial,
\eqn{m(\theta,\omega,\xi,\mathbf{\alpha},\mathbf{\tau})}{m(theta; omega, alpha, tau)},

\deqn{\mathrm P(\mathrm{pick}=1|\omega,\xi,\mathbf{\alpha},\mathbf{\tau},\theta)
= \frac{1}{1+\exp(-(\xi + m(\theta;\omega,\mathbf{\alpha},\mathbf{\tau})))}
}{P(pick=1|omega,xi,alpha,tau,th) = 1/(1+exp(-(xi + m(theta;omega,alpha,tau))))}

where \eqn{\mathbf{\alpha}}{alpha} and \eqn{\mathbf{\tau}}{tau} are vectors
of length q.

The order of the polynomial is always odd and is controlled by
the user specified non-negative integer, q. The model contains
2+2*q parameters and are used in conjunction with the \code{\link{rpf.prob}}
or \code{\link{rpf.dTheta}} function in the following order:
\eqn{\omega}{omega} - the natural log of the slope of the item model when q=0,
\eqn{\xi}{xi} - the intercept,
\eqn{\alpha}{alpha} and \eqn{\tau}{tau} - two parameters that control bends in
the polynomial. These latter parameters are repeated in the same order for
models with q>0. For example, a q=2 polynomial with have an item
parameter vector of: \eqn{\omega, \xi, \alpha_1, \tau_1, \alpha_2, \tau_2}{
omega, xi, alpha1, tau1, alpha2, tau2}.

In general, the polynomial looks like the following:

\deqn{m(\theta;\omega,\alpha,\tau) = b_1\theta + b_2\theta^2 + \dots + b_{2q+1}\theta^{2q+1}
}{m(theta;omega,alpha,tau) = b_1*theta + b_2*theta^2 + \dots + b_(2q+1)*theta^{2q+1}}

However, the coefficients, b, are not directly estimated, but are a function of the
item parameters. In particular, the derivative \eqn{m'(\theta;\omega,\alpha,\tau)}{m'(theta;omega,alpha,tau)} is
parameterized in the following way:

\deqn{m'(\theta;\omega,\alpha,\tau) = \left\{\begin{array}{ll}\exp(\omega) \prod_{u=1}^q(1-2\alpha_{u}\theta + (\alpha_{u}^2 + \exp(\tau_{u}))\theta^2) &  \mbox{if } q > 0 \\
 \exp(\omega) & \mbox{if } q = 0\end{array} \right.}{m'(theta) = m'(theta;omega,alpha,tau) = exp(omega) \prod_{u=1}^q (1-2*alpha_u*theta + (alpha_u^2 + exp(tau_u))*theta^2) (if q > 0) \\
 exp(omega) (if q = 0)}


See Falk & Cai (2016) for more details as to how the polynomial is constructed.
At the lowest order polynomial (q=0) the model reduces to the
two-parameter logistic (2PL) model. However, parameterization of the
slope parameter, \eqn{\omega}{omega}, is currently different than
the 2PL (i.e., slope = exp(\eqn{\omega}{omega})). This parameterization
ensures that the response function is always monotonically increasing
without requiring constrained optimization.

For an alternative parameterization that releases constraints
on \eqn{\omega}{omega}, allowing for monotonically decreasing functions,
see \code{\link{rpf.grmp}}. And for polytomous items, see both
\code{\link{rpf.grmp}} and \code{\link{rpf.gpcmp}}.
}
\examples{
spec <- rpf.lmp(1) # 3rd order polynomial
theta<-seq(-3,3,.1)
p<-rpf.prob(spec, c(-.11,.37,.24,-.21),theta)

spec <- rpf.lmp(2) # 5th order polynomial
p<-rpf.prob(spec, c(.69,.71,-.5,-8.48,.52,-3.32),theta)
}
\references{
Falk, C. F., & Cai, L. (2016). Maximum marginal likelihood
estimation of a monotonic polynomial generalized partial credit model with
applications to multiple group analysis. \emph{Psychometrika, 81}, 434-460.
\doi{10.1007/s11336-014-9428-7}

Liang (2007). \emph{A semi-parametric approach to estimating item response
functions}. Unpublished doctoral dissertation, Department of Psychology,
The Ohio State University.
}
\seealso{
Other response model: 
\code{\link{rpf.drm}()},
\code{\link{rpf.gpcmp}()},
\code{\link{rpf.grmp}()},
\code{\link{rpf.grm}()},
\code{\link{rpf.mcm}()},
\code{\link{rpf.nrm}()}
}
\concept{response model}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{class}
\name{Class rpf.mdim.drm}
\alias{Class rpf.mdim.drm}
\alias{rpf.mdim.drm-class}
\title{Multidimensional dichotomous item models (M1PL, M2PL, and M3PL).}
\description{
Multidimensional dichotomous item models (M1PL, M2PL, and M3PL).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{rpf.dLL}
\alias{rpf.dLL}
\alias{rpf.dLL,rpf.base,numeric,numeric,numeric-method}
\alias{rpf.dLL,rpf.base,numeric,NULL,numeric-method}
\alias{_rpf_dLL}
\title{Item parameter derivatives}
\usage{
rpf.dLL(m, param, where, weight)
}
\arguments{
\item{m}{item model}

\item{param}{item parameters}

\item{where}{location in the latent space}

\item{weight}{per outcome weights (typically derived by observation)}
}
\value{
first and second order partial derivatives of the log
likelihood evaluated at \code{where}. For p parameters, the first
p values are the first derivative and the next p(p+1)/2 columns
are the lower triangle of the second derivative.
}
\description{
Evaluate the partial derivatives of the log likelihood with
respect to each parameter at \code{where} with \code{weight}.
}
\details{
It is not easy to write an example for this function. To evaluate
the derivative, you need to sum the derivatives across a
quadrature. You also need response outcome weights at each
quadrature point. It is not anticipated that this function will be
often used in R code. It's mainly to expose a C-level function for
occasional debugging.
}
\seealso{
The numDeriv package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/openmx.R
\name{as.IFAgroup}
\alias{as.IFAgroup}
\title{Convert an OpenMx MxModel object into an IFA group}
\usage{
as.IFAgroup(
  mxModel,
  data = NULL,
  container = NULL,
  ...,
  minItemsPerScore = NULL
)
}
\arguments{
\item{mxModel}{MxModel object}

\item{data}{observed data (otherwise the data will be taken from the mxModel)}

\item{container}{an MxModel in which to search for the latent distribution matrices}

\item{...}{Not used.  Forces remaining arguments to be specified by name.}

\item{minItemsPerScore}{minimum number of items required to compute a score (also see description)}
}
\value{
a groups with item parameters and latent distribution
}
\description{
When \dQuote{minItemsPerScore} is passed, EAP scores will be computed
from the data and stored. Scores are required for some diagnostic
tests. See discussion of \dQuote{minItemsPerScore} in \link{EAPscores}.
}
\section{Format of a group}{


A model, or group within a model, is represented as a named list.
\describe{
\item{spec}{list of response model objects}
\item{param}{numeric matrix of item parameters}
\item{free}{logical matrix of indicating which parameters are free (TRUE) or fixed (FALSE)}
\item{mean}{numeric vector giving the mean of the latent distribution}
\item{cov}{numeric matrix giving the covariance of the latent distribution}
\item{data}{data.frame containing observed item responses, and optionally,
weights and frequencies}
\item{score}{factors scores with response patterns in rows}
\item{weightColumn}{name of the data column containing the numeric row weights (optional)}
\item{freqColumn}{name of the data column containing the integral row frequencies (optional)}
\item{qwidth}{width of the quadrature expressed in Z units}
\item{qpoints}{number of quadrature points}
\item{minItemsPerScore}{minimum number of non-missing items when estimating factor scores}
}

The \code{param} matrix stores items parameters by column. If a
column has more rows than are required to fully specify a model
then the extra rows are ignored.  The order of the items in
\code{spec} and order of columns in \code{param} are assumed to
match. All items should have the same number of latent dimensions.
Loadings on latent dimensions are given in the first few rows and
can be named by setting rownames. Item names are assigned by
\code{param} colnames.

Currently only a multivariate normal distribution is available,
parameterized by the \code{mean} and \code{cov}. If \code{mean} and
\code{cov} are not specified then a standard normal distribution is
assumed. The quadrature consists of equally spaced points. For
example, \code{qwidth=2} and \code{qpoints=5} would produce points
-2, -1, 0, 1, and 2. The quadrature specification is part of the
group and not passed as extra arguments for the sake of
consistency. As currently implemented, OpenMx uses EAP scores to
estimate latent distribution parameters. By default, the exact same
EAP scores should be produced by \link{EAPscores}.
}

\seealso{
\href{https://cran.r-project.org/package=ifaTools}{ifaTools}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\docType{methods}
\name{rpf.prob}
\alias{rpf.prob}
\alias{rpf.prob,rpf.1dim,numeric,numeric-method}
\alias{rpf.prob,rpf.mdim,numeric,NULL-method}
\alias{rpf.prob,rpf.mdim,numeric,numeric-method}
\alias{rpf.prob,rpf.mdim,numeric,matrix-method}
\alias{rpf.prob,rpf.base,data.frame,numeric-method}
\alias{rpf.prob,rpf.base,matrix,numeric-method}
\alias{rpf.prob,rpf.base,matrix,matrix-method}
\alias{rpf.prob,rpf.1dim,numeric,matrix-method}
\alias{rpf.prob,rpf.1dim.grm,numeric,numeric-method}
\alias{rpf.prob,rpf.mdim.grm,numeric,numeric-method}
\alias{rpf.prob,rpf.mdim.nrm,numeric,matrix-method}
\alias{rpf.prob,rpf.mdim.mcm,numeric,matrix-method}
\alias{rpf.prob,rpf.mdim.grm,numeric,matrix-method}
\alias{_rpf_prob}
\title{Map an item model, item parameters, and person trait score into a
probability vector}
\usage{
rpf.prob(m, param, theta)
}
\arguments{
\item{m}{an item model}

\item{param}{item parameters}

\item{theta}{the trait score(s)}
}
\value{
a vector of probabilities. For dichotomous items,
probabilities are returned in the order incorrect, correct.
Although redundent, both incorrect and correct probabilities are
returned in the dichotomous case for API consistency with
polytomous item models.
}
\description{
This function is known by many names in the literature. When
plotted against latent trait, it is often called a traceline, item
characteristic curve, or item response function. Sometimes the word
'category' or 'outcome' is used in place of 'item'. For example,
'item response function' might become 'category response
function'. All these terms refer to the same thing.
}
\examples{
i1 <- rpf.drm()
i1.p <- rpf.rparam(i1)
rpf.prob(i1, c(i1.p), -1)   # low trait score
rpf.prob(i1, c(i1.p), c(0,1))    # average and high trait score
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{rpf.id_of}
\alias{rpf.id_of}
\title{Convert an rpf item model name to an ID}
\usage{
rpf.id_of(name)
}
\arguments{
\item{name}{name of the item model (string)}
}
\value{
the integer ID assigned to the given model
}
\description{
This is an internal function and should not be used.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnose.R
\name{rpf.1dim.fit}
\alias{rpf.1dim.fit}
\title{Calculate item and person Rasch fit statistics}
\usage{
rpf.1dim.fit(
  spec,
  params,
  responses,
  scores,
  margin,
  group = NULL,
  wh.exact = TRUE
)
}
\arguments{
\item{spec}{list of item response models \lifecycle{deprecated}}

\item{params}{matrix of item parameters, 1 per column \lifecycle{deprecated}}

\item{responses}{persons in rows and items in columns \lifecycle{deprecated}}

\item{scores}{model derived person scores \lifecycle{deprecated}}

\item{margin}{for people 1, for items 2}

\item{group}{spec, params, data, and scores can be provided in a list instead of as arguments}

\item{wh.exact}{whether to use the exact Wilson-Hilferty transformation}
}
\description{
Note: These statistics are only appropriate if all discrimination
parameters are fixed equal and items are conditionally independent
(see \code{\link{ChenThissen1997}}).  A best effort is made to
cope with missing data.
}
\details{
Exact distributional properties of these statistics are unknown
(Masters & Wright, 1997, p. 112).  For details on the calculation,
refer to Wright & Masters (1982, p. 100).

The Wilson-Hilferty transformation is biased for less than 25 items.
Consider wh.exact=FALSE for less than 25 items.
}
\section{Format of a group}{


A model, or group within a model, is represented as a named list.
\describe{
\item{spec}{list of response model objects}
\item{param}{numeric matrix of item parameters}
\item{free}{logical matrix of indicating which parameters are free (TRUE) or fixed (FALSE)}
\item{mean}{numeric vector giving the mean of the latent distribution}
\item{cov}{numeric matrix giving the covariance of the latent distribution}
\item{data}{data.frame containing observed item responses, and optionally,
weights and frequencies}
\item{score}{factors scores with response patterns in rows}
\item{weightColumn}{name of the data column containing the numeric row weights (optional)}
\item{freqColumn}{name of the data column containing the integral row frequencies (optional)}
\item{qwidth}{width of the quadrature expressed in Z units}
\item{qpoints}{number of quadrature points}
\item{minItemsPerScore}{minimum number of non-missing items when estimating factor scores}
}

The \code{param} matrix stores items parameters by column. If a
column has more rows than are required to fully specify a model
then the extra rows are ignored.  The order of the items in
\code{spec} and order of columns in \code{param} are assumed to
match. All items should have the same number of latent dimensions.
Loadings on latent dimensions are given in the first few rows and
can be named by setting rownames. Item names are assigned by
\code{param} colnames.

Currently only a multivariate normal distribution is available,
parameterized by the \code{mean} and \code{cov}. If \code{mean} and
\code{cov} are not specified then a standard normal distribution is
assumed. The quadrature consists of equally spaced points. For
example, \code{qwidth=2} and \code{qpoints=5} would produce points
-2, -1, 0, 1, and 2. The quadrature specification is part of the
group and not passed as extra arguments for the sake of
consistency. As currently implemented, OpenMx uses EAP scores to
estimate latent distribution parameters. By default, the exact same
EAP scores should be produced by \link{EAPscores}.



A model, or group within a model, is represented as a named list.
\describe{
\item{spec}{list of response model objects}
\item{param}{numeric matrix of item parameters}
\item{free}{logical matrix of indicating which parameters are free (TRUE) or fixed (FALSE)}
\item{mean}{numeric vector giving the mean of the latent distribution}
\item{cov}{numeric matrix giving the covariance of the latent distribution}
\item{data}{data.frame containing observed item responses, and optionally,
weights and frequencies}
\item{score}{factors scores with response patterns in rows}
\item{weightColumn}{name of the data column containing the numeric row weights (optional)}
\item{freqColumn}{name of the data column containing the integral row frequencies (optional)}
\item{qwidth}{width of the quadrature expressed in Z units}
\item{qpoints}{number of quadrature points}
\item{minItemsPerScore}{minimum number of non-missing items when estimating factor scores}
}

The \code{param} matrix stores items parameters by column. If a
column has more rows than are required to fully specify a model
then the extra rows are ignored.  The order of the items in
\code{spec} and order of columns in \code{param} are assumed to
match. All items should have the same number of latent dimensions.
Loadings on latent dimensions are given in the first few rows and
can be named by setting rownames. Item names are assigned by
\code{param} colnames.

Currently only a multivariate normal distribution is available,
parameterized by the \code{mean} and \code{cov}. If \code{mean} and
\code{cov} are not specified then a standard normal distribution is
assumed. The quadrature consists of equally spaced points. For
example, \code{qwidth=2} and \code{qpoints=5} would produce points
-2, -1, 0, 1, and 2. The quadrature specification is part of the
group and not passed as extra arguments for the sake of
consistency. As currently implemented, OpenMx uses EAP scores to
estimate latent distribution parameters. By default, the exact same
EAP scores should be produced by \link{EAPscores}.
}

\examples{
data(kct)
responses <- kct.people[,paste("V",2:19, sep="")]
rownames(responses) <- kct.people$NAME
colnames(responses) <- kct.items$NAME
scores <- kct.people$MEASURE
params <- cbind(1, kct.items$MEASURE, logit(0), logit(1))
rownames(params) <- kct.items$NAME
items<-list()
items[1:18] <- rpf.drm()
params[,2] <- -params[,2]
rpf.1dim.fit(items, t(params), responses, scores, 2, wh.exact=TRUE)
}
\references{
Masters, G. N. & Wright, B. D. (1997). The Partial
Credit Model. In W. van der Linden & R. K. Kambleton (Eds.),
\emph{Handbook of modern item response theory}
(pp. 101-121). Springer.

Wilson, E. B., & Hilferty, M. M. (1931). The distribution of
chi-square. \emph{Proceedings of the National Academy of Sciences of the
United States of America,} 17, 684-688.

Wright, B. D. & Masters, G. N. (1982). \emph{Rating Scale
Analysis.} Chicago: Mesa Press.
}
\seealso{
Other diagnostic: 
\code{\link{ChenThissen1997}()},
\code{\link{SitemFit1}()},
\code{\link{SitemFit}()},
\code{\link{multinomialFit}()},
\code{\link{sumScoreEAPTest}()}
}
\concept{diagnostic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnose.R
\name{ptw2011.gof.test}
\alias{ptw2011.gof.test}
\title{Compute the P value that the observed and expected tables come from the same distribution}
\usage{
ptw2011.gof.test(observed, expected)
}
\arguments{
\item{observed}{observed matrix}

\item{expected}{expected matrix}
}
\value{
The P value indicating whether the two tables come from
the same distribution. For example, a significant result (P <
alpha level) rejects the hypothesis that the two matrices are from
the same distribution.
}
\description{
\lifecycle{experimental}
This test is an alternative to Pearson's X^2
goodness-of-fit test.  In contrast to Pearson's X^2, no ad hoc cell
collapsing is needed to avoid an inflated false positive rate
in situations of sparse cell frequences.
The statistic rapidly converges to the Monte-Carlo estimate
as the number of draws increases.
}
\examples{
draws <- 17
observed <- matrix(c(.294, .176, .118, .411), nrow=2) * draws
expected <- matrix(c(.235, .235, .176, .353), nrow=2) * draws
ptw2011.gof.test(observed, expected)  # not signficiant
}
\references{
Perkins, W., Tygert, M., & Ward, R. (2011). Computing
the confidence levels for a root-mean-square test of
goodness-of-fit. \emph{Applied Mathematics and Computations,
217}(22), 9072-9084.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnose.R
\docType{methods}
\name{rpf.1dim.residual}
\alias{rpf.1dim.residual}
\title{Calculate residuals}
\usage{
rpf.1dim.residual(spec, params, responses, scores)
}
\arguments{
\item{spec}{list of item models}

\item{params}{data frame of item parameters, 1 per row}

\item{responses}{persons in rows and items in columns}

\item{scores}{model derived person scores}
}
\value{
residuals
}
\description{
Calculate residuals
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{rpf.info}
\alias{rpf.info}
\title{Map an item model, item parameters, and person trait score into a
information vector}
\usage{
rpf.info(ii, ii.p, where, basis = 1)
}
\arguments{
\item{ii}{an item model}

\item{ii.p}{item parameters}

\item{where}{the location in the latent distribution}

\item{basis}{if more than 1 factor, a positive basis vector}
}
\value{
Fisher information
}
\description{
Map an item model, item parameters, and person trait score into a
information vector
}
\examples{
i1 <- rpf.drm()
i1.p <- c(.6,1,.1,.95)
theta <- seq(0,3,.05)
plot(theta, rpf.info(i1, i1.p, t(theta)), type="l")
}
\references{
Dodd, B. G., De Ayala, R. J. & Koch,
W. R. (1995). Computerized adaptive testing with polytomous items.
\emph{Applied psychological measurement 19}(1), 5-22.
}
